@@ -1,7 +1,7 @@
 // -------------------------------------------------------------------------
 // AAI
 //
-// A skirmish AI for the TA Spring engine.
+// A skirmish AI for the Spring engine.
 // Copyright Alexander Seizinger
 //
 // Released under GPL license: see LICENSE.html for more information.
@@ -15,7 +15,7 @@ AAIConfig::AAIConfig(void)
 	SIDES = 2;
 	SECTOR_SIZE = 100.0;
 	MIN_ENERGY = 18;  // min energy make value to be considered beeing a power plant
-	MAX_UNITS = 10000;
+	MAX_UNITS = 30000;
 	MAX_SCOUTS = 4;
 	MAX_SECTOR_IMPORTANCE = 6;
 	MAX_XROW = 8;
@@ -114,26 +114,29 @@ AAIConfig::~AAIConfig(void)
 
 void AAIConfig::LoadConfig(AAI *ai)
 {
-	char filename[500];
+	// this size equals the one used in "AIAICallback::GetValue(AIVAL_LOCATE_FILE_..."
+	char filename[2048];
 	char buffer[500];
 
+	MAX_UNITS = ai->cb->GetMaxUnits();
+
 	FILE* file = NULL;
 
-	STRCPY(buffer, MAIN_PATH);
-	STRCAT(buffer, MOD_CFG_PATH);
+	STRCPY_T(buffer, sizeof(buffer), MAIN_PATH);
+	STRCAT_T(buffer, sizeof(buffer), MOD_CFG_PATH);
 	const std::string modHumanName = MakeFileSystemCompatible(ai->cb->GetModHumanName());
-	STRCAT(buffer, modHumanName.c_str());
-	STRCAT(buffer, ".cfg");
-	STRCPY(filename, buffer);
+	STRCAT_T(buffer, sizeof(buffer), modHumanName.c_str());
+	STRCAT_T(buffer, sizeof(buffer), ".cfg");
+	STRCPY_T(filename, sizeof(filename), buffer);
 	ai->cb->GetValue(AIVAL_LOCATE_FILE_R, filename);
 	file = fopen(filename, "r");
 	if (file == NULL) {
 		fprintf(ai->file, "Mod config file %s not found\n", filename);
 		fprintf(ai->file, "Now trying with legacy mod config file name ...\n");
-		STRCPY(buffer, MAIN_PATH);
-		STRCAT(buffer, MOD_CFG_PATH);
+		STRCPY_T(buffer, sizeof(buffer), MAIN_PATH);
+		STRCAT_T(buffer, sizeof(buffer), MOD_CFG_PATH);
 		const std::string modName = MakeFileSystemCompatible(ai->cb->GetModName());
-		STRCAT(buffer, modName.c_str());
+		STRCAT_T(buffer, sizeof(buffer), modName.c_str());
 		ReplaceExtension(buffer, filename, sizeof(filename), ".cfg");
 		ai->cb->GetValue(AIVAL_LOCATE_FILE_R, filename);
 		file = fopen(filename, "r");
@@ -141,12 +144,12 @@ void AAIConfig::LoadConfig(AAI *ai)
 	if (file == NULL) {
 		fprintf(ai->file, "Mod config file %s not found\n", filename);
 		fprintf(ai->file, "Now trying with version independent mod config file name ...\n");
-		STRCPY(buffer, MAIN_PATH);
-		STRCAT(buffer, MOD_CFG_PATH);
+		STRCPY_T(buffer, sizeof(buffer), MAIN_PATH);
+		STRCAT_T(buffer, sizeof(buffer), MOD_CFG_PATH);
 		const std::string modShortName = MakeFileSystemCompatible(ai->cb->GetModShortName());
-		STRCAT(buffer, modShortName.c_str());
-		STRCAT(buffer, ".cfg");
-		STRCPY(filename, buffer);
+		STRCAT_T(buffer, sizeof(buffer), modShortName.c_str());
+		STRCAT_T(buffer, sizeof(buffer), ".cfg");
+		STRCPY_T(filename, sizeof(filename), buffer);
 		ai->cb->GetValue(AIVAL_LOCATE_FILE_R, filename);
 		file = fopen(filename, "r");
 	}
@@ -163,7 +166,7 @@ void AAIConfig::LoadConfig(AAI *ai)
 	const UnitDef *def;
 
 	bool error = false;
-	bool loaded = false;
+//	bool loaded = false;
 
 	if(file)
 	{
@@ -611,7 +614,7 @@ void AAIConfig::LoadConfig(AAI *ai)
 		}
 		else
 		{
-			loaded = true;
+//			loaded = true;
 			fclose(file);
 			fprintf(ai->file, "Mod config file loaded\n");
 		}
@@ -619,8 +622,8 @@ void AAIConfig::LoadConfig(AAI *ai)
 
 
 	// load general settings
-	STRCPY(buffer, MAIN_PATH);
-	STRCAT(buffer, GENERAL_CFG_FILE);
+	STRCPY_T(buffer, sizeof(buffer), MAIN_PATH);
+	STRCAT_T(buffer, sizeof(buffer), GENERAL_CFG_FILE);
 	ReplaceExtension(buffer, filename, sizeof(filename), ".cfg");
 	ai->cb->GetValue(AIVAL_LOCATE_FILE_R, filename);
 	file = fopen(filename, "r");
