@@ -1,9 +1,9 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef _ENGINEOUTHANDLER_H
-#define _ENGINEOUTHANDLER_H
+#ifndef ENGINE_OUT_HANDLER_H
+#define ENGINE_OUT_HANDLER_H
 
-#include "Object.h"
+#include "System/Object.h"
 #include "Sim/Misc/GlobalConstants.h"
 
 #include <map>
@@ -19,10 +19,10 @@ class SkirmishAIKey;
 class CSkirmishAIWrapper;
 struct SSkirmishAICallback;
 
+
 void handleAIException(const char* description);
 
 class CEngineOutHandler : public CObject {
-private:
 	CR_DECLARE(CEngineOutHandler);
 
 	CEngineOutHandler();
@@ -63,12 +63,16 @@ public:
 	void SeismicPing(int allyTeamId, const CUnit& unit, const float3& pos, float strength);
 	void WeaponFired(const CUnit& unit, const WeaponDef& def);
 	void PlayerCommandGiven(const std::vector<int>& selectedUnitIds, const Command& c, int playerId);
+
 	/**
 	 * A specific unit has finished a specific command,
 	 * might be a good idea to give new orders to it.
 	*/
 	void CommandFinished(const CUnit& unit, const Command& command);
-	void GotChatMsg(const char* msg, int playerId);
+	void SendChatMessage(const char* msg, int playerId);
+
+	/// send a raw string from unsynced Lua to one or all active skirmish AI's
+	bool SendLuaMessages(int aiTeam, const char* inData, std::vector<const char*>& outData);
 
 
 	// Skirmish AI stuff
@@ -150,13 +154,13 @@ private:
 	static CEngineOutHandler* singleton;
 
 private:
-	typedef std::vector<size_t> ids_t;
+	typedef std::vector<unsigned char> ids_t;
+	typedef std::map<unsigned char, CSkirmishAIWrapper*> id_ai_t;
+	typedef std::map<int, ids_t> team_ais_t;
 
-	typedef std::map<size_t, CSkirmishAIWrapper*> id_ai_t;
 	/// Contains all local Skirmish AIs, indexed by their ID
 	id_ai_t id_skirmishAI;
 
-	typedef std::map<int, ids_t> team_ais_t;
 	/**
 	 * Array mapping team IDs to local Skirmish AI instances.
 	 * There can be multiple Skirmish AIs per team.
@@ -166,4 +170,4 @@ private:
 
 #define eoh CEngineOutHandler::GetInstance()
 
-#endif // _ENGINEOUTHANDLER_H
+#endif // ENGINE_OUT_HANDLER_H
