@@ -2,10 +2,10 @@
 
 #include "SkirmishAIWrapper.h"
 
-#include "System/StdAfx.h"
+#include "System/FileSystem/DataDirsAccess.h"
+#include "System/FileSystem/FileQueryFlags.h"
 #include "System/FileSystem/FileSystem.h"
-#include "System/FileSystem/FileSystemHandler.h"
-#include "System/LogOutput.h"
+#include "System/Log/ILog.h"
 #include "System/mmgr.h"
 #include "System/Util.h"
 #include "Sim/Units/Unit.h"
@@ -28,6 +28,8 @@
 #include <iostream>
 #include <fstream>
 
+#undef DeleteFile
+
 CR_BIND_DERIVED(CSkirmishAIWrapper, CObject, )
 CR_REG_METADATA(CSkirmishAIWrapper, (
 	CR_MEMBER(skirmishAIId),
@@ -40,22 +42,30 @@ CR_REG_METADATA(CSkirmishAIWrapper, (
 
 /// used only by creg
 CSkirmishAIWrapper::CSkirmishAIWrapper():
-	skirmishAIId((size_t) -1),
-	teamId(-1),
-	cheatEvents(false),
-	ai(NULL),
-	initialized(false),
-	released(false),
-	c_callback(NULL)
-{}
+		skirmishAIId((size_t) -1),
+		teamId(-1),
+		cheatEvents(false),
+		ai(NULL),
+		initialized(false),
+		released(false),
+		callback(NULL),
+		cheats(NULL),
+		c_callback(NULL),
+		info(NULL)
+{
+}
 
 CSkirmishAIWrapper::CSkirmishAIWrapper(const size_t skirmishAIId):
-	skirmishAIId(skirmishAIId),
-	cheatEvents(false),
-	ai(NULL),
-	initialized(false),
-	released(false),
-	c_callback(NULL)
+		skirmishAIId(skirmishAIId),
+		teamId(-1),
+		cheatEvents(false),
+		ai(NULL),
+		initialized(false),
+		released(false),
+		callback(NULL),
+		cheats(NULL),
+		c_callback(NULL),
+		info(NULL)
 {
 	const SkirmishAIData* aiData = skirmishAIHandler.GetSkirmishAI(skirmishAIId);
 
@@ -71,7 +81,7 @@ void CSkirmishAIWrapper::CreateCallback() {
 	if (c_callback == NULL) {
 		callback = new CAICallback(teamId);
 		cheats = new CAICheats(this);
-		c_callback = skirmishAiCallback_getInstanceFor(teamId, callback, cheats);
+		c_callback = skirmishAiCallback_getInstanceFor(skirmishAIId, teamId, callback, cheats);
 	}
 }
 
@@ -88,7 +98,7 @@ CSkirmishAIWrapper::~CSkirmishAIWrapper() {
 		delete ai;
 		ai = NULL;
 
-		skirmishAiCallback_release(teamId);
+		skirmishAiCallback_release(skirmishAIId);
 		c_callback = NULL;
 
 		delete callback;
@@ -103,7 +113,7 @@ void CSkirmishAIWrapper::Serialize(creg::ISerializer* s) {
 }
 
 void CSkirmishAIWrapper::PostLoad() {
-	// CreateCallback();
+	//CreateCallback();
 	LoadSkirmishAI(true);
 }
 
@@ -111,7 +121,8 @@ void CSkirmishAIWrapper::PostLoad() {
 
 bool CSkirmishAIWrapper::LoadSkirmishAI(bool postLoad) {
 
-	ai = new CSkirmishAI(teamId, skirmishAIId, key, GetCallback());
+	ai = new CSkirmishAI(skirmishAIId, teamId, key, GetCallback());
+	ai->Init();
 
 	// check if initialization went ok
 	if (skirmishAIHandler.IsLocalSkirmishAIDieing(skirmishAIId)) {
@@ -176,11 +187,11 @@ void CSkirmishAIWrapper::Init() {
 		}
 	}
 
-	SInitEvent evtData = {teamId, GetCallback()};
+	SInitEvent evtData = {skirmishAIId, GetCallback()};
 	int error = ai->HandleEvent(EVENT_INIT, &evtData);
 	if (error != 0) {
 		// init failed
-		logOutput.Print("Failed to handle init event: AI for team %d, error %d",
+		LOG_L(L_ERROR, "Failed to handle init event: AI for team %d, error %d",
 				teamId, error);
 		skirmishAIHandler.SetLocalSkirmishAIDieing(skirmishAIId, 5 /* = AI failed to init */);
 	} else {
@@ -209,28 +220,39 @@ void CSkirmishAIWrapper::Release(int reason) {
 
 static void streamCopy(std::istream* in, std::ostream* out)
 {
-	static const size_t bufferSize = 128;
-	static char buffer[bufferSize];
+	static const size_t buffer_size = 128;
 
-	in->read(buffer, bufferSize);
+	// have to use _new_ here for VS compatibility (no C99 support)
+	char* buffer = new char[buffer_size];
 
+	in->read(buffer, buffer_size);
 	while (in->good()) {
 		out->write(buffer, in->gcount());
-		in->read(buffer, bufferSize);
+		in->read(buffer, buffer_size);
 	}
-
 	out->write(buffer, in->gcount());
+
+	delete[] buffer;
+}
+
+static std::string createTempFileName(const char* action, int teamId,
+		int skirmishAIId) {
+
+	static const size_t tmpFileName_size = 1024;
+
+	// have to use _new_ here for VS compatibility (no C99 support)
+	char* tmpFileName = new char[tmpFileName_size];
+	SNPRINTF(tmpFileName, tmpFileName_size, "%s-team%i_id%i.tmp", action,
+			teamId, skirmishAIId);
+	std::string tmpFile = dataDirsAccess.LocateFile(tmpFileName,
+			FileQueryFlags::WRITE | FileQueryFlags::CREATE_DIRS);
+	delete[] tmpFileName;
+	return tmpFile;
 }
 
 void CSkirmishAIWrapper::Load(std::istream* load_s)
 {
-	static const int fsBits = FileSystem::WRITE | FileSystem::CREATE_DIRS;
-	static const size_t tmpFileNameLen = 1024;
-	static char tmpFileName[tmpFileNameLen];
-
-	SNPRINTF(tmpFileName, tmpFileNameLen, "%s-team_%i.tmp", "load", teamId);
-
-	const std::string& tmpFile = filesystem.LocateFile(tmpFileName, fsBits);
+	const std::string tmpFile = createTempFileName("load", teamId, skirmishAIId);
 
 	std::ofstream tmpFile_s;
 	tmpFile_s.open(tmpFile.c_str(), std::ios::binary);
@@ -240,28 +262,22 @@ void CSkirmishAIWrapper::Load(std::istream* load_s)
 	SLoadEvent evtData = {tmpFile.c_str()};
 	ai->HandleEvent(EVENT_LOAD, &evtData);
 
-	FileSystemHandler::DeleteFile(tmpFile);
+	FileSystem::DeleteFile(tmpFile);
 }
 
 void CSkirmishAIWrapper::Save(std::ostream* save_s)
 {
-	static const int fsBits = FileSystem::WRITE | FileSystem::CREATE_DIRS;
-	static const size_t tmpFileNameLen = 1024;
-	static char tmpFileName[tmpFileNameLen];
-
-	SNPRINTF(tmpFileName, tmpFileNameLen, "%s-team_%i.tmp", "save", teamId);
-
-	const std::string& tmpFile = filesystem.LocateFile(tmpFileName, fsBits);
+	const std::string tmpFile = createTempFileName("save", teamId, skirmishAIId);
 
 	SSaveEvent evtData = {tmpFile.c_str()};
 	ai->HandleEvent(EVENT_SAVE, &evtData);
 
-	if (FileSystemHandler::FileExists(tmpFile)) {
+	if (FileSystem::FileExists(tmpFile)) {
 		std::ifstream tmpFile_s;
 		tmpFile_s.open(tmpFile.c_str(), std::ios::binary);
 		streamCopy(&tmpFile_s, save_s);
 		tmpFile_s.close();
-		FileSystemHandler::DeleteFile(tmpFile);
+		FileSystem::DeleteFile(tmpFile);
 	}
 }
 
@@ -289,15 +305,11 @@ void CSkirmishAIWrapper::UnitDestroyed(int unitId, int attackerUnitId) {
 void CSkirmishAIWrapper::UnitDamaged(int unitId, int attackerUnitId,
 		float damage, const float3& dir, int weaponDefId, bool paralyzer) {
 
-	SUnitDamagedEvent evtData = {
-		unitId,
-		attackerUnitId,
-		damage,
-		dir.toSAIFloat3(),
-		weaponDefId,
-		paralyzer
-	};
+	SUnitDamagedEvent evtData = {unitId, attackerUnitId, damage,
+			new float[3], weaponDefId, paralyzer};
+	dir.copyInto(evtData.dir_posF3);
 	ai->HandleEvent(EVENT_UNIT_DAMAGED, &evtData);
+	delete [] evtData.dir_posF3;
 }
 
 void CSkirmishAIWrapper::UnitMoveFailed(int unitId) {
@@ -316,7 +328,6 @@ void CSkirmishAIWrapper::UnitCaptured(int unitId, int oldTeam, int newTeam) {
 }
 
 
-
 void CSkirmishAIWrapper::EnemyCreated(int unitId) {
 	SEnemyCreatedEvent evtData = {unitId};
 	ai->HandleEvent(EVENT_ENEMY_CREATED, &evtData);
@@ -355,27 +366,26 @@ void CSkirmishAIWrapper::EnemyDestroyed(int enemyUnitId, int attackerUnitId) {
 void CSkirmishAIWrapper::EnemyDamaged(int enemyUnitId, int attackerUnitId,
 		float damage, const float3& dir, int weaponDefId, bool paralyzer) {
 
-	SEnemyDamagedEvent evtData = {
-		enemyUnitId,
-		attackerUnitId,
-		damage,
-		dir.toSAIFloat3(),
-		weaponDefId,
-		paralyzer
-	};
+	SEnemyDamagedEvent evtData = {enemyUnitId, attackerUnitId, damage,
+			new float[3], weaponDefId, paralyzer};
+	dir.copyInto(evtData.dir_posF3);
 	ai->HandleEvent(EVENT_ENEMY_DAMAGED, &evtData);
+	delete [] evtData.dir_posF3;
 }
 
-
-
 void CSkirmishAIWrapper::Update(int frame) {
 	SUpdateEvent evtData = {frame};
 	ai->HandleEvent(EVENT_UPDATE, &evtData);
 }
 
-void CSkirmishAIWrapper::GotChatMsg(const char* msg, int fromPlayerId) {
-	SMessageEvent evtData = {fromPlayerId, msg};
-	ai->HandleEvent(EVENT_MESSAGE, &evtData);
+void CSkirmishAIWrapper::SendChatMessage(const char* msg, int fromPlayerId) {
+	SChatMessageEvent evtData = {fromPlayerId, msg};
+	ai->HandleEvent(EVENT_CHAT_MESSAGE, &evtData);
+}
+
+void CSkirmishAIWrapper::SendLuaMessage(const char* inData, const char** outData) {
+	SLuaMessageEvent evtData = {inData, outData};
+	ai->HandleEvent(EVENT_LUA_MESSAGE, &evtData);
 }
 
 void CSkirmishAIWrapper::WeaponFired(int unitId, int weaponDefId) {
@@ -386,26 +396,19 @@ void CSkirmishAIWrapper::WeaponFired(int unitId, int weaponDefId) {
 void CSkirmishAIWrapper::PlayerCommandGiven(
 		const std::vector<int>& selectedUnits, const Command& c, int playerId) {
 
-	unsigned int numUnits = selectedUnits.size();
-	int* unitIds = new int[numUnits];
-
-	for (unsigned int i = 0; i < numUnits; ++i) {
+	const int unitIds_size = selectedUnits.size();
+	int* unitIds = new int[unitIds_size];
+	for (int i = 0; i < unitIds_size; ++i) {
 		unitIds[i] = selectedUnits.at(i);
 	}
+	const int cCommandId = extractAICommandTopic(&c, uh->MaxUnits());
 
-	int sCommandId;
-	void* sCommandData = mallocSUnitCommand(-1, -1, &c, &sCommandId);
-
-	SPlayerCommandEvent evtData = {unitIds, numUnits, sCommandId, sCommandData,
-			playerId};
+	SPlayerCommandEvent evtData = {unitIds, unitIds_size, cCommandId, playerId};
 	ai->HandleEvent(EVENT_PLAYER_COMMAND, &evtData);
 	delete [] unitIds;
 }
 
-void CSkirmishAIWrapper::CommandFinished(int unitId, int commandTopicId) {
-	// TODO: add support for commandIds:
-	// each issued command would have its own id
-	const int commandId = -1;
+void CSkirmishAIWrapper::CommandFinished(int unitId, int commandId, int commandTopicId) {
 	SCommandFinishedEvent evtData = {unitId, commandId, commandTopicId};
 	ai->HandleEvent(EVENT_COMMAND_FINISHED, &evtData);
 }
@@ -413,17 +416,24 @@ void CSkirmishAIWrapper::CommandFinished(int unitId, int commandTopicId) {
 void CSkirmishAIWrapper::SeismicPing(int allyTeam, int unitId,
 		const float3& pos, float strength) {
 
-	SSeismicPingEvent evtData = {pos.toSAIFloat3(), strength};
+	SSeismicPingEvent evtData = {new float[3], strength};
+	pos.copyInto(evtData.pos_posF3);
 	ai->HandleEvent(EVENT_SEISMIC_PING, &evtData);
+	delete [] evtData.pos_posF3;
 }
 
 
+int CSkirmishAIWrapper::GetTeamId() const {
+	return teamId;
+}
 
-int CSkirmishAIWrapper::GetTeamId() const { return teamId; }
-const SkirmishAIKey& CSkirmishAIWrapper::GetKey() const { return key; }
-const SSkirmishAICallback* CSkirmishAIWrapper::GetCallback() const { return c_callback; }
-
+const SkirmishAIKey& CSkirmishAIWrapper::GetKey() const {
+	return key;
+}
 
+const SSkirmishAICallback* CSkirmishAIWrapper::GetCallback() const {
+	return c_callback;
+}
 
 void CSkirmishAIWrapper::SetCheatEventsEnabled(bool enable) {
 	cheatEvents = enable;
