@@ -15,14 +15,6 @@
 
 class TdfParser;
 
-namespace GameMode
-{
-	const int ComContinue = 0;
-	const int ComEnd = 1;
-	const int Lineage = 2;
-	const int OpenEnd = 3;
-};
-
 class CGameSetup
 {
 public:
@@ -64,8 +56,7 @@ public:
 	std::vector<PlayerBase> playerStartingData;
 
 	const std::vector<SkirmishAIData>& GetSkirmishAIs() const;
-private:
-	const SkirmishAIData* GetSkirmishAIDataForTeam(int teamId) const;
+
 public:
 
 	std::vector<TeamBase> teamStartingData;
@@ -79,7 +70,6 @@ public:
 	int maxUnits;
 
 	bool ghostedBuildings;
-	bool limitDgun;
 	bool disableMapDamage;
 
 	float maxSpeed;
@@ -101,8 +91,7 @@ public:
 	 */
 	unsigned int gameStartDelay;
 
-	int gameMode;
-	int noHelperAIs;
+	bool noHelperAIs;
 
 private:
 	/**
