@@ -362,8 +362,10 @@ void CGame::ClientReadNet()
 				SimFrame();
 				// both NETMSG_SYNCRESPONSE and NETMSG_NEWFRAME are used for ping calculation by server
 #ifdef SYNCCHECK
-				net->Send(CBaseNetProtocol::Get().SendSyncResponse(gs->frameNum, CSyncChecker::GetChecksum()));
-				if ((gs->frameNum & 4095) == 0) {// reset checksum every ~2.5 minute gametime
+				net->Send(CBaseNetProtocol::Get().SendSyncResponse(gu->myPlayerNum, gs->frameNum, CSyncChecker::GetChecksum()));
+
+				if ((gs->frameNum & 4095) == 0) {
+					// reset checksum every 4096 frames =~ 2.5 minutes
 					CSyncChecker::NewFrame();
 				}
 #endif
@@ -375,69 +377,107 @@ void CGame::ClientReadNet()
 				break;
 			}
 
+			case NETMSG_SYNCRESPONSE: {
+#if (defined(SYNCCHECK) && !defined(NDEBUG))
+				// NOTE:
+				//     this packet is also sent during live games,
+				//     during which we should just ignore it (the
+				//     server does sync-checking for us)
+				netcode::UnpackPacket pckt(packet, 1);
+
+				unsigned char playerNum; pckt >> playerNum;
+				          int  frameNum; pckt >> frameNum;
+				unsigned  int  checkSum; pckt >> checkSum;
+
+				const unsigned int ourCheckSum = CSyncChecker::GetChecksum();
+				const char* fmtStr =
+					"[DESYNC_WARNING] checksum %x from player %d (%s)"
+					" does not match our checksum %x for frame-number %d";
+				const CPlayer* player = playerHandler->Player(playerNum);
+
+				// check if our checksum for this frame matches what
+				// player <playerNum> sent to the server at the same
+				// frame in the original game (in case of a demo)
+				if (playerNum == gu->myPlayerNum) { return; }
+				if (gs->frameNum != frameNum) { return; }
+				if (checkSum == ourCheckSum) { return; }
+
+				LOG_L(L_ERROR, fmtStr, checkSum, playerNum, player->name.c_str(), ourCheckSum, gs->frameNum);
+#endif
+			} break;
+
+
 			case NETMSG_COMMAND: {
 				try {
 					netcode::UnpackPacket pckt(packet, 1);
-					short int psize;
-					pckt >> psize;
-					unsigned char player;
-					pckt >> player;
-					if (!playerHandler->IsValidPlayer(player))
+
+					unsigned short packetSize; pckt >> packetSize;
+					unsigned char playerNum; pckt >> playerNum;
+					const unsigned int numParams = (packetSize - 9) / sizeof(float);
+
+					if (!playerHandler->IsValidPlayer(playerNum))
 						throw netcode::UnpackPacketException("Invalid player number");
 
-					int cmd_id;
-					unsigned char cmd_opt;
-					pckt >> cmd_id;
-					pckt >> cmd_opt;
+					int cmdID;
+					unsigned char cmdOpt;
+					pckt >> cmdID;
+					pckt >> cmdOpt;
 
-					Command c(cmd_id, cmd_opt);
-					for (int a = 0; a < ((psize-9)/4); ++a) {
-						float param;
-						pckt >> param;
+					Command c(cmdID, cmdOpt);
+					c.params.reserve(numParams);
+
+					for (int a = 0; a < numParams; ++a) {
+						float param; pckt >> param;
 						c.params.push_back(param);
 					}
-					selectedUnits.NetOrder(c,player);
-					AddTraffic(player, packetCode, dataLength);
+
+					selectedUnits.NetOrder(c, playerNum);
+					AddTraffic(playerNum, packetCode, dataLength);
 				} catch (const netcode::UnpackPacketException& ex) {
 					LOG_L(L_ERROR, "Got invalid Command: %s", ex.what());
 				}
+
 				break;
 			}
 
 			case NETMSG_SELECT: {
 				try {
 					netcode::UnpackPacket pckt(packet, 1);
-					short int psize;
-					pckt >> psize;
-					unsigned char player;
-					pckt >> player;
-					if (!playerHandler->IsValidPlayer(player))
+
+					unsigned short packetSize; pckt >> packetSize;
+					unsigned char playerNum; pckt >> playerNum;
+					const unsigned int numUnitIDs = (packetSize - 4) / sizeof(short int);
+
+					if (!playerHandler->IsValidPlayer(playerNum)) {
 						throw netcode::UnpackPacketException("Invalid player number");
+					}
 
-					vector<int> selected;
-					bool firsterr = true;
-					for (int a = 0; a < ((psize-4)/2); ++a) {
-						short int unitid;
-						pckt >> unitid;
+					std::vector<int> selectedUnitIDs;
+					selectedUnitIDs.reserve(numUnitIDs);
 
-						if (uh->GetUnit(unitid) == NULL) {
-							if (firsterr) {
-								LOG_L(L_WARNING, "Got invalid Select: Invalid unit ID (%i)", unitid);
-							}
-							firsterr = false;
+					for (int a = 0; a < numUnitIDs; ++a) {
+						short int unitID; pckt >> unitID;
+						const CUnit* unit = uh->GetUnit(unitID);
+
+						if (unit == NULL) {
+							// unit was destroyed in simulation (without its ID being recycled)
+							// after sending a command but before receiving it back, more likely
+							// to happen in high-latency situations
+							// LOG_L(L_WARNING, "[NETMSG_SELECT] invalid unitID (%i) from player %i", unitID, playerNum);
 							continue;
 						}
 
-						if ((uh->GetUnit(unitid)->team == playerHandler->Player(player)->team) || gs->godMode) {
-							selected.push_back(unitid);
+						if ((unit->team == playerHandler->Player(playerNum)->team) || gs->godMode) {
+							selectedUnitIDs.push_back(unitID);
 						}
 					}
-					selectedUnits.NetSelect(selected, player);
 
-					AddTraffic(player, packetCode, dataLength);
+					selectedUnits.NetSelect(selectedUnitIDs, playerNum);
+					AddTraffic(playerNum, packetCode, dataLength);
 				} catch (const netcode::UnpackPacketException& ex) {
 					LOG_L(L_ERROR, "Got invalid Select: %s", ex.what());
 				}
+
 				break;
 			}
 
@@ -1033,10 +1073,13 @@ void CGame::ClientReadNet()
 					player.team = team;
 					player.playerNum = playerNum;
 					// add the new player
+					// TODO NETMSG_CREATE_NEWPLAYER perhaps add a lua hook; hook should be able to reassign the player to a team and/or create a new team/allyteam
 					playerHandler->AddPlayer(player);
 					eventHandler.PlayerAdded(player.playerNum);
 					LOG("Added new player: %s", name.c_str());
-					// TODO NETMSG_CREATE_NEWPLAYER perhaps add a lua hook; hook should be able to reassign the player to a team and/or create a new team/allyteam
+					if (!player.spectator) {
+						eventHandler.TeamChanged(player.team);
+					}
 					AddTraffic(-1, packetCode, dataLength);
 				} catch (const netcode::UnpackPacketException& ex) {
 					LOG_L(L_ERROR, "Got invalid New player message: %s", ex.what());
@@ -1062,7 +1105,4 @@ void CGame::ClientReadNet()
 			}
 		}
 	}
-
-	return;
 }
-
