@@ -1,14 +1,12 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-
-#include <cstdio>
-#include "mmgr.h"
-
 #include "EndGameBox.h"
+
 #include "MouseHandler.h"
+#include "Game/Player.h"
 #include "Game/PlayerHandler.h"
 #include "Game/Game.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/SelectedUnits.h"
 #include "Rendering/glFont.h"
 #include "Rendering/GL/myGL.h"
@@ -16,11 +14,13 @@
 #include "Sim/Misc/GlobalSynced.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/Misc/TeamStatistics.h"
-#include "System/LogOutput.h"
+#include "System/mmgr.h"
 #include "System/Exceptions.h"
 
+#include <cstdio>
+#include <sstream>
+
 using std::sprintf;
-extern bool globalQuit;
 
 
 static std::string FloatToSmallString(float num, float mul = 1) {
@@ -43,15 +43,16 @@ static std::string FloatToSmallString(float num, float mul = 1) {
 };
 
 
-bool CEndGameBox::disabled = false;
+bool CEndGameBox::enabled = true;
 
 
-CEndGameBox::CEndGameBox()
+CEndGameBox::CEndGameBox(const std::vector<unsigned char>& winningAllyTeams)
 	: CInputReceiver()
 	, moveBox(false)
 	, dispMode(0)
 	, stat1(1)
 	, stat2(-1)
+	, winners(winningAllyTeams)
 	, graphTex(0)
 {
 	box.x1 = 0.14f;
@@ -93,7 +94,7 @@ CEndGameBox::~CEndGameBox()
 
 bool CEndGameBox::MousePress(int x, int y, int button)
 {
-	if (disabled) {
+	if (!enabled) {
 		return false;
 	}
 
@@ -124,7 +125,7 @@ bool CEndGameBox::MousePress(int x, int y, int button)
 
 void CEndGameBox::MouseMove(int x, int y, int dx, int dy, int button)
 {
-	if (disabled) {
+	if (!enabled) {
 		return;
 	}
 
@@ -138,7 +139,7 @@ void CEndGameBox::MouseMove(int x, int y, int dx, int dy, int button)
 
 void CEndGameBox::MouseRelease(int x, int y, int button)
 {
-	if (disabled) {
+	if (!enabled) {
 		return;
 	}
 
@@ -147,7 +148,7 @@ void CEndGameBox::MouseRelease(int x, int y, int button)
 
 	if (InBox(mx, my, box + exitBox)) {
 		delete this;
-		globalQuit = true;
+		gu->globalQuit = true;
 		return;
 	}
 
@@ -179,7 +180,7 @@ void CEndGameBox::MouseRelease(int x, int y, int button)
 
 bool CEndGameBox::IsAbove(int x, int y)
 {
-	if (disabled) {
+	if (!enabled) {
 		return false;
 	}
 
@@ -194,7 +195,7 @@ void CEndGameBox::Draw()
 		graphTex = bm.CreateTexture();
 	}
 
-	if (disabled) {
+	if (!enabled) {
 		return;
 	}
 
@@ -242,10 +243,41 @@ void CEndGameBox::Draw()
 	font->glPrint(box.x1 + sumBox.x1    + 0.015f, box.y1 + sumBox.y1    + 0.005f, 0.7f, FONT_SCALE | FONT_NORM, "Team stats");
 	font->glPrint(box.x1 + difBox.x1    + 0.015f, box.y1 + difBox.y1    + 0.005f, 0.7f, FONT_SCALE | FONT_NORM, "Team delta stats");
 
-	if (teamHandler->Team(gu->myTeam)->isDead) {
-		font->glPrint(box.x1 + 0.25f, box.y1 + 0.65f, 1.0f, FONT_SCALE | FONT_NORM, "You lost the game");
+	if (winners.empty()) {
+		font->glPrint(box.x1 + 0.25f, box.y1 + 0.65f, 1.0f, FONT_SCALE | FONT_NORM, "Game result was undecided");
 	} else {
-		font->glPrint(box.x1 + 0.25f, box.y1 + 0.65f, 1.0f, FONT_SCALE | FONT_NORM, "You won the game");
+		std::stringstream winnersText;
+		std::stringstream winnersList;
+
+		// myPlayingAllyTeam is >= 0 iff we ever joined a team
+		const bool neverPlayed = (gu->myPlayingAllyTeam < 0);
+		      bool playedAndWon = false;
+
+		for (unsigned int i = 0; i < winners.size(); i++) {
+			const int winner = winners[i];
+
+			if (!neverPlayed && winner == gu->myPlayingAllyTeam) {
+				// we actually played and won!
+				playedAndWon = true; break;
+			}
+
+			winnersList << winner;
+
+			if (i < (winners.size() - 1))
+				winnersList << ", ";
+		}
+
+		if (neverPlayed) {
+			winnersText << "Game Over! Winning ally-teams are: ";
+			winnersText << winnersList.str();
+
+			font->glPrint(box.x1 + 0.25f, box.y1 + 0.65f, 1.0f, FONT_SCALE | FONT_NORM, (winnersText.str()).c_str());
+		} else {
+			winnersText.str("");
+			winnersText << "Game Over! Your ally-team ";
+			winnersText << (playedAndWon? "won the game": "lost the game");
+			font->glPrint(box.x1 + 0.25f, box.y1 + 0.65f, 1.0f, FONT_SCALE | FONT_NORM, (winnersText.str()).c_str());
+		}
 	}
 
 	if (gs->frameNum <= 0) {
@@ -412,7 +444,7 @@ void CEndGameBox::Draw()
 
 std::string CEndGameBox::GetTooltip(int x, int y)
 {
-	if (disabled) {
+	if (!enabled) {
 		return "";
 	}
 
@@ -477,7 +509,7 @@ void CEndGameBox::FillTeamStats()
 			continue;
 		}
 
-		for (std::list<CTeam::Statistics>::const_iterator si = pteam->statHistory.begin(); si != pteam->statHistory.end(); si++) {
+		for (std::list<CTeam::Statistics>::const_iterator si = pteam->statHistory.begin(); si != pteam->statHistory.end(); ++si) {
 			stats[0].AddStat(team, 0);
 
 			stats[1].AddStat(team, si->metalUsed);
