@@ -169,7 +169,7 @@ void CGameInfo::Draw()
 		values.push_back(FileSystem::GetBasename(gameSetup->demoName));
 	}
 
-	labels.push_back("Game Version:");
+	labels.push_back("Spring Version:");
 	values.push_back(SpringVersion::GetFull());
 
 #ifdef USE_GML
@@ -199,7 +199,7 @@ void CGameInfo::Draw()
 	labels.push_back("Map Name:");
 	values.push_back(gameSetup->mapName);
 
-	labels.push_back("Mod Name:");
+	labels.push_back("Game Name:");
 	values.push_back(gameSetup->modName);
 
 	if (gs->cheatEnabled) {
