@@ -1,32 +1,30 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "Rendering/GL/myGL.h"
-#include <assert.h>
-#include <string>
-#include <map>
-#include <SDL_timer.h>
-#include <SDL_keysym.h>
-
-#include "mmgr.h"
-
 #include "GameSetupDrawer.h"
 
+#include "KeyBindings.h"
+#include "StartPosSelecter.h"
 #include "Game/GameServer.h"
-
-#include "NetProtocol.h"
-#include "ConfigHandler.h"
 #include "Game/CameraHandler.h"
+#include "Game/Player.h"
 #include "Game/PlayerHandler.h"
 #include "Game/GameSetup.h"
+#include "Game/GlobalUnsynced.h"
 #include "Sim/Misc/GlobalSynced.h"
 #include "Sim/Misc/TeamHandler.h"
-#include "StartPosSelecter.h"
+#include "Rendering/GL/myGL.h"
 #include "Rendering/glFont.h"
-#include "KeyBindings.h"
-#include "EventHandler.h"
+#include "System/NetProtocol.h"
+#include "System/Config/ConfigHandler.h"
+#include "System/EventHandler.h"
+#include "System/mmgr.h"
 
-extern boost::uint8_t *keys;
+#include <SDL_timer.h>
+#include <SDL_keysym.h>
+
+#include <cassert>
+#include <string>
+#include <map>
 
 
 GameSetupDrawer* GameSetupDrawer::instance = NULL;
@@ -52,11 +50,11 @@ void GameSetupDrawer::StartCountdown(unsigned time)
 	if (instance) {
 		instance->lastTick = SDL_GetTicks();
 		instance->readyCountdown = (int)time;
-		const std::string modeName = configHandler->GetString("CamModeName", "");
+		const std::string modeName = configHandler->GetString("CamModeName");
 		if (!modeName.empty()) {
 			camHandler->SetCameraMode(modeName);
 		} else {
-			const int modeIndex = configHandler->Get("CamMode", 1);
+			const int modeIndex = configHandler->GetInt("CamMode");
 			camHandler->SetCameraMode(modeIndex);
 		}
 	}
