@@ -1,21 +1,8 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifdef _MSC_VER
-#include "StdAfx.h"
-#endif
 #include "Lua/LuaLobby.h" // ugh, streflop namespace corruption...
-#ifndef _MSC_VER
-#include "StdAfx.h"
-#endif
 
-#include <stdio.h>
-#include <set>
-#include <cctype>
-#include <SDL_keysym.h>
-#include <SDL_mouse.h>
-#include <SDL_timer.h>
-
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "LuaUI.h"
 
@@ -23,7 +10,6 @@
 
 #include "Lua/LuaUnsyncedCtrl.h"
 #include "Lua/LuaCallInCheck.h"
-#include "Lua/LuaUtils.h"
 #include "Lua/LuaConstGL.h"
 #include "Lua/LuaConstCMD.h"
 #include "Lua/LuaConstCMDTYPE.h"
@@ -36,6 +22,7 @@
 #include "Lua/LuaWeaponDefs.h"
 #include "Lua/LuaScream.h"
 #include "Lua/LuaOpenGL.h"
+#include "Lua/LuaUtils.h"
 #include "Lua/LuaVFS.h"
 #include "Lua/LuaIO.h"
 #include "Lua/LuaZip.h"
@@ -43,6 +30,7 @@
 #include "Game/Camera/CameraController.h"
 #include "Game/Game.h"
 #include "Game/GameHelper.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/PlayerRoster.h"
 #include "Game/SelectedUnits.h"
 #include "CommandColors.h"
@@ -56,32 +44,26 @@
 #include "MouseHandler.h"
 #include "Map/ReadMap.h"
 #include "Rendering/IconHandler.h"
-#include "Rendering/InMapDraw.h"
-#include "EventHandler.h"
-#include "LogOutput.h"
-#include "FileSystem/FileHandler.h"
-#include "FileSystem/VFSHandler.h"
-#include "ConfigHandler.h"
-#include "FileSystem/FileSystem.h"
-#include "Util.h"
+#include "System/EventHandler.h"
+#include "System/Log/ILog.h"
+#include "System/FileSystem/FileHandler.h"
+#include "System/FileSystem/VFSHandler.h"
+#include "System/Config/ConfigHandler.h"
+#include "System/FileSystem/FileSystem.h"
+#include "System/Util.h"
 
-using namespace std;
+#include <stdio.h>
+#include <set>
+#include <cctype>
+#include <SDL_keysym.h>
+#include <SDL_mouse.h>
+#include <SDL_timer.h>
 
-#if (LUA_VERSION_NUM < 500)
-#  define LUA_OPEN_LIB(L, lib) lib(L)
-#else
-#  define LUA_OPEN_LIB(L, lib) \
-     lua_pushcfunction((L), lib); \
-     lua_pcall((L), 0, 0, 0);
-#endif
-
-
-extern boost::uint8_t *keys;
+using std::max;
 
 
 CLuaUI* luaUI = NULL;
 
-
 const int CMD_INDEX_OFFSET = 1; // starting index for command descriptions
 
 
@@ -95,9 +77,11 @@ void CLuaUI::LoadHandler()
 		return;
 	}
 
+	GML_STDMUTEX_LOCK(luaui); // LoadHandler
+
 	new CLuaUI();
 
-	if (luaUI->L == NULL) {
+	if (!luaUI->IsValid()) {
 		delete luaUI;
 	}
 }
@@ -107,6 +91,9 @@ void CLuaUI::FreeHandler()
 {
 	static bool inFree = false;
 	if (!inFree) {
+
+		GML_STDMUTEX_LOCK(luaui); // FreeHandler
+
 		inFree = true;
 		delete luaUI;
 		inFree = false;
@@ -121,7 +108,9 @@ CLuaUI::CLuaUI()
 {
 	luaUI = this;
 
-	if (L == NULL) {
+	BEGIN_ITERATE_LUA_STATES();
+
+	if (!IsValid()) {
 		return;
 	}
 
@@ -172,7 +161,7 @@ CLuaUI::CLuaUI()
 
 	lua_pushvalue(L, LUA_GLOBALSINDEX);
 
-	AddBasicCalls(); // into Global
+	AddBasicCalls(L); // into Global
 
 	lua_pushstring(L, "Script");
 	lua_rawget(L, -2);
@@ -203,7 +192,7 @@ CLuaUI::CLuaUI()
 	}
 
 	lua_settop(L, 0);
-	if (!LoadCode(code, "luaui.lua")) {
+	if (!LoadCode(L, code, "luaui.lua")) {
 		KillLua();
 		return;
 	}
@@ -212,16 +201,20 @@ CLuaUI::CLuaUI()
 	eventHandler.AddClient(this);
 
 	// update extra call-ins
-	UnsyncedUpdateCallIn("WorldTooltip");
-	UnsyncedUpdateCallIn("MapDrawCmd");
+	UnsyncedUpdateCallIn(L, "WorldTooltip");
+	UnsyncedUpdateCallIn(L, "MapDrawCmd");
+
+	UpdateUnsyncedXCalls(L);
 
 	lua_settop(L, 0);
+
+	END_ITERATE_LUA_STATES();
 }
 
 
 CLuaUI::~CLuaUI()
 {
-	if (L != NULL) {
+	if (L_Sim != NULL || L_Draw != NULL) {
 		Shutdown();
 		KillLua();
 	}
@@ -235,10 +228,10 @@ string CLuaUI::LoadFile(const string& filename) const
 	CFileHandler lockFile("gamedata/LockLuaUI.txt", SPRING_VFS_MOD);
 	if (lockFile.FileExists()) {
 		if (!CLuaHandle::GetDevMode()) {
-			logOutput.Print("This mod has locked LuaUI access");
+			LOG("This game has locked LuaUI access");
 			accessMode = SPRING_VFS_MOD;
 		} else {
-			logOutput.Print("Bypassing this mod's LuaUI access lock");
+			LOG("Bypassing this game's LuaUI access lock");
 			accessMode = SPRING_VFS_RAW SPRING_VFS_MOD;
 		}
 	}
@@ -253,9 +246,9 @@ string CLuaUI::LoadFile(const string& filename) const
 }
 
 
-bool CLuaUI::HasCallIn(const string& name)
+bool CLuaUI::HasCallIn(lua_State *L, const string& name)
 {
-	if (L == NULL) {
+	if (!IsValid()) {
 		return false;
 	}
 
@@ -274,18 +267,21 @@ bool CLuaUI::HasCallIn(const string& name)
 }
 
 
-bool CLuaUI::UnsyncedUpdateCallIn(const string& name)
+bool CLuaUI::UnsyncedUpdateCallIn(lua_State *L, const string& name)
 {
 	// never allow this call-in
 	if (name == "Explosion") {
 		return false;
 	}
 
-	if (HasCallIn(name)) {
+	if (HasCallIn(L, name)) {
 		eventHandler.InsertEvent(this, name);
 	} else {
 		eventHandler.RemoveEvent(this, name);
 	}
+
+	UpdateUnsyncedXCalls(L);
+
 	return true;
 }
 
@@ -293,13 +289,13 @@ bool CLuaUI::UnsyncedUpdateCallIn(const string& name)
 void CLuaUI::UpdateTeams()
 {
 	if (luaUI) {
-		luaUI->fullCtrl = gs->godMode;
-		luaUI->ctrlTeam = gs->godMode ? AllAccessTeam :
-		                  (gu->spectating ? NoAccessTeam : gu->myTeam);
-		luaUI->fullRead = gu->spectatingFullView;
-		luaUI->readTeam = luaUI->fullRead ? AllAccessTeam : gu->myTeam;
-		luaUI->readAllyTeam = luaUI->fullRead ? AllAccessTeam : gu->myAllyTeam;
-		luaUI->selectTeam = gu->spectatingFullSelect ? AllAccessTeam : gu->myTeam;
+		luaUI->SetFullCtrl(gs->godMode, true);
+		luaUI->SetCtrlTeam(gs->godMode ? AllAccessTeam :
+		                  (gu->spectating ? NoAccessTeam : gu->myTeam), true);
+		luaUI->SetFullRead(gu->spectatingFullView, true);
+		luaUI->SetReadTeam(luaUI->GetFullRead() ? AllAccessTeam : gu->myTeam, true);
+		luaUI->SetReadAllyTeam(luaUI->GetFullRead() ? AllAccessTeam : gu->myAllyTeam, true);
+		luaUI->SetSelectTeam(gu->spectatingFullSelect ? AllAccessTeam : gu->myTeam, true);
 	}
 }
 
@@ -317,6 +313,7 @@ bool CLuaUI::LoadCFunctions(lua_State* L)
 	lua_rawset(L, -3)
 
 	REGISTER_LUA_CFUNC(SetShockFrontFactors);
+	REGISTER_LUA_CFUNC(SendToUnsynced);
 
 	lua_setglobal(L, "Spring");
 
@@ -362,41 +359,6 @@ bool CLuaUI::ConfigCommand(const string& command)
 }
 
 
-bool CLuaUI::AddConsoleLines()
-{
-	CInfoConsole* ic = game->infoConsole;
-	if (ic == NULL) {
-		return true;
-	}
-
-	vector<CInfoConsole::RawLine> lines;
-	ic->GetNewRawLines(lines);
-
-	LUA_CALL_IN_CHECK(L);
-	lua_checkstack(L, 3);
-
-	const int count = (int)lines.size();
-	for (int i = 0; i < count; i++) {
-		const CInfoConsole::RawLine& rl = lines[i];
-
-		static const LuaHashString cmdStr("AddConsoleLine");
-		if (!cmdStr.GetGlobalFunc(L)) {
-			return true; // the call is not defined
-		}
-
-		// FIXME: migrate priority to subsystem...
-		lua_pushstring(L, rl.text.c_str());
-		lua_pushnumber(L, 0 /*priority*/ );
-		//lua_pushstring(L, rl.subsystem->name);
-
-		// call the function
-		if (!RunCallIn(cmdStr, 2, 0)) {
-			return false;
-		}
-	}
-	return true;
-}
-
 
 static inline float fuzzRand(float fuzz)
 {
@@ -404,23 +366,26 @@ static inline float fuzzRand(float fuzz)
 }
 
 
-void CLuaUI::ShockFront(float power, const float3& pos, float areaOfEffect)
+void CLuaUI::ShockFront(float power, const float3& pos, float areaOfEffect, float *distadj)
 {
 	if (!haveShockFront) {
 		return;
 	}
-	if (areaOfEffect < shockFrontMinArea) {
+	if (areaOfEffect < shockFrontMinArea && !distadj) {
 		return;
 	}
-
+#if defined(USE_GML) && GML_ENABLE_SIM
+	float shockFrontDistAdj = distadj ? *distadj : this->shockFrontDistAdj;
+#endif
 	float3 gap = (camera->pos - pos);
 	float dist = gap.Length() + shockFrontDistAdj;
 
 	power = power / (dist * dist);
-	if (power < shockFrontMinPower) {
+	if (power < shockFrontMinPower && !distadj) {
 		return;
 	}
 
+	LUA_UI_BATCH_PUSH(SHOCK_FRONT, power, pos, areaOfEffect, shockFrontDistAdj);
 	LUA_CALL_IN_CHECK(L);
 	lua_checkstack(L, 6);
 	static const LuaHashString cmdStr("ShockFront");
@@ -443,7 +408,7 @@ void CLuaUI::ShockFront(float power, const float3& pos, float areaOfEffect)
 	lua_pushnumber(L, dir.y);
 	lua_pushnumber(L, dir.z);
 
-	// call the routinea
+	// call the routine
 	if (!RunCallIn(cmdStr, 4, 0)) {
 		return;
 	}
@@ -452,11 +417,47 @@ void CLuaUI::ShockFront(float power, const float3& pos, float areaOfEffect)
 }
 
 
+void CLuaUI::ExecuteUIEventBatch() {
+	if(!UseEventBatch()) return;
+
+	std::vector<LuaUIEvent> lleb;
+	{
+		GML_STDMUTEX_LOCK(llbatch);
+
+		if(luaUIEventBatch.empty())
+			return;
+
+		luaUIEventBatch.swap(lleb);
+	}
+
+#if defined(USE_GML) && GML_ENABLE_SIM
+	SELECT_LUA_STATE();
+#endif
+	GML_DRCMUTEX_LOCK(lua); // ExecuteUIEventBatch
+
+	if(Threading::IsSimThread())
+		Threading::SetBatchThread(false);
+	for(std::vector<LuaUIEvent>::iterator i = lleb.begin(); i != lleb.end(); ++i) {
+		LuaUIEvent &e = *i;
+		switch(e.id) {
+			case SHOCK_FRONT:
+				ShockFront(e.flt1, e.pos1, e.flt2, &e.flt3);
+				break;
+			default:
+				LOG_L(L_ERROR, "%s: Invalid Event %d", __FUNCTION__, e.id);
+				break;
+		}
+	}
+	if(Threading::IsSimThread())
+		Threading::SetBatchThread(true);
+}
+
 /******************************************************************************/
 
 bool CLuaUI::HasLayoutButtons()
 {
-	GML_RECMUTEX_LOCK(lua); // HasLayoutButtons
+	SELECT_LUA_STATE();
+	GML_DRCMUTEX_LOCK(lua); // HasLayoutButtons
 
 	lua_checkstack(L, 2);
 
@@ -490,6 +491,10 @@ bool CLuaUI::LayoutButtons(int& xButtons, int& yButtons,
 	buttonList.clear();
 	menuName = "";
 
+	GML_THRMUTEX_LOCK(unit, GML_DRAW); // LayoutButtons
+	GML_THRMUTEX_LOCK(feat, GML_DRAW); // LayoutButtons
+//	GML_THRMUTEX_LOCK(proj, GML_DRAW); // LayoutButtons
+
 	LUA_CALL_IN_CHECK(L);
 	lua_checkstack(L, 6);
 	const int top = lua_gettop(L);
@@ -523,20 +528,20 @@ bool CLuaUI::LayoutButtons(int& xButtons, int& yButtons,
 		return false; // no warnings
 	}
 	else if (args != 11) {
-		logOutput.Print("LayoutButtons() bad number of return values (%i)\n", args);
+		LOG_L(L_WARNING, "LayoutButtons() bad number of return values (%i)", args);
 		lua_settop(L, top);
 		return false;
 	}
 
 	if (!lua_isstring(L, 1)) {
-		logOutput.Print("LayoutButtons() bad menuName value\n");
+		LOG_L(L_WARNING, "LayoutButtons() bad menuName value");
 		lua_settop(L, top);
 		return false;
 	}
 	menuName = string(lua_tostring(L, 1), lua_strlen(L, 1));
 
 	if (!lua_isnumber(L, 2) || !lua_isnumber(L, 3)) {
-		logOutput.Print("LayoutButtons() bad xButtons or yButtons values\n");
+		LOG_L(L_WARNING, "LayoutButtons() bad xButtons or yButtons values");
 		lua_settop(L, top);
 		return false;
 	}
@@ -544,49 +549,49 @@ bool CLuaUI::LayoutButtons(int& xButtons, int& yButtons,
 	yButtons = lua_toint(L, 3);
 
 	if (!GetLuaIntList(L, 4, removeCmds)) {
-		logOutput.Print("LayoutButtons() bad removeCommands table\n");
+		LOG_L(L_WARNING, "LayoutButtons() bad removeCommands table");
 		lua_settop(L, top);
 		return false;
 	}
 
 	if (!GetLuaCmdDescList(L, 5, customCmds)) {
-		logOutput.Print("LayoutButtons() bad customCommands table\n");
+		LOG_L(L_WARNING, "LayoutButtons() bad customCommands table");
 		lua_settop(L, top);
 		return false;
 	}
 
 	if (!GetLuaIntList(L, 6, onlyTextureCmds)) {
-		logOutput.Print("LayoutButtons() bad onlyTexture table\n");
+		LOG_L(L_WARNING, "LayoutButtons() bad onlyTexture table");
 		lua_settop(L, top);
 		return false;
 	}
 
 	if (!GetLuaReStringList(L, 7, reTextureCmds)) {
-		logOutput.Print("LayoutButtons() bad reTexture table\n");
+		LOG_L(L_WARNING, "LayoutButtons() bad reTexture table");
 		lua_settop(L, top);
 		return false;
 	}
 
 	if (!GetLuaReStringList(L, 8, reNamedCmds)) {
-		logOutput.Print("LayoutButtons() bad reNamed table\n");
+		LOG_L(L_WARNING, "LayoutButtons() bad reNamed table");
 		lua_settop(L, top);
 		return false;
 	}
 
 	if (!GetLuaReStringList(L, 9, reTooltipCmds)) {
-		logOutput.Print("LayoutButtons() bad reTooltip table\n");
+		LOG_L(L_WARNING, "LayoutButtons() bad reTooltip table");
 		lua_settop(L, top);
 		return false;
 	}
 
 	if (!GetLuaReParamsList(L, 10, reParamsCmds)) {
-		logOutput.Print("LayoutButtons() bad reParams table\n");
+		LOG_L(L_WARNING, "LayoutButtons() bad reParams table");
 		lua_settop(L, top);
 		return false;
 	}
 
 	if (!GetLuaIntMap(L, 11, buttonList)) {
-		logOutput.Print("LayoutButtons() bad buttonList table\n");
+		LOG_L(L_WARNING, "LayoutButtons() bad buttonList table");
 		lua_settop(L, top);
 		return false;
 	}
@@ -605,29 +610,9 @@ bool CLuaUI::BuildCmdDescTable(lua_State* L,
 	const int cmdDescCount = (int)cmds.size();
 	for (int i = 0; i < cmdDescCount; i++) {
 		lua_pushnumber(L, i + CMD_INDEX_OFFSET);
-		lua_newtable(L); {
-			HSTR_PUSH_NUMBER(L, "id",       cmds[i].id);
-			HSTR_PUSH_NUMBER(L, "type",     cmds[i].type);
-			HSTR_PUSH_STRING(L, "name",     cmds[i].name);
-			HSTR_PUSH_STRING(L, "tooltip",  cmds[i].tooltip);
-			HSTR_PUSH_STRING(L, "action",   cmds[i].action);
-			HSTR_PUSH_STRING(L, "texture",  cmds[i].iconname);
-			HSTR_PUSH_BOOL(L,   "hidden",   cmds[i].hidden);
-			HSTR_PUSH_BOOL(L,   "disabled", cmds[i].disabled);
-			HSTR_PUSH_BOOL(L,"onlyTexture", cmds[i].onlyTexture);
-
-			HSTR_PUSH(L, "params");
-			lua_newtable(L);
-			for (unsigned int p = 0; p < cmds[i].params.size(); p++) {
-				lua_pushnumber(L, p + 1);
-				lua_pushstring(L, cmds[i].params[p].c_str());
-				lua_rawset(L, -3);
-			}
-			lua_rawset(L, -3);
-		}
+		LuaUtils::PushCommandDesc(L, cmds[i]);
 		lua_rawset(L, -3);
 	}
-	HSTR_PUSH_NUMBER(L, "n", cmdDescCount);
 	return true;
 }
 
@@ -768,14 +753,14 @@ bool CLuaUI::GetLuaCmdDescList(lua_State* L, int index,
 				} else if (key == "setext") {
 					cd.params.push_back(string("$se$") + value);
 				} else {
-					logOutput.Print("GetLuaCmdDescList() unknown key  (%s)\n", key.c_str());
+					LOG_L(L_WARNING, "GetLuaCmdDescList() unknown key (%s)", key.c_str());
 				}
 			}
 			else if (lua_israwstring(L, -2) && lua_istable(L, -1)) {
 				const string key = StringToLower(lua_tostring(L, -2));
 				if (key != "actions") {
-					logOutput.Print("GetLuaCmdDescList() non \"actions\" table: %s",
-					                key.c_str());
+					LOG_L(L_WARNING, "GetLuaCmdDescList() non \"actions\" table: %s",
+							key.c_str());
 					continue;
 				}
 				const int actionsTable = lua_gettop(L);
@@ -790,14 +775,14 @@ bool CLuaUI::GetLuaCmdDescList(lua_State* L, int index,
 				}
 			}
 			else {
-				logOutput.Print("GetLuaCmdDescList() bad entry\n");
+				LOG_L(L_WARNING, "GetLuaCmdDescList() bad entry");
 			}
 		}
 		cmdDescs.push_back(cd);
 	}
 
 	if (!lua_isnil(L, -1)) {
-		logOutput.Print("GetLuaCmdDescList() entry %i is not a table\n", paramIndex);
+		LOG_L(L_WARNING, "GetLuaCmdDescList() entry %i is not a table", paramIndex);
 		lua_pop(L, 1);
 		return false;
 	}
@@ -813,8 +798,17 @@ bool CLuaUI::GetLuaCmdDescList(lua_State* L, int index,
 // Custom Call-in
 //
 
-bool CLuaUI::HasUnsyncedXCall(const string& funcName)
+bool CLuaUI::HasUnsyncedXCall(lua_State* srcState, const string& funcName)
 {
+	SELECT_LUA_STATE();
+#if (LUA_MT_OPT & LUA_MUTEX)
+	if (srcState != L && SingleState()) {
+		GML_STDMUTEX_LOCK(xcall); // HasUnsyncedXCall
+
+		return unsyncedXCalls.find(funcName) != unsyncedXCalls.end();
+	}
+#endif
+
 	lua_getglobal(L, funcName.c_str());
 	const bool haveFunc = lua_isfunction(L, -1);
 	lua_pop(L, 1);
@@ -824,6 +818,42 @@ bool CLuaUI::HasUnsyncedXCall(const string& funcName)
 
 int CLuaUI::UnsyncedXCall(lua_State* srcState, const string& funcName)
 {
+#if (LUA_MT_OPT & LUA_MUTEX)
+	{
+		SELECT_UNSYNCED_LUA_STATE();
+		if (srcState != L) {
+			DelayDataDump ddmp;
+
+			LuaUtils::ShallowDataDump sdd;
+			sdd.type = LUA_TSTRING;
+
+			size_t len = funcName.length();
+			sdd.data.str = new std::string;
+			if (len > 0) {
+				sdd.data.str->resize(len);
+				memcpy(&(*sdd.data.str)[0], funcName.c_str(), len);
+			}
+
+			ddmp.data.push_back(sdd);
+
+			LuaUtils::Backup(ddmp.dump, srcState, lua_gettop(srcState));
+
+			lua_settop(srcState, 0);
+
+			GML_STDMUTEX_LOCK(scall);
+
+			delayedCallsFromSynced.push_back(DelayDataDump());
+
+			DelayDataDump &ddb = delayedCallsFromSynced.back();
+			ddb.data.swap(ddmp.data);
+			ddb.dump.swap(ddmp.dump);
+			ddb.xcall = true;
+
+			return 0;
+		}
+	}
+#endif
+
 	LUA_CALL_IN_CHECK(L);
 	const LuaHashString funcHash(funcName);
 	if (!funcHash.GetGlobalFunc(L)) {
@@ -890,6 +920,34 @@ int CLuaUI::SetShockFrontFactors(lua_State* L)
 	return 0;
 }
 
+int CLuaUI::UpdateUnsyncedXCalls(lua_State* L)
+{
+#if (LUA_MT_OPT & LUA_MUTEX)
+	if (!SingleState() || L != L_Sim)
+#endif
+		return 0;
+
+	GML_STDMUTEX_LOCK(xcall); // UpdateUnsyncedXCalls
+
+	unsyncedXCalls.clear();
+
+	for (lua_pushnil(L); lua_next(L, LUA_GLOBALSINDEX) != 0; lua_pop(L, 1)) {
+		const int ktype = lua_type(L, -2);
+		const int vtype = lua_type(L, -1);
+		if (ktype == LUA_TSTRING && vtype == LUA_TFUNCTION) {
+			size_t len = 0;
+			const char* data = lua_tolstring(L, -2, &len);
+			if (len > 0) {
+				std::string name;
+				name.resize(len);
+				memcpy(&name[0], data, len);
+				unsyncedXCalls.insert(name);
+			}
+		}
+	}
+
+	return 0;
+}
 
 /******************************************************************************/
 /******************************************************************************/
