@@ -1,34 +1,32 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include <fstream>
 #include <SDL_keysym.h>
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "Game/Camera/CameraController.h"
 #include "Game/Camera.h"
 #include "Game/CameraHandler.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/SelectedUnits.h"
-#include "Sim/Misc/TeamHandler.h"
-#include "LogOutput.h"
-#include "Map/Ground.h"
+#include "KeyBindings.h"
 #include "MouseHandler.h"
-#include "FileSystem/FileSystem.h"
 #include "SelectionKeyHandler.h"
+#include "Map/Ground.h"
 #include "Sim/Misc/CategoryHandler.h"
+#include "Sim/Misc/TeamHandler.h"
 #include "Sim/Units/CommandAI/CommandAI.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/UnitTypes/Building.h"
-#include "KeyBindings.h"
-#include "myMath.h"
+#include "System/Log/ILog.h"
+#include "System/myMath.h"
+#include "System/FileSystem/DataDirsAccess.h"
 #include <boost/cstdint.hpp>
 
 CSelectionKeyHandler* selectionKeys;
 
-extern boost::uint8_t *keys;
-
 CSelectionKeyHandler::CSelectionKeyHandler()
 {
 	LoadSelectionKeys();
@@ -40,7 +38,7 @@ CSelectionKeyHandler::~CSelectionKeyHandler()
 
 void CSelectionKeyHandler::LoadSelectionKeys()
 {
-	std::ifstream ifs(filesystem.LocateFile("selectkeys.txt").c_str());
+	std::ifstream ifs(dataDirsAccess.LocateFile("selectkeys.txt").c_str());
 
 	selectNumber = 0;
 
@@ -56,7 +54,7 @@ void CSelectionKeyHandler::LoadSelectionKeys()
 		bool shift=false;
 		bool control=false;
 		bool alt=false;
-		unsigned char keyname;
+		unsigned char keyname=0;
 
 		while(true){
 			std::string s=ReadToken(key);
@@ -91,46 +89,33 @@ void CSelectionKeyHandler::LoadSelectionKeys()
 		}
 		if ( keybindstring.size() != 0 ) keybindstring += "+";
 		keybindstring += keyname;
-		keyBindings->Command( "bind " + keybindstring + " select " + sel );
+		keyBindings->ExecuteCommand( "bind " + keybindstring + " select " + sel );
 	}
 }
 
-std::string CSelectionKeyHandler::ReadToken(std::string& s)
+std::string CSelectionKeyHandler::ReadToken(std::string& str)
 {
 	std::string ret;
-	// change made.  avoiding repeated substr calls...
-/*	int index = 0;
-	//char c=s[index];
 
-	while ( index < s.length() && s[index] != '_' && s[index++] != '+' );
-
-	ret = s.substr(0, index);
-	if ( index == s.length() )
-		s.clear();
-	else
-		s = s.substr(index, std::string::npos);
-*/
-	if (s.empty())
-		return std::string();
-
-	char c=s[0];
-	while(c && c!='_' && c!='+'){
-		s=s.substr(1,std::string::npos);
-		ret+=c;
-		c=s[0];
+	size_t index = 0;
+	while ((index < str.length()) && (str[index] != '_') && (str[index] != '+')) {
+		index++;
 	}
 
+	ret = str.substr(0, index);
+	str = str.substr(index, std::string::npos);
+
 	return ret;
 }
 
 
-std::string CSelectionKeyHandler::ReadDelimiter(std::string& s)
+std::string CSelectionKeyHandler::ReadDelimiter(std::string& str)
 {
-	std::string ret = s.substr(0, 1);
-	if (s.size() >= 1) {
-		s = s.substr(1, std::string::npos);
+	std::string ret = str.substr(0, 1);
+	if (str.size() >= 1) {
+		str = str.substr(1, std::string::npos);
 	} else {
-		s = "";
+		str = "";
 	}
 	return ret;
 }
@@ -157,8 +142,10 @@ namespace
 			assert(false);
 		}
 
-		/// Actual filtering, should return false if unit should be removed
-		/// from proposed selection.
+		/**
+		 * Actual filtering, should return false if unit should be removed
+		 * from proposed selection.
+		 */
 		virtual bool ShouldIncludeUnit(const CUnit* unit) const = 0;
 
 		/// Number of arguments this filter has.
@@ -173,15 +160,15 @@ namespace
 	// prototype / factory based approach might be better at some point?
 	// for now these singleton filters seem ok. (they are not reentrant tho!)
 
-#define DECLARE_FILTER_EX(name, args, condition, extra) \
+#define DECLARE_FILTER_EX(name, args, condition, extra, init) \
 	struct name ## _Filter : public Filter { \
-		name ## _Filter() : Filter(#name, args) {} \
+		name ## _Filter() : Filter(#name, args) { init; } \
 		bool ShouldIncludeUnit(const CUnit* unit) const { return condition; } \
 		extra \
 	} name ## _filter_instance; \
 
 #define DECLARE_FILTER(name, condition) \
-	DECLARE_FILTER_EX(name, 0, condition, )
+	DECLARE_FILTER_EX(name, 0, condition, ,)
 
 	DECLARE_FILTER(Builder, unit->unitDef->buildSpeed > 0);
 	DECLARE_FILTER(Building, dynamic_cast<const CBuilding*>(unit) != NULL);
@@ -191,7 +178,7 @@ namespace
 	DECLARE_FILTER(Weapons, !unit->weapons.empty());
 	DECLARE_FILTER(Idle, unit->commandAI->commandQue.empty());
 	DECLARE_FILTER(Waiting, !unit->commandAI->commandQue.empty() &&
-	               (unit->commandAI->commandQue.front().id == CMD_WAIT));
+	               (unit->commandAI->commandQue.front().GetID() == CMD_WAIT));
 	DECLARE_FILTER(InHotkeyGroup, unit->group != NULL);
 	DECLARE_FILTER(Radar, unit->radarRadius || unit->sonarRadius || unit->jammerRadius);
 
@@ -199,21 +186,24 @@ namespace
 		float minRange;
 		void SetParam(int index, const std::string& value) {
 			minRange = atof(value.c_str());
-		}
+		},
+		minRange=0.0f;
 	);
 
 	DECLARE_FILTER_EX(AbsoluteHealth, 1, unit->health > minHealth,
 		float minHealth;
 		void SetParam(int index, const std::string& value) {
 			minHealth = atof(value.c_str());
-		}
+		},
+		minHealth=0.0f;
 	);
 
 	DECLARE_FILTER_EX(RelativeHealth, 1, unit->health / unit->maxHealth > minHealth,
 		float minHealth;
 		void SetParam(int index, const std::string& value) {
 			minHealth = atof(value.c_str()) * 0.01f; // convert from percent
-		}
+		},
+		minHealth=0.0f;
 	);
 
 	DECLARE_FILTER_EX(InPrevSel, 0, prevTypes.find(unit->unitDef->id) != prevTypes.end(),
@@ -224,21 +214,22 @@ namespace
 			for (CUnitSet::const_iterator si = tu.begin(); si != tu.end(); ++si) {
 				prevTypes.insert((*si)->unitDef->id);
 			}
-		}
+		},
 	);
 
 	DECLARE_FILTER_EX(NameContain, 1, unit->unitDef->humanName.find(name) != std::string::npos,
 		std::string name;
 		void SetParam(int index, const std::string& value) {
 			name = value;
-		}
+		},
 	);
 
 	DECLARE_FILTER_EX(Category, 1, unit->category == cat,
 		unsigned int cat;
 		void SetParam(int index, const std::string& value) {
 			cat = CCategoryHandler::Instance()->GetCategory(value);
-		}
+		},
+		cat=0;
 	);
 
 	DECLARE_FILTER_EX(RulesParamEquals, 2, unit->modParamsMap.find(param) != unit->modParamsMap.end() &&
@@ -250,7 +241,8 @@ namespace
 				case 0: param = value; break;
 				case 1: wantedValue = atof(value.c_str()); break;
 			}
-		}
+		},
+		wantedValue=0.0f;
 	);
 
 #undef DECLARE_FILTER_EX
@@ -309,7 +301,7 @@ void CSelectionKeyHandler::DoSelection(std::string selectString)
 		ReadDelimiter(selectString);
 		float maxDist=atof(ReadToken(selectString).c_str());
 
-		float dist=ground->LineGroundCol(camera->pos,camera->pos+mouse->dir*8000);
+		float dist = ground->LineGroundCol(camera->pos, camera->pos + mouse->dir * 8000, false);
 		float3 mp=camera->pos+mouse->dir*dist;
 		if (cylindrical) {
 			mp.y = 0;
@@ -346,7 +338,7 @@ void CSelectionKeyHandler::DoSelection(std::string selectString)
 			selection.push_back(*ui);
 		}
 	} else {
-		logOutput.Print("Unknown source token %s",s.c_str());
+		LOG_L(L_WARNING, "Unknown source token %s", s.c_str());
 		return;
 	}
 
@@ -388,7 +380,7 @@ void CSelectionKeyHandler::DoSelection(std::string selectString)
 			}
 		}
 		else {
-			logOutput.Print("Unknown token in filter %s",s.c_str());
+			LOG_L(L_WARNING, "Unknown token in filter %s", s.c_str());
 			return;
 		}
 	}
@@ -479,6 +471,6 @@ void CSelectionKeyHandler::DoSelection(std::string selectString)
 
 		selectNumber+=num;
 	} else {
-		logOutput.Print("Unknown token in conclusion %s",s.c_str());
+		LOG_L(L_WARNING, "Unknown token in conclusion %s", s.c_str());
 	}
 }
