@@ -1,31 +1,30 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include <SDL_timer.h>
 #include "WaitCommandsAI.h"
 #include "SelectedUnits.h"
-#include "Sim/Misc/TeamHandler.h"
 #include "GameHelper.h"
+#include "GlobalUnsynced.h"
+#include "UI/CommandColors.h"
+#include "UI/CursorIcons.h"
+#include "Rendering/LineDrawer.h"
 #include "Sim/Misc/QuadField.h"
 #include "Sim/Misc/GlobalConstants.h"
+#include "Sim/Misc/GlobalSynced.h"
+#include "Sim/Misc/TeamHandler.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/CommandAI/CommandQueue.h"
 #include "Sim/Units/CommandAI/CommandAI.h"
 #include "Sim/Units/CommandAI/FactoryCAI.h"
-#include "Sim/Units/CommandAI/LineDrawer.h"
 #include "Sim/Units/UnitTypes/Factory.h"
-#include "Sim/Misc/GlobalSynced.h"
-#include "GlobalUnsynced.h"
-#include "Object.h"
-#include "UI/CommandColors.h"
-#include "UI/CursorIcons.h"
-#include "creg/STL_Map.h"
-#include "creg/STL_List.h"
-#include "creg/STL_Set.h"
-#include "Util.h"
+#include "System/Object.h"
+#include "System/Util.h"
+#include "System/creg/STL_Map.h"
+#include "System/creg/STL_List.h"
+#include "System/creg/STL_Set.h"
 #include <assert.h>
 
 CWaitCommandsAI waitCommandsAI;
@@ -127,7 +126,7 @@ void CWaitCommandsAI::Update()
 	while (it != waitMap.end()) {
 		// grab an incremented iterator in case the active iterator deletes itself
 		WaitMap::iterator tmp = it;
-		tmp++;
+		++tmp;
 		it->second->Update();
 		it = tmp;
 	}
@@ -137,7 +136,7 @@ void CWaitCommandsAI::Update()
 	it = unackedMap.begin();
 	while (it != unackedMap.end()) {
 		WaitMap::iterator tmp = it;
-		tmp++;
+		++tmp;
 		Wait* wait = it->second;
 		if (wait->GetDeadTime() < nowTime) {
 			delete wait;
@@ -195,7 +194,7 @@ void CWaitCommandsAI::AddGatherWait(const Command& cmd)
 
 void CWaitCommandsAI::AcknowledgeCommand(const Command& cmd)
 {
-	if ((cmd.id != CMD_WAIT) || (cmd.params.size() != 2)) {
+	if ((cmd.GetID() != CMD_WAIT) || (cmd.params.size() != 2)) {
 		return;
 	}
 	const KeyType key = Wait::GetKeyFromFloat(cmd.params[1]);
@@ -224,7 +223,7 @@ void CWaitCommandsAI::AddLocalUnit(CUnit* unit, const CUnit* builder)
 	CCommandQueue::const_iterator qit;
 	for (qit = dq.begin(); qit != dq.end(); ++qit) {
 		const Command& cmd = *qit;
-		if ((cmd.id != CMD_WAIT) || (cmd.params.size() != 2)) {
+		if ((cmd.GetID() != CMD_WAIT) || (cmd.params.size() != 2)) {
 			continue;
 		}
 
@@ -285,7 +284,7 @@ void CWaitCommandsAI::ClearUnitQueue(CUnit* unit, const CCommandQueue& queue)
 	CCommandQueue::const_iterator qit;
 	for (qit = queue.begin(); qit != queue.end(); ++qit) {
 		const Command& cmd = *qit;
-		if ((cmd.id == CMD_WAIT) && (cmd.params.size() == 2)) {
+		if ((cmd.GetID() == CMD_WAIT) && (cmd.params.size() == 2)) {
 			const KeyType key = Wait::GetKeyFromFloat(cmd.params[1]);
 			WaitMap::iterator wit = waitMap.find(key);
 			if (wit != waitMap.end()) {
@@ -311,19 +310,10 @@ bool CWaitCommandsAI::InsertWaitObject(Wait* wait)
 
 void CWaitCommandsAI::RemoveWaitObject(Wait* wait)
 {
-	WaitMap::iterator it;
-
-	it = waitMap.find(wait->GetKey());
-	if (it != waitMap.end()) {
-		 waitMap.erase(it);
+	if (waitMap.erase(wait->GetKey()))
 		 return;
-	}
-
-	it = unackedMap.find(wait->GetKey());
-	if (it != unackedMap.end()) {
-		 unackedMap.erase(it);
+	if (unackedMap.erase(wait->GetKey()))
 		 return;
-	}
 }
 
 
@@ -351,7 +341,7 @@ void CWaitCommandsAI::AddIcon(const Command& cmd, const float3& pos) const
 	}
 	else if (code == CMD_WAITCODE_DEATHWAIT) {
 		lineDrawer.DrawIconAtLastPos(CMD_DEATHWAIT);
-    it->second->AddUnitPosition(pos);
+		it->second->AddUnitPosition(pos);
 	}
 	else if (code == CMD_WAITCODE_GATHERWAIT) {
 		lineDrawer.DrawIconAtLastPos(CMD_GATHERWAIT);
@@ -424,17 +414,17 @@ CWaitCommandsAI::Wait::WaitState
 		return Missing;
 	}
 	const Command& cmd = dq.front();
-	if ((cmd.id == CMD_WAIT) && (cmd.params.size() == 2) &&
+	if ((cmd.GetID() == CMD_WAIT) && (cmd.params.size() == 2) &&
 			(cmd.params[0] == code) &&
 			(GetKeyFromFloat(cmd.params[1]) == key)) {
 		return Active;
 	}
 
 	CCommandQueue::const_iterator it = dq.begin();
-	it++;
+	++it;
 	for ( ; it != dq.end(); ++it) {
 		const Command& qcmd = *it;
-		if ((qcmd.id == CMD_WAIT) && (qcmd.params.size() == 2) &&
+		if ((qcmd.GetID() == CMD_WAIT) && (qcmd.params.size() == 2) &&
 				(qcmd.params[0] == code) &&
 				(GetKeyFromFloat(qcmd.params[1]) == key)) {
 			return Queued;
@@ -451,7 +441,7 @@ bool CWaitCommandsAI::Wait::IsWaitingOn(const CUnit* unit) const
 		return false;
 	}
 	const Command& cmd = dq.front();
-	if ((cmd.id == CMD_WAIT) && (cmd.params.size() == 2) &&
+	if ((cmd.GetID() == CMD_WAIT) && (cmd.params.size() == 2) &&
 			(cmd.params[0] == code) &&
 			(GetKeyFromFloat(cmd.params[1]) == key)) {
 		return true;
@@ -494,8 +484,7 @@ void CWaitCommandsAI::Wait::SendCommand(const Command& cmd,
 
 void CWaitCommandsAI::Wait::SendWaitCommand(const CUnitSet& unitSet)
 {
-	Command waitCmd;
-	waitCmd.id = CMD_WAIT;
+	Command waitCmd(CMD_WAIT);
 	SendCommand(waitCmd, unitSet);
 }
 
@@ -554,9 +543,7 @@ CWaitCommandsAI::TimeWait::TimeWait(const Command& cmd, CUnit* _unit)
 	duration = GAME_SPEED * (int)cmd.params[0];
 	factory = (dynamic_cast<CFactory*>(unit) != NULL);
 
-	Command waitCmd;
-	waitCmd.id = CMD_WAIT;
-	waitCmd.options = cmd.options;
+	Command waitCmd(CMD_WAIT, cmd.options);
 	waitCmd.params.push_back(code);
 	waitCmd.params.push_back(GetFloatFromKey(key));
 
@@ -564,7 +551,7 @@ CWaitCommandsAI::TimeWait::TimeWait(const Command& cmd, CUnit* _unit)
 	selectedUnits.AddUnit(unit);
 	selectedUnits.GiveCommand(waitCmd);
 
-	AddDeathDependence((CObject*)unit);
+	AddDeathDependence((CObject*)unit, DEPENDENCE_WAITCMD);
 
 	return;
 }
@@ -582,7 +569,7 @@ CWaitCommandsAI::TimeWait::TimeWait(int _duration, CUnit* _unit)
 	duration = _duration;
 	factory = false;
 
-	AddDeathDependence((CObject*)unit);
+	AddDeathDependence((CObject*)unit, DEPENDENCE_WAITCMD);
 }
 
 
@@ -737,19 +724,17 @@ CWaitCommandsAI::DeathWait::DeathWait(const Command& cmd)
 
 	waitUnits = selUnits;
 
-	Command waitCmd;
-	waitCmd.id = CMD_WAIT;
-	waitCmd.options = cmd.options;
+	Command waitCmd(CMD_WAIT, cmd.options);
 	waitCmd.params.push_back(code);
 	waitCmd.params.push_back(GetFloatFromKey(key));
 	selectedUnits.GiveCommand(waitCmd);
 
 	CUnitSet::iterator it;
 	for (it = waitUnits.begin(); it != waitUnits.end(); ++it) {
-		AddDeathDependence((CObject*)(*it));
+		AddDeathDependence((CObject*)(*it), DEPENDENCE_WAITCMD);
 	}
 	for (it = deathUnits.begin(); it != deathUnits.end(); ++it) {
-		AddDeathDependence((CObject*)(*it));
+		AddDeathDependence((CObject*)(*it), DEPENDENCE_WAITCMD);
 	}
 
 	return;
@@ -764,37 +749,26 @@ CWaitCommandsAI::DeathWait::~DeathWait()
 
 void CWaitCommandsAI::DeathWait::DependentDied(CObject* object)
 {
-	CUnit* unit = (CUnit*)object;
+	waitUnits.erase((CUnit*)object);
 
-	CUnitSet::iterator wit = waitUnits.find(unit);
-	if (wit != waitUnits.end()) {
-		waitUnits.erase(wit);
-	}
-	if (waitUnits.empty()) {
+	if (waitUnits.empty())
 		return;
-	}
 
-	CUnitSet::iterator dit = deathUnits.find(unit);
-	if (dit != deathUnits.end()) {
-		deathUnits.erase(dit);
-	}
+	deathUnits.erase((CUnit*)object);
 }
 
 
 void CWaitCommandsAI::DeathWait::AddUnit(CUnit* unit)
 {
-	waitUnits.insert(unit);
-	AddDeathDependence((CObject*)unit);
+	if (waitUnits.insert(unit).second)
+		AddDeathDependence((CObject*)unit, DEPENDENCE_WAITCMD);
 }
 
 
 void CWaitCommandsAI::DeathWait::RemoveUnit(CUnit* unit)
 {
-	CUnitSet::iterator it = waitUnits.find(unit);
-	if (it != waitUnits.end()) {
-		DeleteDeathDependence(*it);
-		waitUnits.erase(it);
-	}
+	if (waitUnits.erase(unit))
+		DeleteDeathDependence(unit, DEPENDENCE_WAITCMD);
 }
 
 
@@ -805,6 +779,8 @@ void CWaitCommandsAI::DeathWait::Update()
 		return;
 	}
 
+	unitPos.clear();
+
 	if (!deathUnits.empty()) {
 		return; // more must die
 	}
@@ -815,7 +791,7 @@ void CWaitCommandsAI::DeathWait::Update()
 		WaitState state = GetWaitState(*it);
 		if (state == Active) {
 			unblockSet.insert(*it);
-			DeleteDeathDependence(*it);
+			DeleteDeathDependence(*it, DEPENDENCE_WAITCMD);
 			it = RemoveUnitFromSet(it, waitUnits);
 			continue;
 		}
@@ -823,7 +799,7 @@ void CWaitCommandsAI::DeathWait::Update()
 			// do nothing
 		}
 		else if (state == Missing) {
-			DeleteDeathDependence(*it);
+			DeleteDeathDependence(*it, DEPENDENCE_WAITCMD);
 			it = RemoveUnitFromSet(it, waitUnits);
 			continue;
 		}
@@ -839,45 +815,40 @@ void CWaitCommandsAI::DeathWait::Update()
 
 void CWaitCommandsAI::DeathWait::Draw() const
 {
-	CUnitSet drawSet;
-	CUnitSet::const_iterator it;
-
 	if (unitPos.empty()) {
 		return;
 	}
 
 	float3 midPos(0.0f, 0.0f, 0.0f);
-	for (int i = 0; i < (int)unitPos.size(); i++) {
-		midPos += unitPos[i];
+	for (size_t i = 0; i < unitPos.size(); i++) {
+		midPos += unitPos.at(i);
 	}
 	midPos /= (float)unitPos.size();
 
 	cursorIcons.AddIcon(CMD_DEATHWAIT, midPos);
 
-	for (int i = 0; i < (int)unitPos.size(); i++) {
-		lineDrawer.StartPath(unitPos[i], cmdColors.start);
+	for (size_t i = 0; i < unitPos.size(); i++) {
+		lineDrawer.StartPath(unitPos.at(i), cmdColors.start);
 		lineDrawer.DrawLine(midPos, cmdColors.deathWait);
 		lineDrawer.FinishPath();
 	}
 
+	CUnitSet::const_iterator it;
 	for (it = deathUnits.begin(); it != deathUnits.end(); ++it) {
 		const CUnit* unit = *it;
 		if (unit->losStatus[gu->myAllyTeam] & (LOS_INLOS | LOS_INRADAR)) {
-			cursorIcons.AddIcon(CMD_SELFD, (*it)->midPos);
+			cursorIcons.AddIcon(CMD_SELFD, unit->midPos);
 			lineDrawer.StartPath(midPos, cmdColors.start);
 			lineDrawer.DrawLine(unit->midPos, cmdColors.deathWait);
 			lineDrawer.FinishPath();
 		}
 	}
-
-	unitPos.clear();
 }
 
 
-void CWaitCommandsAI::DeathWait::AddUnitPosition(const float3& pos) const
+void CWaitCommandsAI::DeathWait::AddUnitPosition(const float3& pos)
 {
 	unitPos.push_back(pos);
-	return;
 }
 
 
@@ -889,7 +860,7 @@ void CWaitCommandsAI::DeathWait::SelectAreaUnits(
 	const float3 mins(std::min(pos0.x, pos1.x), 0.0f, std::min(pos0.z, pos1.z));
 	const float3 maxs(std::max(pos0.x, pos1.x), 0.0f, std::max(pos0.z, pos1.z));
 
-	std::vector<CUnit*> tmpUnits = qf->GetUnitsExact(mins, maxs);
+	const std::vector<CUnit*> &tmpUnits = qf->GetUnitsExact(mins, maxs);
 
 	const int count = (int)tmpUnits.size();
 	for (int i = 0; i < count; i++) {
@@ -953,9 +924,7 @@ CWaitCommandsAI::SquadWait::SquadWait(const Command& cmd)
 	valid = true;
 	key = GetNewKey();
 
-	Command waitCmd;
-	waitCmd.id = CMD_WAIT;
-	waitCmd.options = cmd.options;
+	Command waitCmd(CMD_WAIT, cmd.options);
 	waitCmd.params.push_back(code);
 	waitCmd.params.push_back(GetFloatFromKey(key));
 
@@ -963,10 +932,10 @@ CWaitCommandsAI::SquadWait::SquadWait(const Command& cmd)
 	SendCommand(waitCmd, waitUnits);
 
 	for (it = buildUnits.begin(); it != buildUnits.end(); ++it) {
-		AddDeathDependence((CObject*)(*it));
+		AddDeathDependence((CObject*)(*it), DEPENDENCE_WAITCMD);
 	}
 	for (it = waitUnits.begin(); it != waitUnits.end(); ++it) {
-		AddDeathDependence((CObject*)(*it));
+		AddDeathDependence((CObject*)(*it), DEPENDENCE_WAITCMD);
 	}
 
 	UpdateText();
@@ -983,38 +952,24 @@ CWaitCommandsAI::SquadWait::~SquadWait()
 
 void CWaitCommandsAI::SquadWait::DependentDied(CObject* object)
 {
-	CUnit* unit = (CUnit*)object;
-
-	CUnitSet::iterator bit = buildUnits.find(unit);
-	if (bit != buildUnits.end()) {
-		buildUnits.erase(bit);
-	}
-	CUnitSet::iterator wit = waitUnits.find(unit);
-	if (wit != waitUnits.end()) {
-		waitUnits.erase(wit);
-	}
+	buildUnits.erase((CUnit*)object);
+	waitUnits.erase((CUnit*)object);
 }
 
 
 void CWaitCommandsAI::SquadWait::AddUnit(CUnit* unit)
 {
-	waitUnits.insert(unit);
-	AddDeathDependence((CObject*)unit);
+	if (waitUnits.insert(unit).second)
+		AddDeathDependence((CObject*)unit, DEPENDENCE_WAITCMD);
 }
 
 
 void CWaitCommandsAI::SquadWait::RemoveUnit(CUnit* unit)
 {
-	CUnitSet::iterator bit = buildUnits.find(unit);
-	if (bit != buildUnits.end()) {
-		DeleteDeathDependence(*bit);
-		buildUnits.erase(bit);
-	}
-	CUnitSet::iterator wit = waitUnits.find(unit);
-	if (wit != waitUnits.end()) {
-		DeleteDeathDependence(*wit);
-		waitUnits.erase(wit);
-	}
+	if (buildUnits.erase(unit))
+		DeleteDeathDependence(unit, DEPENDENCE_WAITCMD);
+	if (waitUnits.erase(unit))
+		DeleteDeathDependence(unit, DEPENDENCE_WAITCMD);
 }
 
 
@@ -1041,7 +996,7 @@ void CWaitCommandsAI::SquadWait::Update()
 				// do nothing
 			}
 			else if (state == Missing) {
-				DeleteDeathDependence(*it);
+				DeleteDeathDependence(*it, DEPENDENCE_WAITCMD);
 				it = RemoveUnitFromSet(it, waitUnits);
 				continue;
 			}
@@ -1053,8 +1008,8 @@ void CWaitCommandsAI::SquadWait::Update()
 			//          that formations are created?
 			SendWaitCommand(unblockSet);
 			for (it = unblockSet.begin(); it != unblockSet.end(); ++it) {
-				DeleteDeathDependence(*it);
-				waitUnits.erase(*it);
+				if (waitUnits.erase(*it))
+					DeleteDeathDependence(*it, DEPENDENCE_WAITCMD);
 			}
 		}
 	}
@@ -1100,7 +1055,7 @@ CWaitCommandsAI::GatherWait::GatherWait(const Command& cmd)
 {
 	GML_RECMUTEX_LOCK(sel); // GatherWait
 
-	if (cmd.params.size() != 0) {
+	if (!cmd.params.empty()) {
 		return;
 	}
 
@@ -1122,16 +1077,14 @@ CWaitCommandsAI::GatherWait::GatherWait(const Command& cmd)
 	valid = true;
 	key = GetNewKey();
 
-	Command waitCmd;
-	waitCmd.id = CMD_WAIT;
-	waitCmd.options = SHIFT_KEY;
+	Command waitCmd(CMD_WAIT, SHIFT_KEY);
 	waitCmd.params.push_back(code);
 	waitCmd.params.push_back(GetFloatFromKey(key));
 	selectedUnits.GiveCommand(waitCmd, true);
 
 	CUnitSet::iterator wit;
 	for (wit = waitUnits.begin(); wit != waitUnits.end(); ++wit) {
-		AddDeathDependence((CObject*)(*wit));
+		AddDeathDependence((CObject*)(*wit), DEPENDENCE_WAITCMD);
 	}
 
 	return;
@@ -1146,12 +1099,7 @@ CWaitCommandsAI::GatherWait::~GatherWait()
 
 void CWaitCommandsAI::GatherWait::DependentDied(CObject* object)
 {
-	CUnit* unit = (CUnit*)object;
-
-	CUnitSet::iterator it = waitUnits.find(unit);
-	if (it != waitUnits.end()) {
-		waitUnits.erase(it);
-	}
+	waitUnits.erase((CUnit*)object);
 }
 
 
@@ -1163,11 +1111,8 @@ void CWaitCommandsAI::GatherWait::AddUnit(CUnit* unit)
 
 void CWaitCommandsAI::GatherWait::RemoveUnit(CUnit* unit)
 {
-	CUnitSet::iterator it = waitUnits.find(unit);
-	if (it != waitUnits.end()) {
-		DeleteDeathDependence(*it);
-		waitUnits.erase(it);
-	}
+	if (waitUnits.erase(unit))
+		DeleteDeathDependence(unit, DEPENDENCE_WAITCMD);
 }
 
 
@@ -1188,7 +1133,7 @@ void CWaitCommandsAI::GatherWait::Update()
 			return;
 		}
 		else if (state == Missing) {
-			DeleteDeathDependence(*it);
+			DeleteDeathDependence(*it, DEPENDENCE_WAITCMD);
 			it = RemoveUnitFromSet(it, waitUnits);
 			if (waitUnits.empty()) {
 				delete this;
