@@ -1,13 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 
-#include "TdfParser.h"
-
-#include <cassert>
-#include <deque>
-#include <set>
-#include <algorithm>
+#include "System/TdfParser.h"
 
 #include "TerrainBase.h"
 #include "Terrain.h"
@@ -16,15 +10,20 @@
 #include "TerrainTexEnvCombine.h"
 #include "TerrainTextureGLSL.h"
 #include "Lightcalc.h"
-#include "Util.h"
+#include "System/Util.h"
+#include "System/Log/ILog.h"
+
+#include <cassert>
+#include <deque>
+#include <map>
+#include <vector>
+#include <algorithm>
 
 #include <SDL_keysym.h>
-extern unsigned char *keys;
+extern unsigned char* keys;
 
 namespace terrain {
 
-	using namespace std;
-
 //-----------------------------------------------------------------------
 // RenderSetup
 //-----------------------------------------------------------------------
@@ -36,15 +35,15 @@ namespace terrain {
 
 	void RenderSetup::Clear()
 	{
-		for (size_t a=0;a<passes.size();a++)
+		for (size_t a = 0; a < passes.size(); a++)
 			delete passes[a].shaderSetup;
 		passes.clear();
 	}
 
 	void RenderSetup::DebugOutput()
 	{
-		for (size_t a=0; a < passes.size(); a++) {
-			const char *str=0;
+		for (size_t a = 0; a < passes.size(); a++) {
+			const char* str = 0;
 			RenderPass& p = passes[a];
 
 			if (p.operation == Pass_Mul)
@@ -56,10 +55,12 @@ namespace terrain {
 			if (p.operation == Pass_Replace)
 				str="Pass_Replace";
 
-			string shaderstr = p.shaderSetup ? p.shaderSetup->GetDebugDesc () : string("none");
-			d_trace ("Pass ("_STPF_"): %s, %s. Shader: %s\n", a, str, p.invertAlpha ? "invertalpha" : "", shaderstr.c_str());
+			std::string shaderstr = p.shaderSetup ? p.shaderSetup->GetDebugDesc() : std::string("none");
+			LOG_L(L_DEBUG, "Pass ("_STPF_"): %s, %s. Shader: %s",
+					a, str, (p.invertAlpha ? "invertalpha" : ""),
+					shaderstr.c_str());
 		}
-		d_trace ("\n");
+		LOG_L(L_DEBUG, "Shader passes finnished.");
 	}
 
 //-----------------------------------------------------------------------
@@ -68,7 +69,7 @@ namespace terrain {
 
 	RenderSetupCollection::~RenderSetupCollection()
 	{
-		for (size_t a=0;a<renderSetup.size();a++)
+		for (size_t a = 0; a < renderSetup.size(); a++)
 			delete renderSetup[a];
 	}
 
@@ -77,7 +78,7 @@ namespace terrain {
 //-----------------------------------------------------------------------
 
 
-	TerrainTexture::TerrainTexture ()
+	TerrainTexture::TerrainTexture()
 	{
 		tdfParser = 0;
 		heightmapW = 0;
@@ -89,63 +90,65 @@ namespace terrain {
 		lightmap = 0;
 		flatBumpmap = 0;
 		shadowMapParams = 0;
+		optimizeEpsilon = 0.0f;
+		useBumpMaps = false;
 	}
 
-	TerrainTexture::~TerrainTexture ()
+	TerrainTexture::~TerrainTexture()
 	{
 		delete shadowMapParams;
 		delete lightmap;
-		for (size_t a=0;a<texNodeSetup.size();a++)
+		for (size_t a = 0; a < texNodeSetup.size(); a++)
 			delete texNodeSetup[a];
 
 		delete shaderHandler;
 		texNodeSetup.clear();
 
 		// free all blendmaps
-		for (size_t a=0;a<blendMaps.size();a++)
+		for (size_t a = 0; a < blendMaps.size(); a++)
 			delete blendMaps[a];
 		blendMaps.clear();
 		// free all textures
-		for (size_t a=0;a<textures.size();a++)
+		for (size_t a = 0; a < textures.size(); a++)
 			delete textures[a];
 		textures.clear();
 	}
 
 	struct TerrainTexture::GenerateInfo
 	{
-		deque<AlphaImage*>* bmMipmaps;
-		vector<int> testlod; // the level
+		std::deque<AlphaImage*>* bmMipmaps;
+		std::vector<int> testlod; // the level
 
 		// set of texture node setup objects, sorted by blend sort key
-		map<uint, RenderSetupCollection*> nodesetup;
+		std::map<uint, RenderSetupCollection*> nodesetup;
 	};
 
 	BaseTexture* TerrainTexture::LoadImageSource(
 		const std::string& name,
 		const std::string& basepath,
-		Heightmap *heightmap, ILoadCallback *cb,
+		Heightmap* heightmap, ILoadCallback* cb,
 		Config* cfg, bool isBumpmap)
 	{
-		BaseTexture *btex = 0;
+		BaseTexture* btex = NULL;
 		if (!!atoi(tdfParser->SGetValueDef("0", basepath + name + "\\Blendmap").c_str()))
 		{
 			if (isBumpmap)
 				throw content_error(name + " should be a bumpmap, not a blendmap.");
 
-			Blendmap *bm = new Blendmap;
-			bm->Load (name, basepath + name, heightmap, cb, tdfParser);
-			blendMaps.push_back (bm);
+			Blendmap* bm = new Blendmap;
+			bm->Load(name, basepath + name, heightmap, cb, tdfParser);
+			blendMaps.push_back(bm);
 			btex = bm;
 		}
 		else // a regular texturemap?
 		{
-			TiledTexture *tex = new TiledTexture;
-			tex->Load (name, basepath + name, cb, tdfParser, isBumpmap);
+			TiledTexture* tex = new TiledTexture;
+			tex->Load(name, basepath + name, cb, tdfParser, isBumpmap);
 			if (!tex->id) {
 				delete tex;
 				return 0;
 			}
-			textures.push_back (tex);
+			textures.push_back(tex);
 			btex = tex;
 		}
 
@@ -156,16 +159,18 @@ namespace terrain {
 		return btex;
 	}
 
-	void TerrainTexture::Load (const TdfParser *tdf, Heightmap *heightmap, TQuad *quadtree, const vector<QuadMap*>& qmaps, Config *cfg, ILoadCallback *cb, LightingInfo *li)
+	void TerrainTexture::Load(const TdfParser* tdf, Heightmap* heightmap,
+			TQuad* quadtree, const std::vector<QuadMap*>& qmaps, Config* cfg,
+			ILoadCallback* cb, LightingInfo* li)
 	{
-		string basepath="MAP\\TERRAIN\\";
+		std::string basepath = "MAP\\TERRAIN\\";
 
-		if (cb) cb->PrintMsg ("  parsing texture stages...");
+		if (cb) cb->PrintMsg("  parsing texture stages...");
 
 		if (!GLEW_ARB_multitexture)
-			throw std::runtime_error ("No multitexture available");
+			throw std::runtime_error("No multitexture available");
 		if (!GLEW_ARB_texture_env_combine)
-			throw std::runtime_error ("Texture env combine extension not available");
+			throw std::runtime_error("Texture env combine extension not available");
 
 		heightmapW = heightmap->w;
 		tdfParser = tdf;
@@ -173,24 +178,24 @@ namespace terrain {
 		shaderDef.Parse(*tdf, cfg->useBumpMaps);
 		shaderDef.useShadowMapping = cfg->useShadowMaps;
 
-		optimizeEpsilon = atof(tdf->SGetValueDef ("0.04", basepath + "LayerOptimizeConst").c_str());
+		optimizeEpsilon = atof(tdf->SGetValueDef("0.04", basepath + "LayerOptimizeConst").c_str());
 		if (optimizeEpsilon < 0.0f) optimizeEpsilon = 0.0f;
 
 		// Load textures
-		map<string, BaseTexture*> nametbl;
+		std::map<std::string, BaseTexture*> nametbl;
 
-		if (cb) cb->PrintMsg ("  loading textures and blendmaps...");
+		if (cb) cb->PrintMsg("  loading textures and blendmaps...");
 		bool hasBumpmaps = false;
 		for (size_t a=0;a<shaderDef.stages.size();a++)
 		{
 			ShaderDef::Stage* st = &shaderDef.stages [a];
 			// Already loaded?
-			if (nametbl.find (st->sourceName) == nametbl.end())
+			if (nametbl.find(st->sourceName) == nametbl.end())
 				nametbl[st->sourceName] = LoadImageSource(st->sourceName, basepath, heightmap, cb, cfg);
 			st->source = nametbl[st->sourceName];
 
 			std::string bm;
-			if (tdf->SGetValue (bm, basepath + st->sourceName + "\\Bumpmap"))
+			if (tdf->SGetValue(bm, basepath + st->sourceName + "\\Bumpmap"))
 				hasBumpmaps = true;
 		}
 
@@ -198,19 +203,19 @@ namespace terrain {
 			for (size_t a = 0; a < shaderDef.normalMapStages.size(); a++) {
 				ShaderDef::Stage* st = &shaderDef.normalMapStages [a];
 
-				string name = st->sourceName;
+				std::string name = st->sourceName;
 				if (st->operation != ShaderDef::Alpha)
 					name += "_BM"; // make it unique, this whole naming thing is very hackish though..
 
 				// Already loaded?
-				if (nametbl.find (name) == nametbl.end()) {
+				if (nametbl.find(name) == nametbl.end()) {
 					// load a bumpmap unless it's an alpha operation
 					st->source = LoadImageSource(st->sourceName, basepath,
 						heightmap, cb, cfg, st->operation != ShaderDef::Alpha);
 					if (!st->source) {
 						if (!flatBumpmap) {
 							flatBumpmap = TiledTexture::CreateFlatBumpmap();
-							textures.push_back (flatBumpmap);
+							textures.push_back(flatBumpmap);
 						}
 						st->source = flatBumpmap;
 					}
@@ -223,55 +228,55 @@ namespace terrain {
 		else cfg->useBumpMaps = false;
 
 		// Generate blendmap mipmaps
-		deque<AlphaImage*>* bmMipmaps = new deque<AlphaImage*>[blendMaps.size()];
+		std::deque<AlphaImage*>* bmMipmaps = new std::deque<AlphaImage*>[blendMaps.size()];
 		GenerateInfo gi;
 		gi.bmMipmaps = bmMipmaps;
 
-		if (cb) { cb->PrintMsg ("  generating blendmap mipmaps..."); }
+		if (cb) { cb->PrintMsg("  generating blendmap mipmaps..."); }
 
 		for (size_t a=0;a<blendMaps.size();a++) {
-			Blendmap *bm = blendMaps[a];
+			Blendmap* bm = blendMaps[a];
 
-			AlphaImage *cur = bm->image;
+			AlphaImage* cur = bm->image;
 			bm->image = 0;
 
 			do {
-				bmMipmaps[a].push_front (cur);
-				cur = cur->CreateMipmap ();
+				bmMipmaps[a].push_front(cur);
+				cur = cur->CreateMipmap();
 			} while (cur);
 
 			for (size_t c=0;c<bmMipmaps[a].size();c++)
 				if (bmMipmaps[a][c]->w == QUAD_W) {
-					gi.testlod.push_back ((int)c);
+					gi.testlod.push_back((int)c);
 					break;
 				}
 		}
 
-		if (cb) cb->PrintMsg ("  loading blendmaps into OpenGL...");
+		if (cb) cb->PrintMsg("  loading blendmaps into OpenGL...");
 
 		// Convert to textures
-		for (size_t a=0;a<blendMaps.size();a++) {
-			AlphaImage *bm = bmMipmaps[a].back();
+		for (size_t a = 0; a < blendMaps.size(); a++) {
+			AlphaImage* bm = bmMipmaps[a].back();
 
 			// Save image
 			if (blendMaps[a]->generatorInfo) {
 				char fn[32];
-				SNPRINTF (fn,32, "blendmap"_STPF_".jpg", a);
+				SNPRINTF(fn,32, "blendmap"_STPF_".jpg", a);
 				remove(fn);
-				bm->Save (fn);
+				bm->Save(fn);
 			}
 			// Upload
-			glGenTextures (1, &blendMaps[a]->id);
-			glBindTexture (GL_TEXTURE_2D, blendMaps[a]->id);
+			glGenTextures(1, &blendMaps[a]->id);
+			glBindTexture(GL_TEXTURE_2D, blendMaps[a]->id);
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
 			if (cfg->anisotropicFiltering > 0.0f)
 				glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, cfg->anisotropicFiltering);
 
-			deque<AlphaImage*>& mipmaps = bmMipmaps[a];
-			for (size_t d=0;d<mipmaps.size();d++) {
-				AlphaImage *lod = mipmaps[mipmaps.size()-d-1];
-				glTexImage2D (GL_TEXTURE_2D, (int)d, GL_ALPHA, lod->w, lod->h, 0, GL_ALPHA, GL_FLOAT, lod->data);
+			std::deque<AlphaImage*>& mipmaps = bmMipmaps[a];
+			for (size_t d = 0; d < mipmaps.size(); d++) {
+				AlphaImage* lod = mipmaps[mipmaps.size()-d-1];
+				glTexImage2D(GL_TEXTURE_2D, (int)d, GL_ALPHA, lod->w, lod->h, 0, GL_ALPHA, GL_FLOAT, lod->data);
 			}
 			blendMaps[a]->image = 0;
 		}
@@ -294,30 +299,30 @@ namespace terrain {
 		// see how lighting should be implemented, based on config and available textures
 		InstantiateShaders(cfg, cb);
 
-		if (cb) { cb->PrintMsg ("  initializing terrain node shaders..."); }
+		if (cb) { cb->PrintMsg("  initializing terrain node shaders..."); }
 
 		shaderHandler->BeginBuild();
-		CreateTexProg (quadtree, &gi);
+		CreateTexProg(quadtree, &gi);
 		shaderHandler->EndBuild();
 
 		// count passes
 		maxPasses = 0;
-		for (map<uint, RenderSetupCollection*>::iterator mi=gi.nodesetup.begin();mi!=gi.nodesetup.end();++mi) {
+		for (std::map<uint, RenderSetupCollection*>::iterator mi=gi.nodesetup.begin();mi!=gi.nodesetup.end();++mi) {
 			for (size_t i = 0; i < mi->second->renderSetup.size(); i++) {
-				RenderSetup *rs = mi->second->renderSetup[i];
+				RenderSetup* rs = mi->second->renderSetup[i];
 
-				if (rs->passes.size () > maxPasses) {
+				if (rs->passes.size() > maxPasses) {
 					maxPasses = rs->passes.size();
 				}
 			}
-			texNodeSetup.push_back (mi->second);
+			texNodeSetup.push_back(mi->second);
 		}
 
-		if (cb) cb->PrintMsg ("  deleting temporary blendmap data...");
+		if (cb) cb->PrintMsg("  deleting temporary blendmap data...");
 
 		// Free blendmap mipmap images
-		for (size_t a=0;a<blendMaps.size();a++) {
-			for (deque<AlphaImage*>::iterator i=bmMipmaps[a].begin();i!=bmMipmaps[a].end();++i) {
+		for (size_t a = 0; a < blendMaps.size(); a++) {
+			for (std::deque<AlphaImage*>::iterator i = bmMipmaps[a].begin(); i != bmMipmaps[a].end(); ++i) {
 				delete *i;
 			}
 		}
@@ -327,7 +332,7 @@ namespace terrain {
 		shadowMapParams = new ShadowMapParams;
 	}
 
-	void TerrainTexture::ReloadShaders(TQuad *quadtree, Config *cfg)
+	void TerrainTexture::ReloadShaders(TQuad* quadtree, Config* cfg)
 	{
 		GenerateInfo gi;
 
@@ -336,17 +341,17 @@ namespace terrain {
 		shaderDef.useShadowMapping = cfg->useShadowMaps;
 
 		shaderHandler->BeginBuild();
-		ReloadTexProg (quadtree, &gi);
+		ReloadTexProg(quadtree, &gi);
 		shaderHandler->EndBuild();
 
 		// FIXME: count passes again?
 	}
 
-	void TerrainTexture::ReloadTexProg (TQuad *node, TerrainTexture::GenerateInfo *gi)
+	void TerrainTexture::ReloadTexProg(TQuad* node, TerrainTexture::GenerateInfo* gi)
 	{
-		RenderSetupCollection *tns = node->textureSetup;
+		RenderSetupCollection* tns = node->textureSetup;
 
-		if (gi->nodesetup.find (tns->sortkey) == gi->nodesetup.end()) {
+		if (gi->nodesetup.find(tns->sortkey) == gi->nodesetup.end()) {
 			gi->nodesetup[tns->sortkey] = tns;
 
 			for (size_t a = 0; a < shaders.size(); a++) {
@@ -364,33 +369,33 @@ namespace terrain {
 
 		if (!node->isLeaf()) {
 			for (int a=0;a<4;a++)
-				ReloadTexProg (node->childs[a], gi);
+				ReloadTexProg(node->childs[a], gi);
 		}
 	}
 
-	void TerrainTexture::CreateTexProg (TQuad *node, TerrainTexture::GenerateInfo *gi)
+	void TerrainTexture::CreateTexProg(TQuad* node, TerrainTexture::GenerateInfo* gi)
 	{
 		// Test blendmaps
-		for (size_t a=0;a<blendMaps.size();a++) {
-			deque <AlphaImage*>& mipmaps = gi->bmMipmaps[a];
+		for (size_t a = 0; a < blendMaps.size(); a++) {
+			std::deque<AlphaImage*>& mipmaps = gi->bmMipmaps[a];
 			int mipIndex = node->depth + gi->testlod[a];
 
-			if (mipIndex >= mipmaps.size ())
-				mipIndex = mipmaps.size () - 1;
+			if (mipIndex >= mipmaps.size())
+				mipIndex = mipmaps.size() - 1;
 
 			// scaling divisor, needed because the blendmap maybe lower res than the corresponding part of heightmap
 			int d = ( (1<<node->depth) * QUAD_W ) / mipmaps[mipIndex]->w;
 
 			// Calculate "constants" representing the blendmap in the area that the node covers
-			blendMaps[a]->curAreaResult = mipmaps[mipIndex]->TestArea ((node->qmPos.x * QUAD_W)/d,
+			blendMaps[a]->curAreaResult = mipmaps[mipIndex]->TestArea((node->qmPos.x * QUAD_W)/d,
 				(node->qmPos.y * QUAD_W)/d, ((node->qmPos.x+1)*QUAD_W)/d, ((node->qmPos.y+1)*QUAD_W)/d, optimizeEpsilon);
 		}
 
-		uint key = CalcBlendmapSortKey ();
-		if (gi->nodesetup.find (key) == gi->nodesetup.end()) {
+		uint key = CalcBlendmapSortKey();
+		if (gi->nodesetup.find(key) == gi->nodesetup.end()) {
 			RenderSetupCollection* tns = gi->nodesetup[key] = new RenderSetupCollection;
 
-			tns->renderSetup.resize (shaders.size());
+			tns->renderSetup.resize(shaders.size());
 			uint vda = 0;
 
 			// create a rendersetup for every shader expression
@@ -402,13 +407,13 @@ namespace terrain {
 
 				for (size_t p=0;p<tns->renderSetup [a]->passes.size();p++) {
 					if(tns->renderSetup [a]->passes[p].shaderSetup)
-						vda |= tns->renderSetup [a]->passes[p].shaderSetup->GetVertexDataRequirements ();
+						vda |= tns->renderSetup [a]->passes[p].shaderSetup->GetVertexDataRequirements();
 				}
 			}
 
 			// The unlit shader and far shader have to be supported, near shader is optional
 			if (!tns->renderSetup[unlitShader.index] || !tns->renderSetup[farShader.index])
-				throw runtime_error ("Unable to build full texture shader. ");
+				throw std::runtime_error("Unable to build full texture shader. ");
 
 			tns->vertexDataReq = vda;
 			tns->sortkey = key;
@@ -416,42 +421,42 @@ namespace terrain {
 		node->textureSetup = gi->nodesetup[key];
 
 		if (!node->isLeaf()) {
-			for (int a=0;a<4;a++)
-				CreateTexProg (node->childs[a], gi);
+			for (int a = 0; a < 4; a++)
+				CreateTexProg(node->childs[a], gi);
 		}
 
 	}
 
-	uint TerrainTexture::CalcBlendmapSortKey ()
+	uint TerrainTexture::CalcBlendmapSortKey()
 	{
 		uint key=0;
 		uint mul=1;
-		for (size_t a=0;a<blendMaps.size();a++) {
+		for (size_t a = 0; a < blendMaps.size(); a++) {
 			key += mul * (uint)blendMaps[a]->curAreaResult;
 			mul *= 3; // every blendmap::curAreaResult has 3 different states
 		}
 		return key;
 	}
 
-	void TerrainTexture::InstantiateShaders(Config *cfg, ILoadCallback *cb)
+	void TerrainTexture::InstantiateShaders(Config* cfg, ILoadCallback* cb)
 	{
 		nearShader.def = shaderDef;
 		if (cfg->useStaticShadow) {
-			nearShader.def.stages.push_back (ShaderDef::Stage());
+			nearShader.def.stages.push_back(ShaderDef::Stage());
 			ShaderDef::Stage& st = nearShader.def.stages.back();
 			st.operation = ShaderDef::Mul;
 			st.source = lightmap;
 			nearShader.def.hasLighting = true;
 		}
-		shaders.push_back (&nearShader);
+		shaders.push_back(&nearShader);
 	}
 
-	bool TerrainTexture::SetupNode (TQuad *q, int pass)
+	bool TerrainTexture::SetupNode(TQuad* q, int pass)
 	{
-		RenderSetupCollection *ns = q->textureSetup;
-		if (debugShader>=0) ns = texNodeSetup[debugShader];
+		RenderSetupCollection* ns = q->textureSetup;
+		if (debugShader >= 0) ns = texNodeSetup[debugShader];
 
-		RenderSetup *rs = 0;
+		RenderSetup* rs = NULL;
 
 		if (!q->renderData)
 			return false;
@@ -470,13 +475,13 @@ namespace terrain {
 		}*/
 		rs = ns->renderSetup [nearShader.index];
 
-		if (pass >= rs->passes.size ())
+		if (pass >= rs->passes.size())
 			return false;
 
 		bool r = true;
 		if (rs != currentRenderSetup) {
-			RenderPass *p = &rs->passes [pass];
-			r = SetupShading (p, pass);
+			RenderPass* p = &rs->passes [pass];
+			r = SetupShading(p, pass);
 
 			ns->currentShaderSetup = p->shaderSetup;
 			currentRenderSetup = rs;
@@ -484,12 +489,12 @@ namespace terrain {
 
 		if (q->renderData->normalMap) {
 			int imageUnit, coordUnit;
-			ns->currentShaderSetup->GetTextureUnits (&detailBumpmap, imageUnit, coordUnit);
+			ns->currentShaderSetup->GetTextureUnits(&detailBumpmap, imageUnit, coordUnit);
 
 			glActiveTextureARB(GL_TEXTURE0_ARB+imageUnit);
 			glBindTexture(GL_TEXTURE_2D, q->renderData->normalMap);
 			glActiveTextureARB(GL_TEXTURE0_ARB+coordUnit);
-			detailBumpmap.SetupTexGen ();
+			detailBumpmap.SetupTexGen();
 			glActiveTextureARB(GL_TEXTURE0_ARB);
 		}
 		return r;
@@ -512,11 +517,11 @@ namespace terrain {
         bool blend=true;
 		switch (p->operation) {
 		case Pass_Mul:
-			glBlendFunc (GL_ZERO, GL_SRC_COLOR); // color=src*0+dst*src
+			glBlendFunc(GL_ZERO, GL_SRC_COLOR); // color=src*0+dst*src
 			break;
 
 		case Pass_Add:
-			glBlendFunc (GL_ONE, GL_ONE); // color=dst*1+src*1
+			glBlendFunc(GL_ONE, GL_ONE); // color=dst*1+src*1
 			break;
 
 		case Pass_Replace:
@@ -524,20 +529,20 @@ namespace terrain {
 			break;
 
 		case Pass_Interpolate:      // interpolate dest and src using src alpha
-			glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			break;
 
 		case Pass_MulColor:
-			glBlendFunc (GL_ZERO, GL_SRC_COLOR); // color=src*0+dst*src
+			glBlendFunc(GL_ZERO, GL_SRC_COLOR); // color=src*0+dst*src
 			shaderHandler->EndTexturing();
 			glEnable(GL_BLEND);
 			return true;
 		}
 
 		if (blend)
-			glEnable (GL_BLEND);
+			glEnable(GL_BLEND);
 
-		return shaderHandler->SetupShader (p->shaderSetup, parms);
+		return shaderHandler->SetupShader(p->shaderSetup, parms);
 	}
 
 
@@ -546,7 +551,7 @@ namespace terrain {
 		glDisable(GL_BLEND);
 		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-		shaderHandler->EndTexturing ();
+		shaderHandler->EndTexturing();
 	}
 	void TerrainTexture::BeginTexturing() {
 		/*
@@ -562,7 +567,7 @@ namespace terrain {
 		shaderHandler->BeginTexturing();
 	}
 
-	int TerrainTexture::NumPasses() {
+	int TerrainTexture::NumPasses() const {
 		return maxPasses;
 	}
 
@@ -575,10 +580,10 @@ namespace terrain {
 		shaderHandler->BeginPass(blendMaps, textures, p);
 	}
 
-	void TerrainTexture::SetShaderParams (const Vector3& lightDir, const Vector3& eyePos)
+	void TerrainTexture::SetShaderParams(const Vector3& lightDir, const Vector3& eyePos)
 	{
 		wsLightDir = lightDir;
-		wsLightDir.ANormalize ();
+		wsLightDir.ANormalize();
 		wsEyePos = eyePos;
 	}
 
@@ -587,7 +592,7 @@ namespace terrain {
 			*shadowMapParams = *smp;
 	}
 
-	void TerrainTexture::DebugEvent (const std::string& event)
+	void TerrainTexture::DebugEvent(const std::string& event)
 	{
 		if (event == "t_prev_shader") {
 			debugShader--;
@@ -601,24 +606,29 @@ namespace terrain {
 	}
 
 #ifndef TERRAINRENDERERLIB_EXPORTS
-	void TerrainTexture::DebugPrint (IFontRenderer *fontRenderer)
+	void TerrainTexture::DebugPrint(IFontRenderer* fontRenderer)
 	{
-		fontRenderer->printf(0,75,16.0f,"Numpasses: %d, curshader=%d, texture units: %d, sundir(%1.3f,%1.3f,%1.3f)", maxPasses, debugShader, shaderHandler->MaxTextureUnits (), wsLightDir.x,wsLightDir.y,wsLightDir.z);
+		if (fontRenderer != NULL) { 
+			fontRenderer->printf(0, 75, 16.0f,
+					"Numpasses: %d, curshader=%d, texture units: %d, sundir(%1.3f,%1.3f,%1.3f)",
+					maxPasses, debugShader, shaderHandler->MaxTextureUnits(),
+					wsLightDir.x, wsLightDir.y, wsLightDir.z);
+		}
 	}
 #endif
 
-	void ShaderDef::LoadStages(int numStages,const char *stagename, const TdfParser& tdf, std::vector<ShaderDef::Stage>& stages)
+	void ShaderDef::LoadStages(int numStages,const char* stagename, const TdfParser& tdf, std::vector<ShaderDef::Stage>& stages)
 	{
-		for (int a=0;a<numStages;a++)
+		for (int a = 0; a < numStages; a++)
 		{
-			string path = "map\\terrain\\";
+			std::string path = "map\\terrain\\";
 			char num[10];
 			SNPRINTF(num, 10, "%d", a);
 
-			string ts = path + stagename + num + "\\";
+			std::string ts = path + stagename + num + "\\";
 
-			string opstr = tdf.SGetValueDef("mul", ts + "operation");
-			struct { StageOp op; const char *str; } tbl[] =
+			std::string opstr = tdf.SGetValueDef("mul", ts + "operation");
+			struct { StageOp op; const char* str; } tbl[] =
 			{
 				{ Mul, "mul" },
 				{ Add, "add" },
@@ -640,12 +650,12 @@ namespace terrain {
 			{
 				// insert an alpha stage before the blend stage
 				stages.push_back(Stage());
-				stages.back().sourceName = tdf.SGetValueDef(string(), ts + "blender");
+				stages.back().sourceName = tdf.SGetValueDef(std::string(), ts + "blender");
 				stages.back().operation = Alpha;
 			}
 
 			stages.push_back(Stage());
-			stages.back().sourceName = tdf.SGetValueDef(string(), ts + "source");
+			stages.back().sourceName = tdf.SGetValueDef(std::string(), ts + "source");
 			stages.back().operation = operation;
 			if (stages.back().sourceName.empty())
 				throw content_error(ts + " does not have a source texture");
@@ -653,7 +663,7 @@ namespace terrain {
 	}
 
 	void ShaderDef::Parse(const TdfParser& tdf, bool needNormalMap) {
-		string path = "map\\terrain\\";
+		std::string path = "map\\terrain\\";
 
 		int numStages = atoi(tdf.SGetValueDef("0", path + "NumTextureStages").c_str());
 		bool autoBumpMap = !!atoi(tdf.SGetValueDef("1", path + "AutoBumpmapStages").c_str());
@@ -705,12 +715,12 @@ namespace terrain {
 			switch (st.operation) {
 				case Blend: // previous * (1-alpha) + source * alpha
 					if (atr == AlphaImage::AREA_MIXED)
-						dst.push_back (st);
+						dst.push_back(st);
 					else if(atr == AlphaImage::AREA_ONE)
 					{
 						// only source remains
 						dst.clear();
-						dst.push_back (st);
+						dst.push_back(st);
 						dst.back().operation = Mul;
 					}
 					else if (atr == AlphaImage::AREA_ZERO)
@@ -728,7 +738,7 @@ namespace terrain {
 					Blendmap* bm = dynamic_cast<Blendmap*>(st.source);
 					if (bm) atr = bm->curAreaResult;
 					else atr = AlphaImage::AREA_MIXED;
-					dst.push_back (st);
+					dst.push_back(st);
 					break;}
 				case Add:
 				case Mul:
@@ -740,11 +750,11 @@ namespace terrain {
 
 	void ShaderDef::Output()
 	{
-		const char *opstr[] = { "add", "mul", "alpha" ,"blend" };
-		d_trace ("Shader: "_STPF_" stages.\n", stages.size());
+		const char* opstr[] = { "add", "mul", "alpha" ,"blend" };
+		LOG("Shader: "_STPF_" stages.", stages.size());
 		for (size_t a = 0; a < stages.size(); a++){
-			d_trace (_STPF_"\toperation=%s\n", a, opstr[(int)stages[a].operation]);
-			d_trace ("\tsource=%s\n", stages[a].sourceName.c_str());
+			LOG(_STPF_"\toperation=%s", a, opstr[(int)stages[a].operation]);
+			LOG("\tsource=%s", stages[a].sourceName.c_str());
 		}
 	}
 };
