@@ -1,8 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include <cstring>
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "VertexArray.h"
 
@@ -32,8 +31,8 @@ CVertexArray::~CVertexArray()
 
 void CVertexArray::Initialize()
 {
-	drawArrayPos=drawArray;
-	stripArrayPos=stripArray;
+	drawArrayPos  = drawArray;
+	stripArrayPos = stripArray;
 }
 
 bool CVertexArray::IsReady() const
@@ -43,8 +42,9 @@ bool CVertexArray::IsReady() const
 
 void CVertexArray::EndStrip()
 {
-	if((char*)stripArrayPos > (char*)stripArraySize - 4 * sizeof(unsigned int))
+	if ((char*)stripArrayPos > ((char*)stripArraySize - 4 * sizeof(unsigned int))) {
 		EnlargeStripArray();
+	}
 
 	*stripArrayPos++ = ((char*)drawArrayPos - (char*)drawArray);
 }
