@@ -8,17 +8,10 @@
 #include <string>
 #include <set>
 
-#include "Matrix44f.h"
+#include "System/Matrix44f.h"
 #include "3DModel.h"
 
 
-class CUnit;
-class C3DOParser;
-class CS3OParser;
-class C3DModelLoader;
-extern C3DModelLoader* modelParser;
-
-
 class IModelParser
 {
 public:
@@ -26,23 +19,28 @@ public:
 };
 
 
+class CUnit;
+class CAssParser;
+
 class C3DModelLoader
 {
 public:
-	C3DModelLoader(void);
-	~C3DModelLoader(void);
+	C3DModelLoader();
+	~C3DModelLoader();
 
 	void Update();
 	S3DModel* Load3DModel(std::string name, const float3& centerOffset = ZeroVector);
 
 	void DeleteLocalModel(CUnit* unit);
 	void CreateLocalModel(CUnit* unit);
-	void FixLocalModel(CUnit* unit);
+
+	typedef std::map<std::string, S3DModel*> ModelMap;
+	typedef std::map<std::string, IModelParser*> ParserMap;
 
 private:
-//FIXME make some static?
-	std::map<std::string, S3DModel*> cache;
-	std::map<std::string, IModelParser*> parsers;
+	// FIXME make some static?
+	ModelMap cache;
+	ParserMap parsers;
 
 #if defined(USE_GML) && GML_ENABLE_SIM
 	std::vector<S3DModelPiece*> createLists;
@@ -56,10 +54,10 @@ private:
 
 	void DeleteChilds(S3DModelPiece* o);
 
-	LocalModel* CreateLocalModel(S3DModel* model);
-	void CreateLocalModelPieces(S3DModelPiece* model, LocalModel* lmodel, int* piecenum);
-
-	void FixLocalModel(S3DModelPiece* model, LocalModel* lmodel, int* piecenum);
+	void SetLocalModelPieceDisplayLists(CUnit* unit);
+	void SetLocalModelPieceDisplayLists(S3DModelPiece* model, LocalModel* lmodel, int* piecenum);
 };
 
+extern C3DModelLoader* modelParser;
+
 #endif /* IMODELPARSER_H */
