@@ -1,18 +1,22 @@
-#include "StdAfx.h"
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "ProjectileDrawer.hpp"
+
+#include "ProjectileDrawer.h"
 
 #include "Game/Camera.h"
+#include "Game/GlobalUnsynced.h"
+#include "Game/LoadScreen.h"
 #include "Lua/LuaParser.h"
 #include "Map/MapInfo.h"
 #include "Rendering/GroundFlash.h"
 #include "Rendering/GlobalRendering.h"
 #include "Rendering/ShadowHandler.h"
 #include "Rendering/UnitDrawer.h"
+#include "Rendering/Env/ISky.h"
 #include "Rendering/GL/FBO.h"
 #include "Rendering/GL/myGL.h"
 #include "Rendering/GL/VertexArray.h"
-#include "Rendering/Shaders/Shader.hpp"
+#include "Rendering/Shaders/Shader.h"
 #include "Rendering/Textures/Bitmap.h"
 #include "Rendering/Textures/ColorMap.h"
 #include "Rendering/Textures/S3OTextureHandler.h"
@@ -24,15 +28,14 @@
 #include "Sim/Projectiles/Projectile.h"
 #include "Sim/Projectiles/ProjectileHandler.h"
 #include "Sim/Projectiles/PieceProjectile.h"
-#include "Sim/Projectiles/Unsynced/FlyingPiece.hpp"
+#include "Sim/Projectiles/Unsynced/FlyingPiece.h"
 #include "Sim/Projectiles/Unsynced/ShieldPartProjectile.h"
 #include "Sim/Projectiles/WeaponProjectiles/WeaponProjectile.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
 #include "Sim/Weapons/WeaponDef.h"
 #include "System/EventHandler.h"
 #include "System/Exceptions.h"
-#include "System/GlobalUnsynced.h"
-#include "System/LogOutput.h"
+#include "System/Log/ILog.h"
 #include "System/Util.h"
 
 bool distcmp::operator() (const CProjectile* arg1, const CProjectile* arg2) const {
@@ -51,162 +54,125 @@ CProjectileDrawer* projectileDrawer = NULL;
 CProjectileDrawer::CProjectileDrawer(): CEventClient("[CProjectileDrawer]", 123456, false) {
 	eventHandler.AddClient(this);
 
-	PrintLoadMsg("Creating Projectile Textures");
+	loadscreen->SetLoadMessage("Creating Projectile Textures");
 
 	textureAtlas = new CTextureAtlas(2048, 2048);
-
-	// used to block resources_map.tdf from loading textures
-	std::set<std::string> blockMapTexNames;
+	groundFXAtlas = new CTextureAtlas(2048, 2048);
 
 	LuaParser resourcesParser("gamedata/resources.lua", SPRING_VFS_MOD_BASE, SPRING_VFS_ZIP);
+	LuaParser mapResParser("gamedata/resources_map.lua", SPRING_VFS_MOD_BASE, SPRING_VFS_ZIP);
+
 	resourcesParser.Execute();
 
-	const LuaTable rootTable = resourcesParser.GetRoot();
-	const LuaTable gfxTable = rootTable.SubTable("graphics");
-	const LuaTable ptTable = gfxTable.SubTable("projectileTextures");
+	const LuaTable& resTable = resourcesParser.GetRoot();
+	const LuaTable& resGraphicsTable = resTable.SubTable("graphics");
+	const LuaTable& resProjTexturesTable = resGraphicsTable.SubTable("projectileTextures");
+	const LuaTable& resSmokeTexturesTable = resGraphicsTable.SubTable("smoke");
+	const LuaTable& resGroundFXTexturesTable = resGraphicsTable.SubTable("groundfx");
 
-	// add all textures in projectiletextures section
-	std::map<std::string, std::string> ptex;
-	ptTable.GetMap(ptex);
+	// used to block resources_map.* from overriding any of
+	// resources.lua:{projectile, smoke, groundfx}textures,
+	// as well as various defaults (repulsegfxtexture, etc)
+	std::set<std::string> blockedTexNames;
 
-	for (std::map<std::string, std::string>::iterator pi = ptex.begin(); pi != ptex.end(); ++pi) {
-		textureAtlas->AddTexFromFile(pi->first, "bitmaps/" + pi->second);
-		blockMapTexNames.insert(StringToLower(pi->first));
-	}
-	ptex.clear();
+	ParseAtlasTextures(true, resProjTexturesTable, blockedTexNames, textureAtlas);
+	ParseAtlasTextures(true, resGroundFXTexturesTable, blockedTexNames, groundFXAtlas);
 
-	// add all texture from sections within projectiletextures section
-	std::vector<std::string> seclist;
-	ptTable.GetKeys(seclist);
+	int smokeTexCount = -1;
 
-	for (size_t i = 0; i < seclist.size(); i++) {
-		const LuaTable ptSubTable = ptTable.SubTable(seclist[i]);
-
-		if (ptSubTable.IsValid()) {
-			ptSubTable.GetMap(ptex);
-
-			for (std::map<std::string, std::string>::iterator pi = ptex.begin(); pi != ptex.end(); ++pi) {
-				textureAtlas->AddTexFromFile(pi->first, "bitmaps/" + pi->second);
-				blockMapTexNames.insert(StringToLower(pi->first));
-			}
-			ptex.clear();
-		}
-	}
-
-	// get the smoke textures, hold the count in 'smokeCount'
-	const LuaTable smokeTable = gfxTable.SubTable("smoke");
-	int smokeCount;
-
-	if (smokeTable.IsValid()) {
-		for (smokeCount = 0; true; smokeCount++) {
-			const std::string tex = smokeTable.GetString(smokeCount + 1, "");
-			if (tex.empty()) {
-				break;
-			}
-			const std::string texName = "bitmaps/" + tex;
-			const std::string smokeName = "ismoke" + IntToString(smokeCount, "%02i");
-
-			textureAtlas->AddTexFromFile(smokeName, texName);
-			blockMapTexNames.insert(StringToLower(smokeName));
-		}
-	} else {
-		// setup the defaults
-		for (smokeCount = 0; smokeCount < 12; smokeCount++) {
-			const std::string smokeNum = IntToString(smokeCount, "%02i");
-			const std::string smokeName = "ismoke" + smokeNum;
-			const std::string texName = "bitmaps/smoke/smoke" + smokeNum + ".tga";
-
-			textureAtlas->AddTexFromFile(smokeName, texName);
-			blockMapTexNames.insert(StringToLower(smokeName));
-		}
-	}
-
-	if (smokeCount <= 0) {
-		throw content_error("missing smoke textures");
-	}
-
-	char tex[128][128][4];
-	for (int y = 0; y < 128; y++) { // shield
-		for (int x = 0; x < 128; x++) {
-			tex[y][x][0] = 70;
-			tex[y][x][1] = 70;
-			tex[y][x][2] = 70;
-			tex[y][x][3] = 70;
-		}
-	}
-
-	textureAtlas->AddTexFromMem("perlintex", 128, 128, CTextureAtlas::RGBA32, tex);
-	blockMapTexNames.insert("perlintex");
-
-	blockMapTexNames.insert("flare");
-	blockMapTexNames.insert("explo");
-	blockMapTexNames.insert("explofade");
-	blockMapTexNames.insert("heatcloud");
-	blockMapTexNames.insert("laserend");
-	blockMapTexNames.insert("laserfalloff");
-	blockMapTexNames.insert("randdots");
-	blockMapTexNames.insert("smoketrail");
-	blockMapTexNames.insert("wake");
-	blockMapTexNames.insert("perlintex");
-	blockMapTexNames.insert("flame");
-
-	blockMapTexNames.insert("sbtrailtexture");
-	blockMapTexNames.insert("missiletrailtexture");
-	blockMapTexNames.insert("muzzleflametexture");
-	blockMapTexNames.insert("repulsetexture");
-	blockMapTexNames.insert("dguntexture");
-	blockMapTexNames.insert("flareprojectiletexture");
-	blockMapTexNames.insert("sbflaretexture");
-	blockMapTexNames.insert("missileflaretexture");
-	blockMapTexNames.insert("beamlaserflaretexture");
-	blockMapTexNames.insert("bubbletexture");
-	blockMapTexNames.insert("geosquaretexture");
-	blockMapTexNames.insert("gfxtexture");
-	blockMapTexNames.insert("projectiletexture");
-	blockMapTexNames.insert("repulsegfxtexture");
-	blockMapTexNames.insert("sphereparttexture");
-	blockMapTexNames.insert("torpedotexture");
-	blockMapTexNames.insert("wrecktexture");
-	blockMapTexNames.insert("plasmatexture");
-
-
-	// allow map specified atlas textures for gaia unit projectiles
-	LuaParser mapResParser("gamedata/resources_map.lua", SPRING_VFS_MOD_BASE, SPRING_VFS_ZIP);
-	
-	if (mapResParser.Execute()) {
-		const LuaTable mapRoot = mapResParser.GetRoot();
-		const LuaTable mapTable = mapRoot.SubTable("projectileTextures");
-
-		// add all textures in projectiletextures section
-		std::map<std::string, std::string>::iterator pi;
-
-		mapTable.GetMap(ptex);
-
-		for (pi = ptex.begin(); pi != ptex.end(); ++pi) {
-			if (blockMapTexNames.find(StringToLower(pi->first)) == blockMapTexNames.end()) {
-				textureAtlas->AddTexFromFile(pi->first, "bitmaps/" + pi->second);
-			}
-		}
-		ptex.clear();
-
-		// add all texture from sections within projectiletextures section
-		mapTable.GetKeys(seclist);
-		for (size_t i = 0; i < seclist.size(); i++) {
-			const LuaTable mapSubTable = mapTable.SubTable(seclist[i]);
-			if (mapSubTable.IsValid()) {
-				mapSubTable.GetMap(ptex);
-				for (pi = ptex.begin(); pi != ptex.end(); ++pi) {
-					if (blockMapTexNames.find(StringToLower(pi->first)) == blockMapTexNames.end()) {
-						textureAtlas->AddTexFromFile(pi->first, "bitmaps/" + pi->second);
-					}
+	{
+		// get the smoke textures, hold the count in 'smokeTexCount'
+		if (resSmokeTexturesTable.IsValid()) {
+			for (smokeTexCount = 0; true; smokeTexCount++) {
+				const std::string& tex = resSmokeTexturesTable.GetString(smokeTexCount + 1, "");
+				if (tex.empty()) {
+					break;
 				}
-				ptex.clear();
+				const std::string texName = "bitmaps/" + tex;
+				const std::string smokeName = "ismoke" + IntToString(smokeTexCount, "%02i");
+
+				textureAtlas->AddTexFromFile(smokeName, texName);
+				blockedTexNames.insert(StringToLower(smokeName));
+			}
+		} else {
+			// setup the defaults
+			for (smokeTexCount = 0; smokeTexCount < 12; smokeTexCount++) {
+				const std::string smokeNum = IntToString(smokeTexCount, "%02i");
+				const std::string smokeName = "ismoke" + smokeNum;
+				const std::string texName = "bitmaps/smoke/smoke" + smokeNum + ".tga";
+
+				textureAtlas->AddTexFromFile(smokeName, texName);
+				blockedTexNames.insert(StringToLower(smokeName));
 			}
 		}
+
+		if (smokeTexCount <= 0) {
+			// this needs to be an exception, other code
+			// assumes at least one smoke-texture exists
+			throw content_error("missing smoke textures");
+		}
+	}
+
+	{
+		// shield-texture memory
+		char perlinTexMem[128][128][4];
+		for (int y = 0; y < 128; y++) {
+			for (int x = 0; x < 128; x++) {
+				perlinTexMem[y][x][0] = 70;
+				perlinTexMem[y][x][1] = 70;
+				perlinTexMem[y][x][2] = 70;
+				perlinTexMem[y][x][3] = 70;
+			}
+		}
+
+		textureAtlas->AddTexFromMem("perlintex", 128, 128, CTextureAtlas::RGBA32, perlinTexMem);
+	}
+
+	blockedTexNames.insert("perlintex");
+	blockedTexNames.insert("flare");
+	blockedTexNames.insert("explo");
+	blockedTexNames.insert("explofade");
+	blockedTexNames.insert("heatcloud");
+	blockedTexNames.insert("laserend");
+	blockedTexNames.insert("laserfalloff");
+	blockedTexNames.insert("randdots");
+	blockedTexNames.insert("smoketrail");
+	blockedTexNames.insert("wake");
+	blockedTexNames.insert("perlintex");
+	blockedTexNames.insert("flame");
+
+	blockedTexNames.insert("sbtrailtexture");
+	blockedTexNames.insert("missiletrailtexture");
+	blockedTexNames.insert("muzzleflametexture");
+	blockedTexNames.insert("repulsetexture");
+	blockedTexNames.insert("dguntexture");
+	blockedTexNames.insert("flareprojectiletexture");
+	blockedTexNames.insert("sbflaretexture");
+	blockedTexNames.insert("missileflaretexture");
+	blockedTexNames.insert("beamlaserflaretexture");
+	blockedTexNames.insert("bubbletexture");
+	blockedTexNames.insert("geosquaretexture");
+	blockedTexNames.insert("gfxtexture");
+	blockedTexNames.insert("projectiletexture");
+	blockedTexNames.insert("repulsegfxtexture");
+	blockedTexNames.insert("sphereparttexture");
+	blockedTexNames.insert("torpedotexture");
+	blockedTexNames.insert("wrecktexture");
+	blockedTexNames.insert("plasmatexture");
+
+	if (mapResParser.Execute()) {
+		// allow map-specified atlas textures (for gaia-projectiles and ground-flashes)
+		const LuaTable& mapResTable = mapResParser.GetRoot();
+		const LuaTable& mapResGraphicsTable = mapResTable.SubTable("graphics");
+		const LuaTable& mapResProjTexturesTable = mapResGraphicsTable.SubTable("projectileTextures");
+		const LuaTable& mapResGroundFXTexturesTable = mapResGraphicsTable.SubTable("groundfx");
+
+		ParseAtlasTextures(false, mapResProjTexturesTable, blockedTexNames, textureAtlas);
+		ParseAtlasTextures(false, mapResGroundFXTexturesTable, blockedTexNames, groundFXAtlas);
 	}
 
 	if (!textureAtlas->Finalize()) {
-		logOutput.Print("Could not finalize projectile texture atlas. Use less/smaller textures.");
+		LOG_L(L_ERROR, "Could not finalize projectile-texture atlas. Use less/smaller textures.");
 	}
 
 	flaretex        = textureAtlas->GetTexturePtr("flare");
@@ -221,9 +187,10 @@ CProjectileDrawer::CProjectileDrawer(): CEventClient("[CProjectileDrawer]", 1234
 	perlintex       = textureAtlas->GetTexturePtr("perlintex");
 	flametex        = textureAtlas->GetTexturePtr("flame");
 
-	for (int i = 0; i < smokeCount; i++) {
+	for (int i = 0; i < smokeTexCount; i++) {
 		const std::string smokeName = "ismoke" + IntToString(i, "%02i");
-		smoketex.push_back(textureAtlas->GetTexturePtr(smokeName));
+		const AtlasedTexture* smokeTex = textureAtlas->GetTexturePtr(smokeName);
+		smoketex.push_back(smokeTex);
 	}
 
 #define GETTEX(t, b) (textureAtlas->GetTexturePtrWithBackup((t), (b)))
@@ -248,34 +215,8 @@ CProjectileDrawer::CProjectileDrawer(): CEventClient("[CProjectileDrawer]", 1234
 #undef GETTEX
 
 
-	groundFXAtlas = new CTextureAtlas(2048, 2048);
-	// add all textures in groundfx section
-	const LuaTable groundfxTable = gfxTable.SubTable("groundfx");
-	groundfxTable.GetMap(ptex);
-
-	for (std::map<std::string, std::string>::iterator pi = ptex.begin(); pi != ptex.end(); ++pi) {
-		groundFXAtlas->AddTexFromFile(pi->first, "bitmaps/" + pi->second);
-	}
-	ptex.clear();
-
-	// add all textures from sections within groundfx section
-	groundfxTable.GetKeys(seclist);
-
-	for (size_t i = 0; i < seclist.size(); i++) {
-		const LuaTable gfxSubTable = groundfxTable.SubTable(seclist[i]);
-
-		if (gfxSubTable.IsValid()) {
-			gfxSubTable.GetMap(ptex);
-
-			for (std::map<std::string, std::string>::iterator pi = ptex.begin(); pi != ptex.end(); ++pi) {
-				groundFXAtlas->AddTexFromFile(pi->first, "bitmaps/" + pi->second);
-			}
-			ptex.clear();
-		}
-	}
-
 	if (!groundFXAtlas->Finalize()) {
-		logOutput.Print("Could not finalize groundFX texture atlas. Use less/smaller textures.");
+		LOG_L(L_ERROR, "Could not finalize groundFX texture atlas. Use less/smaller textures.");
 	}
 
 	groundflashtex = groundFXAtlas->GetTexturePtr("groundflash");
@@ -283,19 +224,19 @@ CProjectileDrawer::CProjectileDrawer(): CEventClient("[CProjectileDrawer]", 1234
 	seismictex = groundFXAtlas->GetTexturePtr("seismic");
 
 	for (int a = 0; a < 4; ++a) {
-		perlinBlend[a]=0;
+		perlinBlend[a] = 0.0f;
 	}
 
-	unsigned char tempmem[4 * 16 * 16];
-	for (int a = 0; a < 4 * 16 * 16; ++a) {
-		tempmem[a] = 0;
-	}
-	for (int a = 0; a < 8; ++a) {
-		glGenTextures(1, &perlinTex[a]);
-		glBindTexture(GL_TEXTURE_2D, perlinTex[a]);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 16,16, 0, GL_RGBA, GL_UNSIGNED_BYTE, tempmem);
+	{
+		unsigned char tempmem[4 * 16 * 16] = {0};
+
+		for (int a = 0; a < 8; ++a) {
+			glGenTextures(1, &perlinTex[a]);
+			glBindTexture(GL_TEXTURE_2D, perlinTex[a]);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 16,16, 0, GL_RGBA, GL_UNSIGNED_BYTE, tempmem);
+		}
 	}
 
 	drawPerlinTex = false;
@@ -339,6 +280,57 @@ CProjectileDrawer::~CProjectileDrawer() {
 
 
 
+void CProjectileDrawer::ParseAtlasTextures(
+	const bool blockTextures,
+	const LuaTable& textureTable,
+	std::set<std::string>& blockedTextures,
+	CTextureAtlas* textureAtlas)
+{
+	std::vector<std::string> subTables;
+	std::map<std::string, std::string> texturesMap;
+	std::map<std::string, std::string>::iterator texturesMapIt;
+
+	textureTable.GetMap(texturesMap);
+	textureTable.GetKeys(subTables);
+
+	for (texturesMapIt = texturesMap.begin(); texturesMapIt != texturesMap.end(); ++texturesMapIt) {
+		const std::string textureName = StringToLower(texturesMapIt->first);
+
+		if (blockTextures) {
+			// no textures added to this atlas are allowed
+			// to be overwritten later by other textures of
+			// the same name
+			blockedTextures.insert(textureName);
+		}
+		if (blockTextures || (blockedTextures.find(textureName) == blockedTextures.end())) {
+			textureAtlas->AddTexFromFile(texturesMapIt->first, "bitmaps/" + texturesMapIt->second);
+		}
+	}
+
+	texturesMap.clear();
+
+	for (size_t i = 0; i < subTables.size(); i++) {
+		const LuaTable& textureSubTable = textureTable.SubTable(subTables[i]);
+
+		if (textureSubTable.IsValid()) {
+			textureSubTable.GetMap(texturesMap);
+
+			for (texturesMapIt = texturesMap.begin(); texturesMapIt != texturesMap.end(); ++texturesMapIt) {
+				const std::string textureName = StringToLower(texturesMapIt->first);
+
+				if (blockTextures) {
+					blockedTextures.insert(textureName);
+				}
+				if (blockTextures || (blockedTextures.find(textureName) == blockedTextures.end())) {
+					textureAtlas->AddTexFromFile(texturesMapIt->first, "bitmaps/" + texturesMapIt->second);
+				}
+			}
+
+			texturesMap.clear();
+		}
+	}
+}
+
 void CProjectileDrawer::LoadWeaponTextures() {
 	// post-process the synced weapon-defs to set unsynced fields
 	// (this requires CWeaponDefHandler to have been initialized)
@@ -485,7 +477,7 @@ void CProjectileDrawer::DrawProjectile(CProjectile* pro, bool drawReflection, bo
 			float dif = pro->pos.y - camera->pos.y;
 			float3 zeroPos = camera->pos * (pro->pos.y / dif) + pro->pos * (-camera->pos.y / dif);
 
-			if (ground->GetApproximateHeight(zeroPos.x, zeroPos.z) > 3 + 0.5f * pro->drawRadius) {
+			if (ground->GetApproximateHeight(zeroPos.x, zeroPos.z, false) > 3 + 0.5f * pro->drawRadius) {
 				return;
 			}
 		}
@@ -524,8 +516,9 @@ void CProjectileDrawer::DrawProjectilesSetShadow(std::set<CProjectile*>& project
 
 void CProjectileDrawer::DrawProjectileShadow(CProjectile* p)
 {
+	const CUnit* owner = p->owner();
 	if ((gu->spectatingFullView || loshandler->InLos(p, gu->myAllyTeam) ||
-		(p->owner() && teamHandler->Ally(p->owner()->allyteam, gu->myAllyTeam)))) {
+		(owner && teamHandler->Ally(owner->allyteam, gu->myAllyTeam)))) {
 
 		if (!DrawProjectileModel(p, true)) {
 			if (p->castShadow) {
@@ -541,7 +534,7 @@ void CProjectileDrawer::DrawProjectileShadow(CProjectile* p)
 
 void CProjectileDrawer::DrawProjectilesMiniMap()
 {
-	GML_STDMUTEX_LOCK(proj); // DrawProjectilesMiniMap
+	GML_RECMUTEX_LOCK(proj); // DrawProjectilesMiniMap
 
 	typedef std::set<CProjectile*> ProjectileSet;
 	typedef std::set<CProjectile*>::const_iterator ProjectileSetIt;
@@ -592,7 +585,8 @@ void CProjectileDrawer::DrawProjectilesMiniMap()
 		for (std::set<CProjectile*>::iterator it = renderProjectiles.begin(); it != renderProjectiles.end(); ++it) {
 			CProjectile* p = *it;
 
-			if ((p->owner() && (p->owner()->allyteam == gu->myAllyTeam)) ||
+			const CUnit* owner = p->owner();
+			if ((owner && (owner->allyteam == gu->myAllyTeam)) ||
 				gu->spectatingFullView || loshandler->InLos(p, gu->myAllyTeam)) {
 				p->DrawOnMinimap(*lines, *points);
 			}
@@ -641,12 +635,9 @@ void CProjectileDrawer::Update() {
 void CProjectileDrawer::Draw(bool drawReflection, bool drawRefraction) {
 	glDisable(GL_BLEND);
 	glEnable(GL_TEXTURE_2D);
-	glDepthMask(1);
+	glDepthMask(GL_TRUE);
 
-	if (globalRendering->drawFog) {
-		glEnable(GL_FOG);
-		glFogfv(GL_FOG_COLOR, mapInfo->atmosphere.fogColor);
-	}
+	ISky::SetupFog();
 
 	{
 		GML_STDMUTEX_LOCK(rpiece); // Draw
@@ -665,7 +656,7 @@ void CProjectileDrawer::Draw(bool drawReflection, bool drawRefraction) {
 	Update();
 
 	{
-		GML_STDMUTEX_LOCK(proj); // Draw
+		GML_RECMUTEX_LOCK(proj); // Draw
 
 		unitDrawer->SetupForUnitDrawing();
 
@@ -702,7 +693,7 @@ void CProjectileDrawer::Draw(bool drawReflection, bool drawRefraction) {
 		glColor4f(1.0f, 1.0f, 1.0f, 0.2f);
 		glAlphaFunc(GL_GREATER, 0.0f);
 		glEnable(GL_ALPHA_TEST);
-		glDepthMask(0);
+		glDepthMask(GL_FALSE);
 
 		// note: nano-particles (CGfxProjectile instances) also
 		// contribute to the count, but have their own creation
@@ -713,7 +704,7 @@ void CProjectileDrawer::Draw(bool drawReflection, bool drawRefraction) {
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	glDisable(GL_ALPHA_TEST);
 	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
-	glDepthMask(1);
+	glDepthMask(GL_TRUE);
 
 	ph->currentParticles  = int(ph->currentParticles * 0.2f);
 	ph->currentParticles += int(0.2f * drawnPieces + 0.3f * numFlyingPieces);
@@ -724,11 +715,10 @@ void CProjectileDrawer::Draw(bool drawReflection, bool drawRefraction) {
 		ph->currentParticles += (modelRenderers[modelType]->GetNumProjectiles() * 0.8f);
 	}
 
-	ph->particleSaturation     = ph->currentParticles     / float(ph->maxParticles);
-	ph->nanoParticleSaturation = ph->currentNanoParticles / float(ph->maxNanoParticles);
+	ph->UpdateParticleSaturation();
 }
 
-void CProjectileDrawer::DrawShadowPass(void)
+void CProjectileDrawer::DrawShadowPass()
 {
 	Shader::IProgramObject* po =
 		shadowHandler->GetShadowGenProg(CShadowHandler::SHADOWGEN_PROGRAM_PROJECTILE);
@@ -741,7 +731,7 @@ void CProjectileDrawer::DrawShadowPass(void)
 	CProjectile::va->Initialize();
 
 	{
-		GML_STDMUTEX_LOCK(proj); // DrawShadowPass
+		GML_RECMUTEX_LOCK(proj); // DrawShadowPass
 
 		for (int modelType = MODELTYPE_3DO; modelType < MODELTYPE_OTHER; modelType++) {
 			DrawProjectilesShadow(modelType);
@@ -867,32 +857,27 @@ bool CProjectileDrawer::DrawProjectileModel(const CProjectile* p, bool shadowPas
 		if (pp->alphaThreshold != 0.1f) {
 			glPopAttrib();
 		}
-
-		*(pp->numCallback) = 0;
 	}
 
 	return true;
 }
 
-void CProjectileDrawer::DrawGroundFlashes(void)
+void CProjectileDrawer::DrawGroundFlashes()
 {
 	static const GLfloat black[] = {0.0f, 0.0f, 0.0f, 0.0f};
 
+	glDepthMask(GL_FALSE);
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
 	glActiveTexture(GL_TEXTURE0);
 	groundFXAtlas->BindTexture();
 	glEnable(GL_TEXTURE_2D);
-	glDepthMask(GL_FALSE);
 	glEnable(GL_ALPHA_TEST);
 	glAlphaFunc(GL_GREATER, 0.01f);
 	glPolygonOffset(-20, -1000);
 	glEnable(GL_POLYGON_OFFSET_FILL);
 	glFogfv(GL_FOG_COLOR, black);
 
-	CGroundFlash::va = GetVertexArray();
-	CGroundFlash::va->Initialize();
-
 	{
 		GML_STDMUTEX_LOCK(rflash); // DrawGroundFlashes
 
@@ -900,26 +885,57 @@ void CProjectileDrawer::DrawGroundFlashes(void)
 		ph->groundFlashes.add_delayed();
 	}
 
-	CGroundFlash::va->EnlargeArrays(8 * ph->groundFlashes.render_size(), 0, VA_SIZE_TC);
+	CGroundFlash::va = GetVertexArray();
+	CGroundFlash::va->Initialize();
+	CGroundFlash::va->EnlargeArrays(8, 0, VA_SIZE_TC);
 
+	GroundFlashContainer& gfc = ph->groundFlashes;
 	GroundFlashContainer::render_iterator gfi;
-	for (gfi = ph->groundFlashes.render_begin(); gfi != ph->groundFlashes.render_end(); ++gfi) {
-		if (
-			((*gfi)->alwaysVisible || 
-				gu->spectatingFullView ||
-				loshandler->InAirLos((*gfi)->pos,gu->myAllyTeam)
-			) && camera->InView((*gfi)->pos, (*gfi)->size)
-		)
-			(*gfi)->Draw();
+
+	bool depthTest = true;
+	bool depthMask = false;
+
+	for (gfi = gfc.render_begin(); gfi != gfc.render_end(); ++gfi) {
+		CGroundFlash* gf = *gfi;
+
+		const bool los = gu->spectatingFullView || loshandler->InAirLos(gf->pos, gu->myAllyTeam);
+		const bool vis = camera->InView(gf->pos, gf->size);
+
+		if (depthTest != gf->depthTest) {
+			depthTest = gf->depthTest;
+
+			if (depthTest) {
+				glEnable(GL_DEPTH_TEST);
+			} else {
+				glDisable(GL_DEPTH_TEST);
+			}
+		}
+		if (depthMask != gf->depthMask) {
+			depthMask = gf->depthMask;
+
+			if (depthMask) {
+				glDepthMask(GL_TRUE);
+			} else {
+				glDepthMask(GL_FALSE);
+			}
+		}
+
+		if ((gf->alwaysVisible || los) && vis) {
+			gf->Draw();
+		}
+
+		CGroundFlash::va->DrawArrayTC(GL_QUADS);
+		CGroundFlash::va->Initialize();
 	}
 
-	CGroundFlash::va->DrawArrayTC(GL_QUADS);
 
 	glFogfv(GL_FOG_COLOR, mapInfo->atmosphere.fogColor);
 	glDisable(GL_POLYGON_OFFSET_FILL);
 	glDisable(GL_ALPHA_TEST);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	glDisable(GL_BLEND);
+	glEnable(GL_DEPTH_TEST);
+	glDepthMask(GL_TRUE);
 }
 
 
@@ -1040,8 +1056,10 @@ void CProjectileDrawer::GenerateNoiseTex(unsigned int tex, int size)
 
 void CProjectileDrawer::RenderProjectileCreated(const CProjectile* p)
 {
+	texturehandlerS3O->UpdateDraw();
+
 #if defined(USE_GML) && GML_ENABLE_SIM
-	if(p->model && TEX_TYPE(p) < 0)
+	if(!gmlShareLists && p->model && TEX_TYPE(p) < 0)
 		TEX_TYPE(p) = texturehandlerS3O->LoadS3OTextureNow(p->model);
 #endif
 
