@@ -1,19 +1,17 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "Feature.h"
 #include "FeatureHandler.h"
-#include "LogOutput.h"
+#include "Game/GlobalUnsynced.h"
 #include "Lua/LuaRules.h"
 #include "Map/Ground.h"
 #include "Map/ReadMap.h"
 #include "Map/MapInfo.h"
-#include "myMath.h"
 #include "Sim/Misc/DamageArray.h"
 #include "Sim/Misc/QuadField.h"
-#include "Rendering/Env/BaseTreeDrawer.h"
+#include "Rendering/Env/ITreeDrawer.h"
 #include "Rendering/Models/3DModel.h"
 #include "Sim/Misc/CollisionVolume.h"
 #include "Sim/Misc/ModInfo.h"
@@ -25,62 +23,62 @@
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/Unit.h"
 #include "System/EventHandler.h"
-#include "GlobalUnsynced.h"
+#include "System/Log/ILog.h"
+#include "System/myMath.h"
 #include <assert.h>
 
 CR_BIND_DERIVED(CFeature, CSolidObject, )
 
 CR_REG_METADATA(CFeature, (
-				//CR_MEMBER(model),
-				CR_MEMBER(createdFromUnit),
-				CR_MEMBER(isRepairingBeforeResurrect),
-				CR_MEMBER(resurrectProgress),
-				CR_MEMBER(health),
-				CR_MEMBER(reclaimLeft),
-				CR_MEMBER(luaDraw),
-				CR_MEMBER(noSelect),
-				CR_MEMBER(tempNum),
-				CR_MEMBER(lastReclaim),
-//				CR_MEMBER(def),
-				CR_MEMBER(defName),
-				CR_MEMBER(transMatrix),
-				CR_MEMBER(inUpdateQue),
-				CR_MEMBER(drawQuad),
-				CR_MEMBER(finalHeight),
-				CR_MEMBER(myFire),
-				CR_MEMBER(fireTime),
-				CR_MEMBER(emitSmokeTime),
-				CR_RESERVED(64),
-				CR_POSTLOAD(PostLoad)
-				));
+	// CR_MEMBER(model),
+	CR_MEMBER(isRepairingBeforeResurrect),
+	CR_MEMBER(resurrectProgress),
+	CR_MEMBER(health),
+	CR_MEMBER(reclaimLeft),
+	CR_MEMBER(luaDraw),
+	CR_MEMBER(noSelect),
+	CR_MEMBER(tempNum),
+	CR_MEMBER(lastReclaim),
+	// CR_MEMBER(def),
+	// CR_MEMBER(udef),
+	CR_MEMBER(defName),
+	CR_MEMBER(transMatrix),
+	CR_MEMBER(inUpdateQue),
+	CR_MEMBER(drawQuad),
+	CR_MEMBER(finalHeight),
+	CR_MEMBER(myFire),
+	CR_MEMBER(fireTime),
+	CR_MEMBER(emitSmokeTime),
+	CR_RESERVED(64),
+	CR_POSTLOAD(PostLoad)
+));
 
 
-CFeature::CFeature():
+CFeature::CFeature() : CSolidObject(),
 	isRepairingBeforeResurrect(false),
-	resurrectProgress(0),
-	health(0),
-	reclaimLeft(1),
+	resurrectProgress(0.0f),
+	health(0.0f),
+	reclaimLeft(1.0f),
 	luaDraw(false),
 	noSelect(false),
 	tempNum(0),
 	lastReclaim(0),
-	def(0),
-	collisionVolume(0),
+	def(NULL),
+	udef(NULL),
 	inUpdateQue(false),
 	drawQuad(-2),
-	finalHeight(0),
-	reachedFinalPos(false),
-	myFire(0),
+	finalHeight(0.0f),
+	reachedFinalPos(true),
+	myFire(NULL),
 	fireTime(0),
 	emitSmokeTime(0),
-	solidOnTop(0),
-	tempalpha(1)
+	solidOnTop(NULL)
 {
 	immobile = true;
 	physicalState = OnGround;
 }
 
-CFeature::~CFeature(void)
+CFeature::~CFeature()
 {
 	if (blocking) {
 		UnBlock();
@@ -88,7 +86,7 @@ CFeature::~CFeature(void)
 
 	qf->RemoveFeature(this);
 
-	if (def->drawType >= DRAWTYPE_TREE) {
+	if (def->drawType >= DRAWTYPE_TREE && treeDrawer) {
 		treeDrawer->DeleteTree(pos);
 	}
 
@@ -100,24 +98,25 @@ CFeature::~CFeature(void)
 	if (def->geoThermal) {
 		CGeoThermSmokeProjectile::GeoThermDestroyed(this);
 	}
-
-	delete collisionVolume; collisionVolume = NULL;
 }
 
 void CFeature::PostLoad()
 {
 	def = featureHandler->GetFeatureDef(defName);
+
+	//FIXME is this really needed (aren't all those tags saved via creg?)
 	if (def->drawType == DRAWTYPE_MODEL) {
 		model = def->LoadModel();
 		height = model->height;
 		SetRadius(model->radius);
-		midPos = pos + model->relMidPos;
+		relMidPos = model->relMidPos;
 	} else if (def->drawType >= DRAWTYPE_TREE) {
-		midPos = pos + (UpVector * TREE_RADIUS);
+		relMidPos = UpVector * TREE_RADIUS;
 		height = 2 * TREE_RADIUS;
 	} else {
-		midPos = pos;
+		relMidPos = ZeroVector;
 	}
+	midPos = pos + relMidPos;
 }
 
 
@@ -134,17 +133,17 @@ void CFeature::ChangeTeam(int newTeam)
 
 
 void CFeature::Initialize(const float3& _pos, const FeatureDef* _def, short int _heading,
-	int facing, int _team, int _allyteam, std::string fromUnit, const float3& speed, int _smokeTime)
+	int facing, int _team, int _allyteam, const UnitDef* _udef, const float3& speed, int _smokeTime)
 {
 	pos = _pos;
 	def = _def;
+	udef = _udef;
 	defName = def->myName;
 	heading = _heading;
 	buildFacing = facing;
 	team = _team;
 	allyteam = _allyteam;
 	emitSmokeTime = _smokeTime;
-	createdFromUnit = fromUnit;
 
 	ChangeTeam(team); // maybe should not be here, but it prevents crashes caused by team = -1
 
@@ -160,29 +159,33 @@ void CFeature::Initialize(const float3& _pos, const FeatureDef* _def, short int
 	if (def->drawType == DRAWTYPE_MODEL) {
 		model = def->LoadModel();
 		if (!model) {
-			logOutput.Print("Features: Couldn't load model for " + defName);
+			LOG_L(L_ERROR, "Features: Couldn't load model for %s", defName.c_str());
 			SetRadius(0.0f);
-			midPos = pos;
+			relMidPos = ZeroVector;
 		} else {
 			height = model->height;
 			SetRadius(model->radius);
-			midPos = pos + model->relMidPos;
+			relMidPos = model->relMidPos;
+
+			// note: gets deleted in ~CSolidObject
 			collisionVolume = new CollisionVolume(def->collisionVolume, model->radius);
 		}
 	}
 	else if (def->drawType >= DRAWTYPE_TREE) {
 		SetRadius(TREE_RADIUS);
-		midPos = pos + (UpVector * TREE_RADIUS);
+		relMidPos = UpVector * TREE_RADIUS;
 		height = 2 * TREE_RADIUS;
 
 		// LoadFeaturesFromMap() doesn't set a scale for trees
+		// note: gets deleted in ~CSolidObject
 		collisionVolume = new CollisionVolume(def->collisionVolume, TREE_RADIUS);
 	}
 	else {
 		// geothermal (no collision volume)
 		SetRadius(0.0f);
-		midPos = pos;
+		relMidPos = ZeroVector;
 	}
+	midPos = pos + relMidPos;
 
 	featureHandler->AddFeature(this);
 	qf->AddFeature(this);
@@ -194,14 +197,16 @@ void CFeature::Initialize(const float3& _pos, const FeatureDef* _def, short int
 	}
 
 	if (def->floating) {
-		finalHeight = ground->GetHeight(pos.x, pos.z);
+		finalHeight = ground->GetHeightAboveWater(pos.x, pos.z);
 	} else {
-		finalHeight = ground->GetHeight2(pos.x, pos.z);
+		finalHeight = ground->GetHeightReal(pos.x, pos.z);
 	}
 
 	if (speed != ZeroVector) {
 		deathSpeed = speed;
 	}
+
+	reachedFinalPos = (speed == ZeroVector && pos.y == finalHeight);
 }
 
 
@@ -228,7 +233,7 @@ bool CFeature::AddBuildPower(float amount, CUnit* builder)
 
 	if (amount > 0.0f) {
 		// Check they are trying to repair a feature that can be resurrected
-		if (createdFromUnit == "") {
+		if (udef == NULL) {
 			return false;
 		}
 
@@ -366,7 +371,7 @@ void CFeature::DoDamage(const DamageArray& damages, const float3& impulse)
 	if (health <= 0 && def->destructable) {
 		CFeature* deathFeature = featureHandler->CreateWreckage(
 			pos, def->deathFeature, heading,
-			buildFacing, 1, team, -1, false, ""
+			buildFacing, 1, team, -1, false, NULL
 		);
 
 		if (deathFeature) {
@@ -388,9 +393,11 @@ void CFeature::DoDamage(const DamageArray& damages, const float3& impulse)
 }
 
 
-void CFeature::Kill(float3& impulse) {
+void CFeature::Kill(const float3& impulse, bool crushKill) {
+	crushKilled = crushKill;
+
 	DamageArray damage;
-	DoDamage(damage * (health + 1), impulse);
+	DoDamage(damage * (health + 1.0f), impulse);
 }
 
 
@@ -422,22 +429,16 @@ void CFeature::ForcedMove(const float3& newPos, bool snapToGround)
 	// setup finalHeight
 	if (snapToGround) {
 		if (def->floating) {
-			finalHeight = ground->GetHeight(pos.x, pos.z);
+			finalHeight = ground->GetHeightAboveWater(pos.x, pos.z);
 		} else {
-			finalHeight = ground->GetHeight2(pos.x, pos.z);
+			finalHeight = ground->GetHeightReal(pos.x, pos.z);
 		}
 	} else {
 		finalHeight = newPos.y;
 	}
 
 	// setup midPos
-	if (def->drawType == DRAWTYPE_MODEL) {
-		midPos = pos + model->relMidPos;
-	} else if (def->drawType >= DRAWTYPE_TREE) {
-		midPos = pos + (UpVector * TREE_RADIUS);
-	} else {
-		midPos = pos;
-	}
+	midPos = pos + relMidPos;
 
 	// setup the visual transformation matrix
 	CalculateTransform();
@@ -471,15 +472,9 @@ void CFeature::ForcedSpin(const float3& newDir)
 
 bool CFeature::UpdatePosition()
 {
-	bool finishedUpdate = true;
-
-	if (!createdFromUnit.empty()) {
+	if (udef != NULL) {
 		// we are a wreck of a dead unit
 		if (!reachedFinalPos) {
-			bool haveForwardSpeed = false;
-			bool haveVerticalSpeed = false;
-			bool inBounds = false;
-
 			// NOTE: apply more drag if we were a tank or bot?
 			// (would require passing extra data to Initialize())
 			deathSpeed *= 0.95f;
@@ -489,21 +484,22 @@ bool CFeature::UpdatePosition()
 				qf->RemoveFeature(this);
 
 				// update our forward speed (and quadfield
-				// position) if it's still greater than 0
+				// position) if it is still greater than 0
 				pos += deathSpeed;
 				midPos += deathSpeed;
 
-				haveForwardSpeed = true;
-
 				qf->AddFeature(this);
 				Block();
+			} else {
+				deathSpeed.x = 0.0f;
+				deathSpeed.z = 0.0f;
 			}
 
 			// def->floating is unreliable (true for land unit wrecks),
 			// just assume wrecks always sink even if their "owner" was
 			// a floating object (as is the case for ships anyway)
-			float realGroundHeight = ground->GetHeight2(pos.x, pos.z);
-			bool reachedGround = (pos.y <= realGroundHeight);
+			const float realGroundHeight = ground->GetHeightReal(pos.x, pos.z);
+			const bool reachedGround = (pos.y <= realGroundHeight);
 
 			if (!reachedGround) {
 				if (pos.y > 0.0f) {
@@ -516,21 +512,19 @@ bool CFeature::UpdatePosition()
 
 				pos.y += deathSpeed.y;
 				midPos.y += deathSpeed.y;
-				haveVerticalSpeed = true;
 			} else {
+				deathSpeed.y = 0.0f;
+
 				// last Update() may have sunk us into
 				// ground if pos.y was only marginally
 				// larger than ground height, correct
 				pos.y = realGroundHeight;
 				midPos.y = pos.y + model->relMidPos.y;
-				deathSpeed.y = 0.0f;
 			}
 
-			inBounds = pos.CheckInBounds();
-			reachedFinalPos = (!haveForwardSpeed && !haveVerticalSpeed);
-			// reachedFinalPos = ((!haveForwardSpeed && !haveVerticalSpeed) || !inBounds);
+			reachedFinalPos = (deathSpeed == ZeroVector);
 
-			if (!inBounds) {
+			if (!pos.CheckInBounds()) {
 				// ensure that no more forward-speed updates are done
 				// (prevents wrecks floating in mid-air at edge of map
 				// due to gravity no longer being applied either)
@@ -541,9 +535,6 @@ bool CFeature::UpdatePosition()
 
 			CalculateTransform();
 		}
-
-		if (!reachedFinalPos)
-			finishedUpdate = false;
 	} else {
 		if (pos.y > finalHeight) {
 			//! feature is falling
@@ -566,28 +557,28 @@ bool CFeature::UpdatePosition()
 			if (def->drawType >= DRAWTYPE_TREE)
 				treeDrawer->DeleteTree(pos);
 
-			float diff = finalHeight - pos.y;
+			const float diff = finalHeight - pos.y;
+
 			pos.y = finalHeight;
-			midPos.y += diff;
-			transMatrix[13] += diff;
 			speed.y = 0.0f;
 
+			midPos = pos + relMidPos;
+			transMatrix[13] += diff;
+
 			if (def->drawType >= DRAWTYPE_TREE)
 				treeDrawer->AddTree(def->drawType - 1, pos, 1.0f);
 		}
 
-		if (pos.y != finalHeight)
-			finishedUpdate = false;
+		reachedFinalPos = (pos.y == finalHeight);
 	}
 
 	isUnderWater = ((pos.y + height) < 0.0f);
-	return finishedUpdate;
+	return reachedFinalPos;
 }
 
-bool CFeature::Update(void)
+bool CFeature::Update()
 {
-	bool finishedUpdate = true;
-	finishedUpdate = UpdatePosition();
+	bool finishedUpdate = UpdatePosition();
 
 	if (emitSmokeTime != 0) {
 		--emitSmokeTime;
@@ -609,11 +600,11 @@ bool CFeature::Update(void)
 	if (def->geoThermal) {
 		if ((gs->frameNum + id % 5) % 5 == 0) {
 			// Find the unit closest to the geothermal
-			vector<CSolidObject*> objs = qf->GetSolidsExact(pos, 0.0f);
+			const vector<CSolidObject*> &objs = qf->GetSolidsExact(pos, 0.0f);
 			float bestDist2 = 0;
 			CSolidObject* so = NULL;
 
-			for (vector<CSolidObject*>::iterator oi = objs.begin(); oi != objs.end(); ++oi) {
+			for (vector<CSolidObject*>::const_iterator oi = objs.begin(); oi != objs.end(); ++oi) {
 				float dist2 = ((*oi)->pos - pos).SqLength();
 				if (!so || dist2 < bestDist2)  {
 					bestDist2 = dist2;
@@ -623,9 +614,9 @@ bool CFeature::Update(void)
 
 			if (so != solidOnTop) {
 				if (solidOnTop)
-					DeleteDeathDependence(solidOnTop);
+					DeleteDeathDependence(solidOnTop, DEPENDENCE_SOLIDONTOP);
 				if (so)
-					AddDeathDependence(so);
+					AddDeathDependence(so, DEPENDENCE_SOLIDONTOP);
 			}
 			solidOnTop = so;
 		}
@@ -649,7 +640,7 @@ bool CFeature::Update(void)
 }
 
 
-void CFeature::StartFire(void)
+void CFeature::StartFire()
 {
 	if (fireTime || !def->burnable)
 		return;
