@@ -1,7 +1,6 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "CollisionHandler.h"
 #include "CollisionVolume.h"
@@ -108,13 +107,8 @@ bool CCollisionHandler::Collision(const CFeature* f, const float3& p)
 			return CCollisionHandler::CollisionFootprint(f, p);
 		}
 		default: {
-			// NOTE: CFeature does not have a relMidPos member so
-			// calculate and apply the translation from pos (used
-			// by transMatrix) to midPos manually
-			const float3 relMidPos(f->midPos - f->pos);
-
 			CMatrix44f m(f->transMatrix);
-			m.Translate(relMidPos);
+			m.Translate(f->relMidPos);
 			m.Translate(v->GetOffsets());
 
 			return CCollisionHandler::Collision(v, m, p);
@@ -141,8 +135,6 @@ bool CCollisionHandler::CollisionFootprint(const CSolidObject* o, const float3&
 }
 
 
-// test if point <p> (in world-coors) lies inside the
-// volume whose transformation matrix is given by <m>
 bool CCollisionHandler::Collision(const CollisionVolume* v, const CMatrix44f& m, const float3& p)
 {
 	// get the inverse volume transformation matrix and
@@ -218,25 +210,26 @@ void CCollisionHandler::IntersectPieceTreeHelper(
 	const float3& p1,
 	std::list<CollisionQuery>* hits)
 {
-	const CollisionVolume* vol = lmp->colvol;
+	const CollisionVolume* vol = lmp->GetCollisionVolume();
 	const float3& offset = vol->GetOffsets();
+	const float3& rot = lmp->GetRotation();
 
-	mat.Translate(lmp->pos);
-	mat.RotateY(-lmp->rot[1]);
-	mat.RotateX(-lmp->rot[0]);
-	mat.RotateZ(-lmp->rot[2]);
-	mat.Translate(offset);
+	mat.Translate(lmp->GetPosition());
+	mat.RotateY(-rot[1]);
+	mat.RotateX(-rot[0]);
+	mat.RotateZ(-rot[2]);
 
 	if (lmp->visible && !vol->IsDisabled()) {
-		CollisionQuery q;
+		mat.Translate(offset);
 
+		CollisionQuery q;
 		if (CCollisionHandler::Intersect(vol, mat, p0, p1, &q)) {
 			q.lmp = lmp;
 			hits->push_back(q);
 		}
-	}
 
-	mat.Translate(-offset);
+		mat.Translate(-offset);
+	}
 
 	for (unsigned int i = 0; i < lmp->childs.size(); i++) {
 		IntersectPieceTreeHelper(lmp->childs[i], mat, p0, p1, hits);
@@ -252,12 +245,12 @@ bool CCollisionHandler::IntersectPieceTree(const CUnit* u, const float3& p0, con
 	CMatrix44f mat = u->GetTransformMatrix(true);
 	mat.Translate(u->relMidPos * float3(-1.0f, 0.0f, 1.0f));
 
-	IntersectPieceTreeHelper(u->localmodel->pieces[0], mat, p0, p1, &hits);
+	IntersectPieceTreeHelper(u->localmodel->GetRoot(), mat, p0, p1, &hits);
 
 	float dstNearSq = 1e30f;
 
 	// save the closest intersection
-	for (hitsIt = hits.begin(); hitsIt != hits.end(); hitsIt++) {
+	for (hitsIt = hits.begin(); hitsIt != hits.end(); ++hitsIt) {
 		const CollisionQuery& qTmp = *hitsIt;
 		const float dstSq = (qTmp.p0 - p0).SqLength();
 
@@ -315,9 +308,6 @@ bool CCollisionHandler::IntersectAlt(const collisionVolume* d, const CMatrix44f&
 */
 
 
-
-// test if ray from <p0> to <p1> (in world-coors) intersects
-// the volume whose transformation matrix is given by <m>
 bool CCollisionHandler::Intersect(const CollisionVolume* v, const CMatrix44f& m, const float3& p0, const float3& p1, CollisionQuery* q)
 {
 	if (q) {
@@ -557,39 +547,50 @@ bool CCollisionHandler::IntersectCylinder(const CollisionVolume* v, const float3
 		} break;
 	}
 
-	float d = (b * b) - (4.0f * a * c);
-	float rd = 0.0f;
-	float dp = 0.0f, rdp = 0.0f;
-
 	// volume-space intersection points
 	float3 p0 = ZVEC;
 	float3 p1 = ZVEC;
 
 	bool b0 = false;
 	bool b1 = false;
-	float r0 = 0.0f, s0 = 0.0f, t0 = 0.0f;
-	float r1 = 0.0f, s1 = 0.0f, t1 = 0.0f;
+	float
+		d = (b * b) - (4.0f * a * c),
+		rd = 0.0f, // sqrt(d) or 1/dp
+		dp = 0.0f, // dot(n{0, 1}, dir)
+		ra = 0.0f; // ellipsoid ratio of p{0, 1}
+	float s0 = 0.0f, t0 = 0.0f;
+	float s1 = 0.0f, t1 = 0.0f;
 
 	// get the length of the ray segment in volume-space
 	const float segLenSq = (pi1 - pi0).SqLength();
 
 	if (d >= -EPS) {
-		// one or two surface intersections
-		if (d < EPS) {
-			t0 = -b / (2.0f * a);
-			p0 = (pii0 + (diir * t0)) * inv;
-			s0 = (p0 - pi0).SqLength();
-			b0 = (s0 < segLenSq && (p0[pAx] > -ahs[pAx] && p0[pAx] < ahs[pAx]));
+		if (a != 0.0f) {
+			// quadratic eq.; one or two surface intersections
+			if (d < EPS) {
+				t0 = -b / (2.0f * a);
+				p0 = (pii0 + (diir * t0)) * inv;
+				s0 = (p0 - pi0).SqLength();
+				b0 = (s0 < segLenSq && (p0[pAx] > -ahs[pAx] && p0[pAx] < ahs[pAx]));
+			} else {
+				rd = fastmath::apxsqrt(d);
+				t0 = (-b - rd) / (2.0f * a);
+				t1 = (-b + rd) / (2.0f * a);
+				p0 = (pii0 + (diir * t0)) * inv;
+				p1 = (pii0 + (diir * t1)) * inv;
+				s0 = (p0 - pi0).SqLength();
+				s1 = (p1 - pi0).SqLength();
+				b0 = (s0 < segLenSq && (p0[pAx] > -ahs[pAx] && p0[pAx] < ahs[pAx]));
+				b1 = (s1 < segLenSq && (p1[pAx] > -ahs[pAx] && p1[pAx] < ahs[pAx]));
+			}
 		} else {
-			rd = fastmath::apxsqrt(d);
-			t0 = (-b - rd) / (2.0f * a);
-			t1 = (-b + rd) / (2.0f * a);
-			p0 = (pii0 + (diir * t0)) * inv;
-			p1 = (pii0 + (diir * t1)) * inv;
-			s0 = (p0 - pi0).SqLength();
-			s1 = (p1 - pi0).SqLength();
-			b0 = (s0 < segLenSq && (p0[pAx] > -ahs[pAx] && p0[pAx] < ahs[pAx]));
-			b1 = (s1 < segLenSq && (p1[pAx] > -ahs[pAx] && p1[pAx] < ahs[pAx]));
+			if (b != 0.0f) {
+				// linear eq.; one surface intersection
+				t0 = -c / b;
+				p0 = (pii0 + (diir * t0)) * inv;
+				s0 = (p0 - pi0).SqLength();
+				b0 = (s0 < segLenSq && (p0[pAx] > -ahs[pAx] && p0[pAx] < ahs[pAx]));
+			}
 		}
 	}
 
@@ -599,15 +600,15 @@ bool CCollisionHandler::IntersectCylinder(const CollisionVolume* v, const float3
 		// through front cap (plane)
 		// NOTE: normal n0 and dir should not be orthogonal
 		dp = n0.dot(dir);
-		rdp = (dp != 0.0f)? 1.0f / dp: 0.01f;
+		rd = (dp != 0.0f)? 1.0f / dp: 0.01f;
 
-		t0 = -(n0.dot(pi0) - ahs[pAx]) * rdp;
+		t0 = -(n0.dot(pi0) - ahs[pAx]) * rd;
 		p0 = pi0 + (dir * t0);
 		s0 = (p0 - pi0).SqLength();
-		r0 =
+		ra =
 			(((p0[sAx0] * p0[sAx0]) / ahsq[sAx0]) +
 			 ((p0[sAx1] * p0[sAx1]) / ahsq[sAx1]));
-		b0 = (t0 >= 0.0f && r0 <= 1.0f && s0 <= segLenSq);
+		b0 = (t0 >= 0.0f && ra <= 1.0f && s0 <= segLenSq);
 	}
 	if (!b1) {
 		// p1 does not lie on ray segment, or does not fall
@@ -615,15 +616,15 @@ bool CCollisionHandler::IntersectCylinder(const CollisionVolume* v, const float3
 		// through rear cap (plane)
 		// NOTE: normal n1 and dir should not be orthogonal
 		dp = n1.dot(dir);
-		rdp = (dp != 0.0f)? 1.0f / dp: 0.01f;
+		rd = (dp != 0.0f)? 1.0f / dp: 0.01f;
 
-		t1 = -(n1.dot(pi0) - ahs[pAx]) * rdp;
+		t1 = -(n1.dot(pi0) - ahs[pAx]) * rd;
 		p1 = pi0 + (dir * t1);
 		s1 = (p1 - pi0).SqLength();
-		r1 =
+		ra =
 			(((p1[sAx0] * p1[sAx0]) / ahsq[sAx0]) +
 			 ((p1[sAx1] * p1[sAx1]) / ahsq[sAx1]));
-		b1 = (t1 >= 0.0f && r1 <= 1.0f && s1 <= segLenSq);
+		b1 = (t1 >= 0.0f && ra <= 1.0f && s1 <= segLenSq);
 	}
 
 	if (q) {
