@@ -1,12 +1,12 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef GLOBAL_UNSYNCED_H
-#define GLOBAL_UNSYNCED_H
+#ifndef _GLOBAL_SYNCED_H
+#define _GLOBAL_SYNCED_H
 
 #include <string>
 
-#include "float3.h"
-#include "creg/creg_cond.h"
+#include "System/float3.h"
+#include "System/creg/creg_cond.h"
 
 
 class CGameSetup;
@@ -75,6 +75,8 @@ public:
 	* (note that the number of vertices is one more)
 	*/
 	int mapx;
+	int mapxm1; // mapx minus one
+	int mapxp1; // mapx plus one
 
 	/**
 	* @brief map y
@@ -82,6 +84,8 @@ public:
 	* The map's number of squares in the y direction
 	*/
 	int mapy;
+	int mapym1; // mapy minus one
+	int mapyp1; // mapy plus one
 
 	/**
 	* @brief map squares
@@ -131,7 +135,8 @@ public:
 	/**
 	* @brief god mode
 	*
-	* Whether god mode is enabled, allows all players to control all units (even specs)
+	* Whether god-mode is enabled, which allows all players (even spectators)
+	* to control all units.
 	*/
 	bool godMode;
 
@@ -159,7 +164,7 @@ public:
 	/**
 	* @brief definition editing enabled
 	*
-	* Whether definition editing is enabled
+	* Whether editing of unit-, feature- and weapon-defs through Lua is enabled.
 	*/
 	bool editDefsEnabled;
 
@@ -198,4 +203,4 @@ public:
 	};
 };
 
-#endif //GLOBAL_UNSYNCED_H
+#endif // _GLOBAL_SYNCED_H
