@@ -11,6 +11,7 @@ public:
 	CModInfo()
 		: allowTeamColors(true)
 		, allowAirPlanesToLeaveMap(true)
+		, allowPushingEnemyUnits(false)
 		, constructionDecay(true)
 		, constructionDecayTime(1000)
 		, constructionDecaySpeed(1.0f)
@@ -39,6 +40,7 @@ public:
 		, airLosMul(1.0f)
 		, requireSonarUnderWater(true)
 		, featureVisibility(FEATURELOS_NONE)
+		, luaThreadingModel(2)
 	{}
 	~CModInfo() {}
 
@@ -73,6 +75,7 @@ public:
 
 	// Movement behaviour
 	bool allowAirPlanesToLeaveMap;
+	bool allowPushingEnemyUnits;
 
 	// Build behaviour
 	/// Should constructions without builders decay?
@@ -153,9 +156,10 @@ public:
 	/// feature visibility style: 0 - no LOS for features, 1 - gaia features visible
 	/// 2 - gaia/allied features visible, 3 - all features visible
 	int featureVisibility;
+	// Lua threading model: Controls which Lua MT optimizations the mod will use by default (see LuaConfig.h for details)
+	int luaThreadingModel;
 };
 
 extern CModInfo modInfo;
 
-
-#endif
+#endif // MOD_INFO_H
