@@ -1,203 +1,172 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
-
 #include "Team.h"
-#include "TeamHandler.h"
 
+#include "TeamHandler.h"
 #include "GlobalSynced.h"
-#include "LogOutput.h"
+#include "ExternalAI/SkirmishAIHandler.h"
+#include "Game/Player.h"
 #include "Game/PlayerHandler.h"
 #include "Game/GameSetup.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/Messages.h"
-#include "GlobalUnsynced.h"
 #include "Game/UI/LuaUI.h"
 #include "Lua/LuaRules.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/UnitDef.h"
-#include "ExternalAI/SkirmishAIHandler.h"
-#include "EventHandler.h"
-#include "GlobalUnsynced.h"
-#include "creg/STL_List.h"
-#include "creg/STL_Map.h"
-#include "creg/STL_Set.h"
-#include "NetProtocol.h"
+#include "System/EventHandler.h"
+#include "System/Log/ILog.h"
+#include "System/NetProtocol.h"
+#include "System/mmgr.h"
+#include "System/creg/STL_List.h"
+#include "System/creg/STL_Map.h"
+#include "System/creg/STL_Set.h"
 
 CR_BIND(CTeam,);
 CR_REG_METADATA(CTeam, (
-// from CTeamBase
-				CR_MEMBER(leader),
-				CR_MEMBER(color),
-				CR_MEMBER(handicap),
-				CR_MEMBER(side),
-				CR_MEMBER(startPos),
-				CR_MEMBER(teamStartNum),
-				CR_MEMBER(teamAllyteam),
-//				CR_MEMBER(customValues),
-// from CTeam
-				CR_MEMBER(teamNum),
-				CR_MEMBER(isDead),
-				CR_MEMBER(gaia),
-				CR_MEMBER(lineageRoot),
-				CR_MEMBER(origColor),
-				CR_MEMBER(units),
-				CR_MEMBER(metal),
-				CR_MEMBER(energy),
-				CR_MEMBER(metalPull),
-				CR_MEMBER(prevMetalPull),
-				CR_MEMBER(metalIncome),
-				CR_MEMBER(prevMetalIncome),
-				CR_MEMBER(metalExpense),
-				CR_MEMBER(prevMetalExpense),
-				CR_MEMBER(metalUpkeep),
-				CR_MEMBER(prevMetalUpkeep),
-				CR_MEMBER(energyPull),
-				CR_MEMBER(prevEnergyPull),
-				CR_MEMBER(energyIncome),
-				CR_MEMBER(prevEnergyIncome),
-				CR_MEMBER(energyExpense),
-				CR_MEMBER(prevEnergyExpense),
-				CR_MEMBER(energyUpkeep),
-				CR_MEMBER(prevEnergyUpkeep),
-				CR_MEMBER(metalStorage),
-				CR_MEMBER(energyStorage),
-				CR_MEMBER(metalShare),
-				CR_MEMBER(energyShare),
-				CR_MEMBER(delayedMetalShare),
-				CR_MEMBER(delayedEnergyShare),
-				CR_MEMBER(metalSent),
-				CR_MEMBER(metalReceived),
-				CR_MEMBER(energySent),
-				CR_MEMBER(energyReceived),
-				//CR_MEMBER(currentStats),
-				CR_MEMBER(nextHistoryEntry),
-				CR_MEMBER(numCommanders),
-				//CR_MEMBER(statHistory),
-				CR_MEMBER(modParams),
-				CR_MEMBER(modParamsMap),
-				CR_RESERVED(64)
-				));
+	// from CTeamBase
+	CR_MEMBER(leader),
+	CR_MEMBER(color),
+	CR_MEMBER(incomeMultiplier),
+	CR_MEMBER(side),
+	CR_MEMBER(startPos),
+	CR_MEMBER(teamStartNum),
+	CR_MEMBER(teamAllyteam),
+	// CR_MEMBER(customValues),
+	// from CTeam
+	CR_MEMBER(teamNum),
+	CR_MEMBER(maxUnits),
+	CR_MEMBER(isDead),
+	CR_MEMBER(gaia),
+	CR_MEMBER(origColor),
+	CR_MEMBER(units),
+	CR_MEMBER(metal),
+	CR_MEMBER(energy),
+	CR_MEMBER(metalPull),
+	CR_MEMBER(prevMetalPull),
+	CR_MEMBER(metalIncome),
+	CR_MEMBER(prevMetalIncome),
+	CR_MEMBER(metalExpense),
+	CR_MEMBER(prevMetalExpense),
+	CR_MEMBER(energyPull),
+	CR_MEMBER(prevEnergyPull),
+	CR_MEMBER(energyIncome),
+	CR_MEMBER(prevEnergyIncome),
+	CR_MEMBER(energyExpense),
+	CR_MEMBER(prevEnergyExpense),
+	CR_MEMBER(metalStorage),
+	CR_MEMBER(energyStorage),
+	CR_MEMBER(metalShare),
+	CR_MEMBER(energyShare),
+	CR_MEMBER(delayedMetalShare),
+	CR_MEMBER(delayedEnergyShare),
+	CR_MEMBER(metalSent),
+	CR_MEMBER(metalReceived),
+	CR_MEMBER(energySent),
+	CR_MEMBER(energyReceived),
+	//CR_MEMBER(currentStats),
+	CR_MEMBER(nextHistoryEntry),
+	//CR_MEMBER(statHistory),
+	CR_MEMBER(modParams),
+	CR_MEMBER(modParamsMap),
+	CR_RESERVED(64)
+));
 
 
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
 //////////////////////////////////////////////////////////////////////
 
-CTeam::CTeam()
-: isDead(false),
-  gaia(false),
-  lineageRoot(-1),
-  metal(0),
-  energy(0),
-  metalPull(0),     prevMetalPull(0),
-  metalIncome(0),   prevMetalIncome(0),
-  metalExpense(0),  prevMetalExpense(0),
-  metalUpkeep(0),   prevMetalUpkeep(0),
-  energyPull(0),    prevEnergyPull(0),
-  energyIncome(0),  prevEnergyIncome(0),
-  energyExpense(0), prevEnergyExpense(0),
-  energyUpkeep(0),  prevEnergyUpkeep(0),
-  metalStorage(1000000),
-  energyStorage(1000000),
-  metalShare(0.99f),
-  energyShare(0.95f),
-  delayedMetalShare(0),
-  delayedEnergyShare(0),
-  metalSent(0),
-  metalReceived(0),
-  energySent(0),
-  energyReceived(0),
-  nextHistoryEntry(0),
-  numCommanders(0),
-  highlight(0.0f)
+CTeam::CTeam() :
+	teamNum(-1),
+	maxUnits(0),
+	isDead(false),
+	gaia(false),
+	metal(0.0f),
+	energy(0.0f),
+	metalPull(0.0f),     prevMetalPull(0.0f),
+	metalIncome(0.0f),   prevMetalIncome(0.0f),
+	metalExpense(0.0f),  prevMetalExpense(0.0f),
+	energyPull(0.0f),    prevEnergyPull(0.0f),
+	energyIncome(0.0f),  prevEnergyIncome(0.0f),
+	energyExpense(0.0f), prevEnergyExpense(0.0f),
+	metalStorage(1000000),
+	energyStorage(1000000),
+	metalShare(0.99f),
+	energyShare(0.95f),
+	delayedMetalShare(0.0f),
+	delayedEnergyShare(0.0f),
+	metalSent(0.0f),
+	metalReceived(0.0f),
+	energySent(0.0f),
+	energyReceived(0.0f),
+	nextHistoryEntry(0),
+	highlight(0.0f)
 {
+	origColor[0] = 0;
+	origColor[1] = 0;
+	origColor[2] = 0;
+	origColor[3] = 0;
+	
 	statHistory.push_back(TeamStatistics());
 	currentStats = &statHistory.back();
 }
 
 
-CTeam::~CTeam()
-{
-}
-
 
 bool CTeam::UseMetal(float amount)
-{
-	if ((metal - (prevMetalUpkeep * 10)) >= amount) {
-		metal -= amount;
-		metalExpense += amount;
-		return true;
-	}
-	return false;
-}
-
-
-bool CTeam::UseEnergy(float amount)
-{
-	if ((energy - (prevEnergyUpkeep * 10)) >= amount) {
-		energy -= amount;
-		energyExpense += amount;
-		return true;
-	}
-	return false;
-}
-
-
-bool CTeam::UseMetalUpkeep(float amount)
 {
 	if (metal >= amount) {
 		metal -= amount;
 		metalExpense += amount;
-		metalUpkeep += amount;
 		return true;
 	}
 	return false;
 }
 
-
-bool CTeam::UseEnergyUpkeep(float amount)
+bool CTeam::UseEnergy(float amount)
 {
 	if (energy >= amount) {
 		energy -= amount;
 		energyExpense += amount;
-		energyUpkeep += amount;
 		return true;
 	}
 	return false;
 }
 
 
-void CTeam::AddMetal(float amount, bool hc)
+
+void CTeam::AddMetal(float amount, bool useIncomeMultiplier)
 {
-	if (hc) { amount *= handicap; }
+	if (useIncomeMultiplier) { amount *= GetIncomeMultiplier(); }
 	metal += amount;
 	metalIncome += amount;
 	if (metal > metalStorage) {
-		delayedMetalShare += metal - metalStorage;
+		delayedMetalShare += (metal - metalStorage);
 		metal = metalStorage;
 	}
 }
 
-
-void CTeam::AddEnergy(float amount, bool hc)
+void CTeam::AddEnergy(float amount, bool useIncomeMultiplier)
 {
-	if (hc) { amount *= handicap; }
+	if (useIncomeMultiplier) { amount *= GetIncomeMultiplier(); }
 	energy += amount;
 	energyIncome += amount;
 	if (energy > energyStorage) {
-		delayedEnergyShare += energy - energyStorage;
+		delayedEnergyShare += (energy - energyStorage);
 		energy = energyStorage;
 	}
 }
 
+
+
 void CTeam::GiveEverythingTo(const unsigned toTeam)
 {
 	CTeam* target = teamHandler->Team(toTeam);
 
 	if (!target) {
-		logOutput.Print("Team %i does not exist, can't give units", toTeam);
+		LOG_L(L_WARNING, "Team %i does not exist, can't give units", toTeam);
 		return;
 	}
 
@@ -216,91 +185,102 @@ void CTeam::GiveEverythingTo(const unsigned toTeam)
 		(*ui)->ChangeTeam(toTeam, CUnit::ChangeGiven);
 		ui = next;
 	}
-
-	Died();
 }
 
 
-void CTeam::Died()
+void CTeam::Died(bool normalDeath)
 {
 	if (isDead)
 		return;
 
-	if (leader >= 0) {
-		logOutput.Print(CMessages::Tr("Team%i(%s) is no more").c_str(),
-		                teamNum, playerHandler->Player(leader)->name.c_str());
-	} else {
-		logOutput.Print(CMessages::Tr("Team%i is no more").c_str(), teamNum);
-	}
 	isDead = true;
 
-	CSkirmishAIHandler::ids_t localTeamAIs = skirmishAIHandler.GetSkirmishAIsInTeam(teamNum, gu->myPlayerNum);
-	for (CSkirmishAIHandler::ids_t::const_iterator ai = localTeamAIs.begin(); ai != localTeamAIs.end(); ++ai) {
-		skirmishAIHandler.SetLocalSkirmishAIDieing(*ai, 2 /* = team died */);
+	if (normalDeath) {
+		if (leader >= 0) {
+			const CPlayer* leadPlayer = playerHandler->Player(leader);
+			const char* leaderName = leadPlayer->name.c_str();
+			LOG(CMessages::Tr("Team %i (lead by %s) is no more").c_str(), teamNum, leaderName);
+		} else {
+			LOG(CMessages::Tr("Team %i is no more").c_str(), teamNum);
+		}
+
+		// this message is not relayed to clients, it's only for the server
+		net->Send(CBaseNetProtocol::Get().SendTeamDied(gu->myPlayerNum, teamNum));
 	}
 
-	// this message is not relayed to clients, it's only for the server
-	net->Send(CBaseNetProtocol::Get().SendTeamDied(gu->myPlayerNum, teamNum));
+	KillAIs();
 
+	// demote all players in _this_ team to spectators
 	for (int a = 0; a < playerHandler->ActivePlayers(); ++a) {
 		if (playerHandler->Player(a)->team == teamNum) {
 			playerHandler->Player(a)->StartSpectating();
+			playerHandler->Player(a)->SetControlledTeams();
 		}
 	}
 
-	CLuaUI::UpdateTeams();
-	CPlayer::UpdateControlledTeams();
 	eventHandler.TeamDied(teamNum);
 }
 
-void CTeam::StartposMessage(const float3& pos)
+void CTeam::AddPlayer(int playerNum)
 {
-	startPos = pos;
+	// note: does it matter if this team was already dead?
+	isDead = false;
+
+	if (leader == -1) {
+		leader = playerNum;
+	}
+
+	playerHandler->Player(playerNum)->JoinTeam(teamNum);
+	playerHandler->Player(playerNum)->SetControlledTeams();
 }
 
+void CTeam::KillAIs()
+{
+	const CSkirmishAIHandler::ids_t& localTeamAIs = skirmishAIHandler.GetSkirmishAIsInTeam(teamNum, gu->myPlayerNum);
+
+	for (CSkirmishAIHandler::ids_t::const_iterator ai = localTeamAIs.begin(); ai != localTeamAIs.end(); ++ai) {
+		skirmishAIHandler.SetLocalSkirmishAIDieing(*ai, 2 /* = team died */);
+	}
+}
+
+
+
 CTeam& CTeam::operator=(const TeamBase& base)
 {
 	TeamBase::operator=(base);
 	return *this;
 }
 
-void CTeam::ResetFrameVariables()
+
+
+void CTeam::ResetResourceState()
 {
-	prevMetalPull     = metalPull;
-	prevMetalIncome   = metalIncome;
-	prevMetalExpense  = metalExpense;
-	prevMetalUpkeep   = metalUpkeep;
-	prevEnergyPull    = energyPull;
-	prevEnergyIncome  = energyIncome;
-	prevEnergyExpense = energyExpense;
-	prevEnergyUpkeep  = energyUpkeep;
+	// reset all state variables that were
+	// potentially modified during the last
+	// <TEAM_SLOWUPDATE_RATE> frames
+	prevMetalPull     = metalPull;     metalPull     = 0.0f;
+	prevMetalIncome   = metalIncome;   metalIncome   = 0.0f;
+	prevMetalExpense  = metalExpense;  metalExpense  = 0.0f;
+	prevEnergyPull    = energyPull;    energyPull    = 0.0f;
+	prevEnergyIncome  = energyIncome;  energyIncome  = 0.0f;
+	prevEnergyExpense = energyExpense; energyExpense = 0.0f;
 
-	metalPull = 0;
-	metalIncome = 0;
-	metalExpense = 0;
-	metalUpkeep = 0;
-	energyPull = 0;
-	energyIncome = 0;
-	energyExpense = 0;
-	energyUpkeep = 0;
-
-	metalSent = 0;
-	energySent = 0;
-	metalReceived = 0;
-	energyReceived = 0;
+	// reset the sharing accumulators
+	metalSent = 0.0f; metalReceived = 0.0f;
+	energySent = 0.0f; energyReceived = 0.0f;
 }
 
 void CTeam::SlowUpdate()
 {
-	currentStats->metalProduced  += prevMetalIncome;
-	currentStats->energyProduced += prevEnergyIncome;
-	currentStats->metalUsed  += prevMetalUpkeep + prevMetalExpense;
-	currentStats->energyUsed += prevEnergyUpkeep + prevEnergyExpense;
-
 	float eShare = 0.0f, mShare = 0.0f;
+
+	// calculate the total amount of resources that all
+	// (allied) teams can collectively receive through
+	// sharing
 	for (int a = 0; a < teamHandler->ActiveTeams(); ++a) {
+		CTeam* team = teamHandler->Team(a);
+
 		if ((a != teamNum) && (teamHandler->AllyTeam(teamNum) == teamHandler->AllyTeam(a))) {
-			CTeam* team = teamHandler->Team(a);
 			if (team->isDead)
 				continue;
 
@@ -309,21 +289,25 @@ void CTeam::SlowUpdate()
 		}
 	}
 
-	metal += delayedMetalShare;
-	energy += delayedEnergyShare;
-	delayedMetalShare = 0;
-	delayedEnergyShare = 0;
 
+	currentStats->metalProduced  += prevMetalIncome;
+	currentStats->energyProduced += prevEnergyIncome;
+	currentStats->metalUsed  += prevMetalExpense;
+	currentStats->energyUsed += prevEnergyExpense;
+
+	metal  += delayedMetalShare;  delayedMetalShare  = 0.0f;
+	energy += delayedEnergyShare; delayedEnergyShare = 0.0f;
+
+
+	// calculate how much we can share in total (any and all excess resources)
 	const float eExcess = std::max(0.0f, energy - (energyStorage * energyShare));
 	const float mExcess = std::max(0.0f, metal  - (metalStorage  * metalShare));
 
 	float de = 0.0f, dm = 0.0f;
-	if (eShare > 0.0f) {
-		de = std::min(1.0f, eExcess/eShare);
-	}
-	if (mShare > 0.0f) {
-		dm = std::min(1.0f, mExcess/mShare);
-	}
+	if (eShare > 0.0f) { de = std::min(1.0f, eExcess / eShare); }
+	if (mShare > 0.0f) { dm = std::min(1.0f, mExcess / mShare); }
+
+	// now evenly distribute our excess resources among allied teams
 	for (int a = 0; a < teamHandler->ActiveTeams(); ++a) {
 		if ((a != teamNum) && (teamHandler->AllyTeam(teamNum) == teamHandler->AllyTeam(a))) {
 			CTeam* team = teamHandler->Team(a);
@@ -331,23 +315,19 @@ void CTeam::SlowUpdate()
 				continue;
 
 			const float edif = std::max(0.0f, (team->energyStorage * 0.99f) - team->energy) * de;
-			energy -= edif;
-			energySent += edif;
-			currentStats->energySent += edif;
-			team->energy += edif;
-			team->energyReceived += edif;
-			team->currentStats->energyReceived += edif;
-
 			const float mdif = std::max(0.0f, (team->metalStorage * 0.99f) - team->metal) * dm;
-			metal -= mdif;
-			metalSent += mdif;
-			currentStats->metalSent += mdif;
-			team->metal += mdif;
-			team->metalReceived += mdif;
-			team->currentStats->metalReceived += mdif;
+
+			energy     -= edif; team->energy         += edif;
+			energySent += edif; team->energyReceived += edif;
+			metal      -= mdif; team->metal          += mdif;
+			metalSent  += mdif; team->metalReceived  += mdif;
+
+			currentStats->energySent += edif; team->currentStats->energyReceived += edif;
+			currentStats->metalSent  += mdif; team->currentStats->metalReceived  += mdif;
 		}
 	}
 
+	// clamp resource levels to storage capacity
 	if (metal > metalStorage) {
 		currentStats->metalExcess += (metal - metalStorage);
 		metal = metalStorage;
@@ -369,25 +349,10 @@ void CTeam::SlowUpdate()
 		nextHistoryEntry = gs->frameNum + statsFrames;
 		currentStats->frame = nextHistoryEntry;
 	}
-
-	/* Kill the player on 'com dies = game ends' games.  This can't be done in
-	CTeam::CommanderDied anymore, because that function is called in
-	CUnit::ChangeTeam(), hence it'd cause a random amount of the shared units
-	to be killed if the commander is among them. Also, ".take" would kill all
-	units once it transfered the commander. */
-	if (gameSetup->gameMode == GameMode::ComEnd && numCommanders<=0 && !gaia){
-		for(std::list<CUnit*>::iterator ui=uh->activeUnits.begin();ui!=uh->activeUnits.end();++ui){
-			if ((*ui)->team==teamNum && !(*ui)->unitDef->isCommander) {
-				(*ui)->KillUnit(true, false, NULL);
-			}
-		}
-		// Set to 1 to prevent above loop from being done every update.
-		numCommanders = 1;
-	}
 }
 
 
-void CTeam::AddUnit(CUnit* unit,AddType type)
+void CTeam::AddUnit(CUnit* unit, AddType type)
 {
 	units.insert(unit);
 	switch (type) {
@@ -404,13 +369,10 @@ void CTeam::AddUnit(CUnit* unit,AddType type)
 			break;
 		}
 	}
-	if (unit->unitDef->isCommander) {
-		numCommanders++;
-	}
 }
 
 
-void CTeam::RemoveUnit(CUnit* unit,RemoveType type)
+void CTeam::RemoveUnit(CUnit* unit, RemoveType type)
 {
 	units.erase(unit);
 	switch (type) {
@@ -427,33 +389,8 @@ void CTeam::RemoveUnit(CUnit* unit,RemoveType type)
 			break;
 		}
 	}
-
-	if (units.empty() && !gaia) {
-		Died();
-	}
 }
 
-
-void CTeam::CommanderDied(CUnit* commander)
-{
-	assert(commander->unitDef->isCommander);
-	--numCommanders;
-}
-
-
-void CTeam::LeftLineage(CUnit* unit)
-{
-	if (gameSetup->gameMode == GameMode::Lineage && unit->id == this->lineageRoot) {
-		for (std::list<CUnit*>::iterator ui = uh->activeUnits.begin(); ui != uh->activeUnits.end(); ++ui) {
-			if ((*ui)->lineage == this->teamNum) {
-				(*ui)->KillUnit(true, false, NULL);
-			}
-		}
-	}
-}
-
-
-
 std::string CTeam::GetControllerName() const {
 	std::string s;
 
