@@ -1,18 +1,16 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include "TeamHandler.h"
 
 #include <cstring>
 
 #include "Game/GameSetup.h"
 #include "Lua/LuaGaia.h"
+#include "System/mmgr.h"
 #include "Sim/Misc/GlobalConstants.h"
-#include "mmgr.h"
-#include "Util.h"
-#include "LogOutput.h"
-#include "GlobalUnsynced.h"
-#include "GlobalSynced.h"
+#include "Sim/Misc/GlobalSynced.h"
+#include "System/Util.h"
+
 
 CR_BIND(CTeamHandler, );
 
@@ -25,7 +23,7 @@ CR_REG_METADATA(CTeamHandler, (
 ));
 
 
-CTeamHandler* teamHandler;
+CTeamHandler* teamHandler = NULL;
 
 
 CTeamHandler::CTeamHandler():
@@ -44,8 +42,12 @@ CTeamHandler::~CTeamHandler()
 
 void CTeamHandler::LoadFromSetup(const CGameSetup* setup)
 {
+	assert(!setup->teamStartingData.empty());
 	assert(setup->teamStartingData.size() <= MAX_TEAMS);
+	assert(setup->allyStartingData.size() <= MAX_TEAMS);
+
 	teams.resize(setup->teamStartingData.size());
+	allyTeams = setup->allyStartingData;
 
 	for (size_t i = 0; i < teams.size(); ++i) {
 		// TODO: this loop body could use some more refactoring
@@ -53,32 +55,36 @@ void CTeamHandler::LoadFromSetup(const CGameSetup* setup)
 		teams[i] = team;
 		*team = setup->teamStartingData[i];
 		team->teamNum = i;
-		SetAllyTeam(i, team->teamAllyteam);
+		team->maxUnits = std::min(setup->maxUnits, int(MAX_UNITS / teams.size()));
+
+		assert(team->teamAllyteam >=                0);
+		assert(team->teamAllyteam <  allyTeams.size());
 	}
 
-	allyTeams = setup->allyStartingData;
-	assert(setup->allyStartingData.size() <= MAX_TEAMS);
 	if (gs->useLuaGaia) {
 		// Gaia adjustments
 		gaiaTeamID = static_cast<int>(teams.size());
 		gaiaAllyTeamID = static_cast<int>(allyTeams.size());
 
 		// Setup the gaia team
-		CTeam& team = *(new CTeam());
-		team.color[0] = 255;
-		team.color[1] = 255;
-		team.color[2] = 255;
-		team.color[3] = 255;
-		team.gaia = true;
-		team.teamNum = gaiaTeamID;
-		team.StartposMessage(float3(0.0, 0.0, 0.0));
-		team.teamAllyteam = gaiaAllyTeamID;
-		teams.push_back(&team);
+		CTeam* gaia = new CTeam();
+		gaia->color[0] = 255;
+		gaia->color[1] = 255;
+		gaia->color[2] = 255;
+		gaia->color[3] = 255;
+		gaia->gaia = true;
+		gaia->teamNum = gaiaTeamID;
+		gaia->maxUnits = MAX_UNITS - (teams.size() * teams[0]->maxUnits);
+		gaia->StartposMessage(ZeroVector);
+		gaia->teamAllyteam = gaiaAllyTeamID;
+		teams.push_back(gaia);
 
-		for (std::vector< ::AllyTeam >::iterator it = allyTeams.begin(); it != allyTeams.end(); ++it)
-		{
+		assert((((teams.size() - 1) * teams[0]->maxUnits) + gaia->maxUnits) == MAX_UNITS);
+
+		for (std::vector< ::AllyTeam >::iterator it = allyTeams.begin(); it != allyTeams.end(); ++it) {
 			it->allies.push_back(false); // enemy to everyone
 		}
+
 		::AllyTeam allyteam;
 		allyteam.allies.resize(allyTeams.size() + 1, false); // everyones enemy
 		allyteam.allies[gaiaAllyTeamID] = true; // peace with itself
@@ -88,12 +94,12 @@ void CTeamHandler::LoadFromSetup(const CGameSetup* setup)
 
 void CTeamHandler::GameFrame(int frameNum)
 {
-	if (!(frameNum & (TEAM_SLOWUPDATE_RATE-1))) {
+	if ((frameNum % TEAM_SLOWUPDATE_RATE) == 0) {
 		for (int a = 0; a < ActiveTeams(); ++a) {
-			Team(a)->ResetFrameVariables();
+			teams[a]->ResetResourceState();
 		}
 		for (int a = 0; a < ActiveTeams(); ++a) {
-			Team(a)->SlowUpdate();
+			teams[a]->SlowUpdate();
 		}
 	}
 }
