@@ -1,18 +1,16 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "RepulseGfx.h"
 #include "Rendering/GlobalRendering.h"
-#include "Rendering/ProjectileDrawer.hpp"
+#include "Rendering/ProjectileDrawer.h"
 #include "Rendering/GL/myGL.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/TextureAtlas.h"
 #include "Sim/Units/Unit.h"
-#include "System/GlobalUnsynced.h"
 
-CR_BIND_DERIVED(CRepulseGfx, CProjectile, (NULL, NULL, 0, ZeroVector));
+CR_BIND_DERIVED(CRepulseGfx, CProjectile, (NULL, NULL, 0.0f, ZeroVector));
 
 CR_REG_METADATA(CRepulseGfx,(
 	CR_MEMBER(repulsed),
@@ -31,7 +29,7 @@ CRepulseGfx::CRepulseGfx(CUnit* owner, CProjectile* repulsed, float maxDist, con
 	color(color)
 {
 	if (repulsed) {
-		AddDeathDependence(repulsed);
+		AddDeathDependence(repulsed, DEPENDENCE_REPULSE);
 	}
 
 	checkCol = false;
@@ -59,7 +57,7 @@ CRepulseGfx::~CRepulseGfx()
 void CRepulseGfx::DependentDied(CObject* o)
 {
 	if (o == repulsed) {
-		repulsed = 0;
+		repulsed = NULL;
 		deleteMe = true;
 	}
 }
