@@ -1,20 +1,18 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include <algorithm>
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "SpherePartProjectile.h"
 #include "Rendering/GlobalRendering.h"
-#include "Rendering/ProjectileDrawer.hpp"
+#include "Rendering/ProjectileDrawer.h"
 #include "Rendering/GL/myGL.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/TextureAtlas.h"
-#include "System/GlobalUnsynced.h"
 
 using std::min;
 
-CR_BIND_DERIVED(CSpherePartProjectile, CProjectile, (float3(0, 0, 0), 0, 0, 0.0, 0.0, 0, NULL, float3(0, 0, 0)));
+CR_BIND_DERIVED(CSpherePartProjectile, CProjectile, (ZeroVector, 0, 0, 0.0f, 0.0f, 0, NULL, ZeroVector));
 
 CR_REG_METADATA(CSpherePartProjectile, (
 	CR_MEMBER(centerPos),
@@ -57,7 +55,7 @@ CSpherePartProjectile::CSpherePartProjectile(const float3& centerPos, int xpart,
 	pos = centerPos+vectors[12] * sphereSize;
 
 	drawRadius = 60;
-	alwaysVisible = true;
+
 	texx = projectileDrawer->sphereparttex->xstart + (projectileDrawer->sphereparttex->xend - projectileDrawer->sphereparttex->xstart) * 0.5f;
 	texy = projectileDrawer->sphereparttex->ystart + (projectileDrawer->sphereparttex->yend - projectileDrawer->sphereparttex->ystart) * 0.5f;
 }
