@@ -1,13 +1,12 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "TorpedoProjectile.h"
 #include "Game/Camera.h"
 #include "Game/GameHelper.h"
 #include "Map/Ground.h"
-#include "Rendering/ProjectileDrawer.hpp"
+#include "Rendering/ProjectileDrawer.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/TextureAtlas.h"
 #include "Sim/Projectiles/ProjectileHandler.h"
@@ -18,10 +17,10 @@
 #include "System/myMath.h"
 
 #ifdef TRACE_SYNC
-	#include "Sync/SyncTracer.h"
+	#include "System/Sync/SyncTracer.h"
 #endif
 
-CR_BIND_DERIVED(CTorpedoProjectile, CWeaponProjectile, (float3(0,0,0),float3(0,0,0),NULL,0,0,0,0,NULL,NULL));
+CR_BIND_DERIVED(CTorpedoProjectile, CWeaponProjectile, (ZeroVector, ZeroVector, NULL, 0, 0, 0, 0, NULL, NULL));
 
 CR_REG_METADATA(CTorpedoProjectile,(
 	CR_SETFLAG(CF_Synced),
@@ -38,14 +37,13 @@ CR_REG_METADATA(CTorpedoProjectile,(
 	));
 
 CTorpedoProjectile::CTorpedoProjectile(
-	const float3& pos, const float3& speed,
-	CUnit* owner,
-	float areaOfEffect, float maxSpeed,
-	float tracking, int ttl,
-	CUnit* target,
-	const WeaponDef* weaponDef):
-
-	CWeaponProjectile(pos, speed, owner, target, ZeroVector, weaponDef, 0, ttl),
+		const float3& pos, const float3& speed,
+		CUnit* owner,
+		float areaOfEffect, float maxSpeed,
+		float tracking, int ttl,
+		CUnit* target,
+		const WeaponDef* weaponDef)
+	: CWeaponProjectile(pos, speed, owner, target, ZeroVector, weaponDef, NULL, ttl),
 	tracking(tracking),
 	maxSpeed(maxSpeed),
 	areaOfEffect(areaOfEffect),
@@ -56,14 +54,10 @@ CTorpedoProjectile::CTorpedoProjectile(
 	curSpeed = speed.Length();
 	dir = speed / curSpeed;
 
-	if (target) {
-		AddDeathDependence(target);
-	}
-
 	SetRadius(0.0f);
 	drawRadius = maxSpeed * 8;
 
-	const float3 camDir = (pos - camera->pos).Normalize();
+//	const float3 camDir = (pos - camera->pos).Normalize();
 	texx = projectileDrawer->torpedotex->xstart - (projectileDrawer->torpedotex->xend - projectileDrawer->torpedotex->xstart) * 0.5f;
 	texy = projectileDrawer->torpedotex->ystart - (projectileDrawer->torpedotex->yend - projectileDrawer->torpedotex->ystart) * 0.5f;
 #ifdef TRACE_SYNC
@@ -71,26 +65,23 @@ CTorpedoProjectile::CTorpedoProjectile(
 	tracefile << pos.x << " " << pos.y << " " << pos.z << " " << speed.x << " " << speed.y << " " << speed.z << "\n";
 #endif
 
-	if (!cegTag.empty()) {
-		ceg.Load(explGenHandler, cegTag);
-	}
+	cegID = gCEG->Load(explGenHandler, cegTag);
 }
 
-CTorpedoProjectile::~CTorpedoProjectile(void)
+CTorpedoProjectile::~CTorpedoProjectile()
 {
 }
 
 void CTorpedoProjectile::DependentDied(CObject* o)
 {
-	if (o == target)
-		target = 0;
+	if (o == target) {
+		target = NULL;
+	}
 	CWeaponProjectile::DependentDied(o);
 }
 
 void CTorpedoProjectile::Collision()
 {
-	if(ground->GetHeight2(pos.x,pos.z)<pos.y+4)
-		return;
 	CWeaponProjectile::Collision();
 }
 
@@ -99,9 +90,9 @@ void CTorpedoProjectile::Collision(CUnit *unit)
 	CWeaponProjectile::Collision(unit);
 }
 
-void CTorpedoProjectile::Update(void)
+void CTorpedoProjectile::Update()
 {
-	if (!(weaponDef->submissile) && pos.y > -3.0f) {
+	if (!weaponDef->submissile && pos.y > -3.0f) {
 		// tracking etc only works when we are underwater
 		if (!luaMoveCtrl) {
 			speed.y += mygravity;
@@ -110,7 +101,7 @@ void CTorpedoProjectile::Update(void)
 			dir.Normalize();
 		}
 	} else {
-		if (!(weaponDef->submissile) && pos.y-speed.y > -3.0f) {
+		if (!weaponDef->submissile && pos.y-speed.y > -3.0f) {
 			// level out torpedo a bit when hitting water
 			if (!luaMoveCtrl) {
 				dir.y *= 0.5f;
@@ -133,7 +124,7 @@ void CTorpedoProjectile::Update(void)
 						targPos = helper->GetUnitErrorPos(target, owner()->allyteam);
 					}
 
-					if (!(weaponDef->submissile) && targPos.y > 0) {
+					if (!weaponDef->submissile && targPos.y > 0) {
 						targPos.y = 0;
 					}
 
@@ -145,24 +136,22 @@ void CTorpedoProjectile::Update(void)
 						dir = dif;
 					} else {
 						dif2 -= dir * (dif2.dot(dir));
-						dif2.Normalize();
+						dif2.SafeNormalize();
 						dir += dif2 * tracking;
-						dir.Normalize();
+						dir.SafeNormalize();
 					}
 				}
 
 				speed = dir * curSpeed;
 			}
 
-			if (!cegTag.empty()) {
-				ceg.Explosion(pos, ttl, areaOfEffect, 0x0, 0.0f, 0x0, speed);
-			}
+			gCEG->Explosion(cegID, pos, ttl, areaOfEffect, NULL, 0.0f, NULL, speed);
 		} else {
 			if (!luaMoveCtrl) {
 				speed *= 0.98f;
 				speed.y += mygravity;
 				dir = speed;
-				dir.Normalize();
+				dir.SafeNormalize();
 			}
 		}
 	}
@@ -189,67 +178,70 @@ void CTorpedoProjectile::Update(void)
 	UpdateGroundBounce();
 }
 
-void CTorpedoProjectile::Draw(void)
+void CTorpedoProjectile::Draw()
 {
-	if (model)	//dont draw if a 3d model has been defined for us
+	if (model) {
+		// do not draw if a 3D model has been defined for us
 		return;
+	}
 
-	inArray=true;
+	inArray = true;
 
 	unsigned char col[4];
-	col[0]=60;
-	col[1]=60;
-	col[2]=100;
-	col[3]=255;
+	col[0] = 60;
+	col[1] = 60;
+	col[2] = 100;
+	col[3] = 255;
 
-	float3 r=dir.cross(UpVector);
-	if(r.Length()<0.001f)
-		r=float3(1,0,0);
+	float3 r = dir.cross(UpVector);
+	if (r.Length() < 0.001f) {
+		r = float3(1.0f, 0.0f, 0.0f);
+	}
 	r.Normalize();
-	float3 u=dir.cross(r);
-	const float h=12;
-	const float w=2;
+	const float3 u = dir.cross(r);
+	const float h = 12;
+	const float w = 2;
 
-	va->EnlargeArrays(32,0,VA_SIZE_TC);
+	va->EnlargeArrays(32, 0, VA_SIZE_TC);
 
-	va->AddVertexQTC(drawPos+r*w, texx,texy,col);
-	va->AddVertexQTC(drawPos+u*w, texx,texy,col);
-	va->AddVertexQTC(drawPos+u*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos+r*w+dir*h, texx,texy,col);
+	va->AddVertexQTC(drawPos + (r * w),             texx, texy, col);
+	va->AddVertexQTC(drawPos + (u * w),             texx, texy, col);
+	va->AddVertexQTC(drawPos + (u * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos + (r * w) + (dir * h), texx, texy, col);
 
-	va->AddVertexQTC(drawPos+u*w, texx,texy,col);
-	va->AddVertexQTC(drawPos-r*w, texx,texy,col);
-	va->AddVertexQTC(drawPos-r*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos+u*w+dir*h, texx,texy,col);
+	va->AddVertexQTC(drawPos + (u * w),             texx, texy, col);
+	va->AddVertexQTC(drawPos - (r * w),             texx, texy, col);
+	va->AddVertexQTC(drawPos - (r * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos + (u * w) + (dir * h), texx, texy, col);
 
-	va->AddVertexQTC(drawPos-r*w, texx,texy,col);
-	va->AddVertexQTC(drawPos-u*w, texx,texy,col);
-	va->AddVertexQTC(drawPos-u*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos-r*w+dir*h, texx,texy,col);
+	va->AddVertexQTC(drawPos - (r * w),             texx, texy, col);
+	va->AddVertexQTC(drawPos - (u * w),             texx, texy, col);
+	va->AddVertexQTC(drawPos - (u * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos - (r * w) + (dir * h), texx, texy, col);
 
-	va->AddVertexQTC(drawPos-u*w, texx,texy,col);
-	va->AddVertexQTC(drawPos+r*w, texx,texy,col);
-	va->AddVertexQTC(drawPos+r*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos-u*w+dir*h, texx,texy,col);
+	va->AddVertexQTC(drawPos - (u * w),             texx, texy, col);
+	va->AddVertexQTC(drawPos + (r * w),             texx, texy, col);
+	va->AddVertexQTC(drawPos + (r * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos - (u * w) + (dir * h), texx, texy, col);
 
 
-	va->AddVertexQTC(drawPos+r*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos+u*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos+dir*h*1.2f, texx,texy,col);
-	va->AddVertexQTC(drawPos+dir*h*1.2f, texx,texy,col);
+	va->AddVertexQTC(drawPos + (r * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos + (u * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos + (dir * h * 1.2f),    texx, texy, col);
+	va->AddVertexQTC(drawPos + (dir * h * 1.2f),    texx, texy, col);
 
-	va->AddVertexQTC(drawPos+u*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos-r*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos+dir*h*1.2f, texx,texy,col);
-	va->AddVertexQTC(drawPos+dir*h*1.2f, texx,texy,col);
+	va->AddVertexQTC(drawPos + (u * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos - (r * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos + (dir * h * 1.2f),    texx, texy, col);
+	va->AddVertexQTC(drawPos + (dir * h * 1.2f),    texx, texy, col);
 
-	va->AddVertexQTC(drawPos-r*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos-u*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos+dir*h*1.2f, texx,texy,col);
-	va->AddVertexQTC(drawPos+dir*h*1.2f, texx,texy,col);
+	va->AddVertexQTC(drawPos - (r * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos - (u * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos + (dir * h * 1.2f),    texx, texy, col);
+	va->AddVertexQTC(drawPos + (dir * h * 1.2f),    texx, texy, col);
 
-	va->AddVertexQTC(drawPos-u*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos+r*w+dir*h, texx,texy,col);
-	va->AddVertexQTC(drawPos+dir*h*1.2f, texx,texy,col);
-	va->AddVertexQTC(drawPos+dir*h*1.2f, texx,texy,col);
+	va->AddVertexQTC(drawPos - (u * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos + (r * w) + (dir * h), texx, texy, col);
+	va->AddVertexQTC(drawPos + (dir * h * 1.2f),    texx, texy, col);
+	va->AddVertexQTC(drawPos + (dir * h * 1.2f),    texx, texy, col);
 }
