@@ -1,32 +1,27 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
 
 #include "FactoryCAI.h"
 #include "ExternalAI/EngineOutHandler.h"
-#include "LineDrawer.h"
 #include "Sim/Units/Groups/Group.h"
 #include "Sim/Misc/GlobalSynced.h"
 #include "Game/GameHelper.h"
+#include "Game/GlobalUnsynced.h"
 #include "Game/SelectedUnits.h"
 #include "Game/WaitCommandsAI.h"
-#include "Game/UI/CommandColors.h"
-#include "Game/UI/CursorIcons.h"
-#include "Rendering/UnitDrawer.h"
-#include "Rendering/GL/myGL.h"
-#include "Rendering/GL/glExtra.h"
 #include "Lua/LuaRules.h"
 #include "Sim/Misc/TeamHandler.h"
+#include "Sim/Units/BuildInfo.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/UnitLoader.h"
 #include "Sim/Units/UnitDefHandler.h"
 #include "Sim/Units/UnitTypes/Factory.h"
-#include "LogOutput.h"
-#include "creg/STL_Map.h"
-#include "GlobalUnsynced.h"
-#include "Util.h"
-#include "Exceptions.h"
+#include "System/Log/ILog.h"
+#include "System/creg/STL_Map.h"
+#include "System/Util.h"
+#include "System/EventHandler.h"
+#include "System/Exceptions.h"
 
 CR_BIND_DERIVED(CFactoryCAI ,CCommandAI , );
 
@@ -160,32 +155,37 @@ void CFactoryCAI::PostLoad()
 
 void CFactoryCAI::GiveCommandReal(const Command& c, bool fromSynced)
 {
+	const int& cmd_id = c.GetID();
+	
 	// move is always allowed for factories (passed to units it produces)
-	if ((c.id == CMD_SET_WANTED_MAX_SPEED) ||
-	    ((c.id != CMD_MOVE) && !AllowedCommand(c, fromSynced))) {
+	if ((cmd_id == CMD_SET_WANTED_MAX_SPEED) ||
+	    ((cmd_id != CMD_MOVE) && !AllowedCommand(c, fromSynced))) {
 		return;
 	}
 
-	map<int, BuildOption>::iterator boi = buildOptions.find(c.id);
+	map<int, BuildOption>::iterator boi = buildOptions.find(cmd_id);
 
 	// not a build order so queue it to built units
 	if (boi == buildOptions.end()) {
-		if ((nonQueingCommands.find(c.id) != nonQueingCommands.end()) ||
-		    (c.id == CMD_INSERT) || (c.id == CMD_REMOVE) ||
-		    (!(c.options & SHIFT_KEY) && ((c.id == CMD_WAIT) || (c.id == CMD_SELFD)))) {
+		if ((nonQueingCommands.find(cmd_id) != nonQueingCommands.end()) ||
+		    (cmd_id == CMD_INSERT) || (cmd_id == CMD_REMOVE) ||
+		    (!(c.options & SHIFT_KEY) && ((cmd_id == CMD_WAIT) || (cmd_id == CMD_SELFD)))) {
 			CCommandAI::GiveAllowedCommand(c);
 			return;
 		}
 
 		if (!(c.options & SHIFT_KEY)) {
  			waitCommandsAI.ClearUnitQueue(owner, newUnitCommands);
+			CCommandAI::ClearCommandDependencies();
 			newUnitCommands.clear();
 		}
 
-		if (c.id != CMD_STOP) {
-			if ((c.id == CMD_WAIT) || (c.id == CMD_SELFD)) {
-				if (!newUnitCommands.empty() && (newUnitCommands.back().id == c.id)) {
-					if (c.id == CMD_WAIT) {
+		CCommandAI::AddCommandDependency(c);
+
+		if (cmd_id != CMD_STOP) {
+			if ((cmd_id == CMD_WAIT) || (cmd_id == CMD_SELFD)) {
+				if (!newUnitCommands.empty() && (newUnitCommands.back().GetID() == cmd_id)) {
+					if (cmd_id == CMD_WAIT) {
 						waitCommandsAI.RemoveWaitCommand(owner, c);
 					}
 					newUnitCommands.pop_back();
@@ -209,9 +209,9 @@ void CFactoryCAI::GiveCommandReal(const Command& c, bool fromSynced)
 
 		// the first new-unit build order can not be WAIT or SELFD
 		while (!newUnitCommands.empty()) {
-			const int id = newUnitCommands.front().id;
+			const int& id = newUnitCommands.front().GetID();
 			if ((id == CMD_WAIT) || (id == CMD_SELFD)) {
-				if (c.id == CMD_WAIT) {
+				if (cmd_id == CMD_WAIT) {
 					waitCommandsAI.RemoveWaitCommand(owner, c);
 				}
 				newUnitCommands.pop_front();
@@ -238,21 +238,21 @@ void CFactoryCAI::GiveCommandReal(const Command& c, bool fromSynced)
 		int numToErase = numItems;
 		if (c.options & ALT_KEY) {
 			for (unsigned int cmdNum = 0; cmdNum < commandQue.size() && numToErase; ++cmdNum) {
-				if (commandQue[cmdNum].id == c.id) {
-					commandQue[cmdNum].id = CMD_STOP;
+				if (commandQue[cmdNum].GetID() == cmd_id) {
+					commandQue[cmdNum] = Command(CMD_STOP);
 					numToErase--;
 				}
 			}
 		}
 		else {
 			for (int cmdNum = commandQue.size() - 1; cmdNum != -1 && numToErase; --cmdNum) {
-				if (commandQue[cmdNum].id == c.id) {
-					commandQue[cmdNum].id = CMD_STOP;
+				if (commandQue[cmdNum].GetID() == cmd_id) {
+					commandQue[cmdNum] = Command(CMD_STOP);
 					numToErase--;
 				}
 			}
 		}
-		UpdateIconName(c.id,bo);
+		UpdateIconName(cmd_id,bo);
 		SlowUpdate();
 	}
 	else {
@@ -281,7 +281,7 @@ void CFactoryCAI::GiveCommandReal(const Command& c, bool fromSynced)
 			}
 		}
 		bo.numQued += numItems;
-		UpdateIconName(c.id, bo);
+		UpdateIconName(cmd_id, bo);
 
 		SlowUpdate();
 	}
@@ -291,10 +291,10 @@ void CFactoryCAI::GiveCommandReal(const Command& c, bool fromSynced)
 void CFactoryCAI::InsertBuildCommand(CCommandQueue::iterator& it,
                                      const Command& newCmd)
 {
-	map<int, BuildOption>::iterator boi = buildOptions.find(newCmd.id);
+	map<int, BuildOption>::iterator boi = buildOptions.find(newCmd.GetID());
 	if (boi != buildOptions.end()) {
 		boi->second.numQued++;
-		UpdateIconName(newCmd.id, boi->second);
+		UpdateIconName(newCmd.GetID(), boi->second);
 	}
 	if (!commandQue.empty() && (it == commandQue.begin())) {
 		// ExecuteStop(), without the pop_front()
@@ -306,22 +306,25 @@ void CFactoryCAI::InsertBuildCommand(CCommandQueue::iterator& it,
 }
 
 
-void CFactoryCAI::RemoveBuildCommand(CCommandQueue::iterator& it)
+bool CFactoryCAI::RemoveBuildCommand(CCommandQueue::iterator& it)
 {
 	Command& cmd = *it;
-	map<int, BuildOption>::iterator boi = buildOptions.find(cmd.id);
+	map<int, BuildOption>::iterator boi = buildOptions.find(cmd.GetID());
 	if (boi != buildOptions.end()) {
 		boi->second.numQued--;
-		UpdateIconName(cmd.id, boi->second);
+		UpdateIconName(cmd.GetID(), boi->second);
 	}
 	if (!commandQue.empty() && (it == commandQue.begin())) {
 		ExecuteStop(cmd);
-		return;
+		return true;
 	}
-	if (cmd.id < 0) {
-		cmd.id = CMD_STOP;
-		cmd.tag = 0;
+
+	if (cmd.GetID() < 0) {
+		// build command, convert into a stop command
+		cmd = Command(CMD_STOP);
 	}
+
+	return false;
 }
 
 
@@ -331,13 +334,14 @@ void CFactoryCAI::CancelRestrictedUnit(const Command& c, BuildOption& buildOptio
 		buildOption.numQued--;
 		if (owner->team == gu->myTeam) {
 			if(lastRestrictedWarning+100<gs->frameNum) {
-				logOutput.Print("%s: Build failed, unit type limit reached",owner->unitDef->humanName.c_str());
-				logOutput.SetLastMsgPos(owner->pos);
+				LOG_L(L_WARNING, "%s: Build failed, unit type limit reached",
+						owner->unitDef->humanName.c_str());
+				eventHandler.LastMessagePosition(owner->pos);
 				lastRestrictedWarning = gs->frameNum;
 			}
 		}
 	}
-	UpdateIconName(c.id, buildOption);
+	UpdateIconName(c.GetID(), buildOption);
 	FinishCommand();
 }
 
@@ -358,7 +362,7 @@ void CFactoryCAI::SlowUpdate()
 		oldSize = commandQue.size();
 		map<int, BuildOption>::iterator boi;
 
-		if ((boi = buildOptions.find(c.id)) != buildOptions.end()) {
+		if ((boi = buildOptions.find(c.GetID())) != buildOptions.end()) {
 			const UnitDef* def = unitDefHandler->GetUnitDefByName(boi->second.name);
 
 			if (building) {
@@ -367,7 +371,7 @@ void CFactoryCAI::SlowUpdate()
 					if (!repeatOrders || c.options & DONT_REPEAT) {
 						boi->second.numQued--;
 					}
-					UpdateIconName(c.id,boi->second);
+					UpdateIconName(c.GetID(),boi->second);
 					FinishCommand();
 				}
 
@@ -384,20 +388,20 @@ void CFactoryCAI::SlowUpdate()
 					if(!repeatOrders || c.options & DONT_REPEAT){
 						boi->second.numQued--;
 					}
-					UpdateIconName(c.id,boi->second);
+					UpdateIconName(c.GetID(),boi->second);
 					FinishCommand();
 				}
 				else if (uh->unitsByDefs[owner->team][def->id].size() >= def->maxThisUnit) { //unit restricted?
 					CancelRestrictedUnit(c, boi->second);
 				}
-				else if(uh->MaxUnitsPerTeam() > teamHandler->Team(owner->team)->units.size()) {  //max unitlimit reached?
+				else if (!teamHandler->Team(owner->team)->AtUnitLimit()) {
 					fac->StartBuild(def);
-					building=true;
+					building = true;
 				}
 			}
 		}
 		else {
-			switch(c.id){
+			switch(c.GetID()){
 				case CMD_STOP: {
 					ExecuteStop(c);
 					break;
@@ -416,9 +420,11 @@ void CFactoryCAI::SlowUpdate()
 
 void CFactoryCAI::ExecuteStop(Command &c)
 {
-	CFactory* fac=(CFactory*)owner;
-	building=false;
+	building = false;
+
+	CFactory* fac = (CFactory*) owner;
 	fac->StopBuild();
+
 	commandQue.pop_front();
 	return;
 }
@@ -453,100 +459,3 @@ void CFactoryCAI::UpdateIconName(int id,BuildOption& bo)
 	}
 	selectedUnits.PossibleCommandChange(owner);
 }
-
-
-void CFactoryCAI::DrawCommands(void)
-{
-	lineDrawer.StartPath(owner->drawMidPos, cmdColors.start);
-
-	if (owner->selfDCountdown != 0) {
-		lineDrawer.DrawIconAtLastPos(CMD_SELFD);
-	}
-
-	if (!commandQue.empty() && (commandQue.front().id == CMD_WAIT)) {
-		DrawWaitIcon(commandQue.front());
-	}
-
-	CCommandQueue::iterator ci;
-	for(ci=newUnitCommands.begin();ci!=newUnitCommands.end();++ci){
-		switch(ci->id){
-			case CMD_MOVE: {
-				const float3 endPos(ci->params[0], ci->params[1] + 3, ci->params[2]);
-				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.move);
-				break;
-			}
-			case CMD_FIGHT: {
-				const float3 endPos(ci->params[0], ci->params[1] + 3, ci->params[2]);
-				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.fight);
-				break;
-			}
-			case CMD_PATROL: {
-				const float3 endPos(ci->params[0], ci->params[1] + 3, ci->params[2]);
-				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.patrol);
-				break;
-			}
-			case CMD_ATTACK: {
-				if (ci->params.size() == 1) {
-					const CUnit* unit = uh->GetUnit(ci->params[0]);
-
-					if ((unit != NULL) && isTrackable(unit)) {
-						const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
-						lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
-					}
-				} else {
-					const float3 endPos(ci->params[0],ci->params[1]+3,ci->params[2]);
-					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
-				}
-				break;
-			}
-			case CMD_GUARD: {
-				const CUnit* unit = uh->GetUnit(ci->params[0]);
-
-				if ((unit != NULL) && isTrackable(unit)) {
-					const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
-					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.guard);
-				}
-				break;
-			}
-			case CMD_WAIT: {
-				DrawWaitIcon(*ci);
-				break;
-			}
-			case CMD_SELFD: {
-				lineDrawer.DrawIconAtLastPos(ci->id);
-				break;
-			}
-			default:
-				DrawDefaultCommand(*ci);
-				break;
-		}
-
-		if ((ci->id < 0) && (ci->params.size() >= 3)) {
-			BuildInfo bi;
-			bi.def = unitDefHandler->GetUnitDefByID(-(ci->id));
-			if (ci->params.size() == 4) {
-				bi.buildFacing = int(ci->params[3]);
-			}
-			bi.pos = float3(ci->params[0], ci->params[1], ci->params[2]);
-			bi.pos = helper->Pos2BuildPos(bi);
-
-			cursorIcons.AddBuildIcon(ci->id, bi.pos, owner->team, bi.buildFacing);
-			lineDrawer.DrawLine(bi.pos, cmdColors.build);
-
-			// draw metal extraction range
-			if (bi.def->extractRange > 0) {
-				lineDrawer.Break(bi.pos, cmdColors.build);
-				glColor4fv(cmdColors.rangeExtract);
-
-				if (bi.def->extractSquare) {
-					glSurfaceSquare(bi.pos, bi.def->extractRange, bi.def->extractRange);
-				} else {
-					glSurfaceCircle(bi.pos, bi.def->extractRange, 40);
-				}
-
-				lineDrawer.Restart();
-			}
-		}
-	}
-	lineDrawer.FinishPath();
-}
