@@ -1,18 +1,24 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef __FACTORY_AI_H__
-#define __FACTORY_AI_H__
+#ifndef _FACTORY_AI_H_
+#define _FACTORY_AI_H_
 
 #include "CommandAI.h"
+#include "CommandQueue.h"
+
 #include <string>
 #include <map>
 
-class CFactoryCAI :
-	public CCommandAI
+class CUnit;
+class CFeature;
+struct Command;
+
+class CFactoryCAI : public CCommandAI
 {
 public:
 	CR_DECLARE(CFactoryCAI);
 	CR_DECLARE_SUB(BuildOption);
+
 	struct BuildOption {
 		CR_DECLARE_STRUCT(BuildOption)
 		std::string name;
@@ -22,7 +28,7 @@ public:
 
 	CFactoryCAI(CUnit* owner);
 	CFactoryCAI();
-	~CFactoryCAI(void);
+	~CFactoryCAI();
 	void PostLoad();
 
 	int GetDefaultCmd(const CUnit* pointed, const CFeature* feature);
@@ -31,12 +37,11 @@ public:
 	void GiveCommandReal(const Command& c, bool fromSynced = true);
 
 	void InsertBuildCommand(CCommandQueue::iterator& it, const Command& c);
-	void RemoveBuildCommand(CCommandQueue::iterator& it);
+	bool RemoveBuildCommand(CCommandQueue::iterator& it);
 
 	void ExecuteStop(Command &c);
 
-	void DrawCommands(void);
-	void UpdateIconName(int id,BuildOption& bo);
+	void UpdateIconName(int id, BuildOption& bo);
 
 	CCommandQueue newUnitCommands;
 
@@ -51,4 +56,4 @@ private:
 	void CancelRestrictedUnit(const Command& c, BuildOption& buildOption);
 };
 
-#endif // __FACTORY_AI_H__
+#endif // _FACTORY_AI_H_
