@@ -299,6 +299,11 @@ CUnit::~CUnit()
 	// Remove us from our group, if we were in one
 	SetGroup(NULL);
 
+	if (script != &CNullUnitScript::value) {
+		delete script;
+		script = NULL;
+	}
+	// ScriptCallback may reference weapons, so delete the script first
 	for (std::vector<CWeapon*>::const_iterator wi = weapons.begin(); wi != weapons.end(); ++wi) {
 		delete *wi;
 	}
@@ -308,11 +313,6 @@ CUnit::~CUnit()
 	los = NULL;
 	radarhandler->RemoveUnit(this);
 
-	if (script != &CNullUnitScript::value) {
-		delete script;
-		script = NULL;
-	}
-
 	modelParser->DeleteLocalModel(this);
 }
 
@@ -721,11 +721,8 @@ void CUnit::Update()
 
 	if (stunned) {
 		// leave the pad if reserved
-		if (moveType->reservedPad) {
-			airBaseHandler->LeaveLandingPad(moveType->reservedPad);
-			moveType->reservedPad = 0;
-			moveType->padStatus = 0;
-		}
+		moveType->UnreservePad(moveType->GetReservedPad());
+
 		// paralyzed weapons shouldn't reload
 		for (std::vector<CWeapon*>::iterator wi = weapons.begin(); wi != weapons.end(); ++wi) {
 			++(*wi)->reloadStatus;
@@ -912,15 +909,22 @@ void CUnit::SlowUpdate()
 	}
 
 	if (beingBuilt) {
-		if (modInfo.constructionDecay && lastNanoAdd < gs->frameNum - modInfo.constructionDecayTime) {
-			const float buildDecay = 1.0f / (buildTime * modInfo.constructionDecaySpeed);
-			health -= maxHealth * buildDecay;
+		if (modInfo.constructionDecay && (lastNanoAdd < (gs->frameNum - modInfo.constructionDecayTime))) {
+			float buildDecay = buildTime * modInfo.constructionDecaySpeed;
+
+			buildDecay = 1.0f / std::max(0.001f, buildDecay);
+			buildDecay = std::min(buildProgress, buildDecay);
+
+			health         = std::max(0.0f, health - maxHealth * buildDecay);
 			buildProgress -= buildDecay;
+
 			AddMetal(metalCost * buildDecay, false);
-			if (health < 0.0f) {
+
+			if (health <= 0.0f || buildProgress <= 0.0f) {
 				KillUnit(false, true, NULL);
 			}
 		}
+
 		ScriptDecloak(false);
 		return;
 	}
@@ -1314,6 +1318,7 @@ void CUnit::AddExperience(float exp)
 	}
 	if (expHealthScale > 0.0f) {
 		const float oldMaxHealth = maxHealth;
+
 		maxHealth = std::max(0.1f, unitDef->health * (1.0f + (limExperience * expHealthScale)));
 		health = health * (maxHealth / oldMaxHealth);
 	}
@@ -1447,16 +1452,16 @@ bool CUnit::ChangeTeam(int newteam, ChangeType type)
 void CUnit::ChangeTeamReset()
 {
 	// stop friendly units shooting at us
-	const std::map<DependenceType, std::list<CObject*> >& listeners = GetAllListeners();
+	const CObject::TDependenceMap& listeners = GetAllListeners();
 	std::vector<CUnit *> alliedunits;
-	for (std::map<DependenceType, std::list<CObject*> >::const_iterator li = listeners.begin(); li != listeners.end(); ++li) {
-		for (std::list<CObject *>::const_iterator di = li->second.begin(); di != li->second.end(); ++di) {
-			CUnit* u = dynamic_cast<CUnit *>(*di);
+	for (CObject::TDependenceMap::const_iterator li = listeners.begin(); li != listeners.end(); ++li) {
+		for (CObject::TSyncSafeSet::const_iterator di = li->second.begin(); di != li->second.end(); ++di) {
+			CUnit* u = dynamic_cast<CUnit*>(*di);
 			if (u != NULL && teamHandler->AlliedTeams(team, u->team))
 				alliedunits.push_back(u);
 		}
 	}
-	for (std::vector<CUnit *>::const_iterator ui = alliedunits.begin(); ui != alliedunits.end(); ++ui) {
+	for (std::vector<CUnit*>::const_iterator ui = alliedunits.begin(); ui != alliedunits.end(); ++ui) {
 		(*ui)->StopAttackingAllyTeam(allyteam);
 	}
 	// and stop shooting at friendly ally teams
