@@ -9,6 +9,7 @@
 #endif
 
 #include <map>
+#include <list>
 #include <vector>
 #include <string>
 
@@ -41,7 +42,7 @@ class CTransportUnit;
 #define LOS_INLOS      (1 << 0)  // the unit is currently in the los of the allyteam
 #define LOS_INRADAR    (1 << 1)  // the unit is currently in radar from the allyteam
 #define LOS_PREVLOS    (1 << 2)  // the unit has previously been in los from the allyteam
-#define LOS_CONTRADAR  (1 << 3)  // the unit has continously been in radar since it was last inlos by the allyteam
+#define LOS_CONTRADAR  (1 << 3)  // the unit has continuously been in radar since it was last inlos by the allyteam
 
 // LOS mask bits  (masked bits are not automatically updated)
 #define LOS_INLOS_MASK     (1 << 8)   // do not update LOS_INLOS
@@ -454,7 +455,7 @@ public:
 
 	UnitTrackStruct* myTrack;
 
-	std::list<CMissileProjectile*> incomingMissiles;
+	std::list<CMissileProjectile*> incomingMissiles; //FIXME make std::set?
 	int lastFlareDrop;
 
 	float currentFuel;
