@@ -1,32 +1,29 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 
 #include "Factory.h"
 #include "Game/GameHelper.h"
 #include "Game/WaitCommandsAI.h"
 #include "Map/Ground.h"
 #include "Map/ReadMap.h"
-#include "Rendering/GlobalRendering.h"
 #include "Sim/Misc/GroundBlockingObjectMap.h"
 #include "Sim/Misc/QuadField.h"
-#include "Sim/Misc/TeamHandler.h"
 #include "Sim/Projectiles/ProjectileHandler.h"
-#include "Sim/Projectiles/Unsynced/GfxProjectile.h"
-#include "Sim/Units/COB/UnitScript.h"
+#include "Sim/Units/Scripts/UnitScript.h"
 #include "Sim/Units/CommandAI/CommandAI.h"
 #include "Sim/Units/CommandAI/FactoryCAI.h"
 #include "Sim/Units/CommandAI/MobileCAI.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/UnitLoader.h"
 #include "Sim/Units/UnitDefHandler.h"
-#include "System/GlobalUnsynced.h"
 #include "System/EventHandler.h"
 #include "System/Matrix44f.h"
 #include "System/myMath.h"
-#include "System/Sound/IEffectChannel.h"
+#include "System/Sound/SoundChannels.h"
 #include "System/Sync/SyncTracer.h"
-#include "mmgr.h"
+#include "System/mmgr.h"
+
+#define PLAY_SOUNDS 1
 
 CR_BIND_DERIVED(CFactory, CBuilding, );
 
@@ -68,7 +65,7 @@ CFactory::~CFactory() {
 void CFactory::PostLoad()
 {
 	nextBuild = unitDefHandler->GetUnitDefByName(nextBuildName);
-	if(opening){
+	if (opening) {
 		script->Activate();
 	}
 	if (curBuild) {
@@ -76,10 +73,10 @@ void CFactory::PostLoad()
 	}
 }
 
-void CFactory::UnitInit (const UnitDef* def, int team, const float3& position)
+void CFactory::PreInit(const UnitDef* def, int team, int facing, const float3& position, bool build)
 {
 	buildSpeed = def->buildSpeed / TEAM_SLOWUPDATE_RATE;
-	CBuilding::UnitInit(def, team, position);
+	CBuilding::PreInit(def, team, facing, position, build);
 }
 
 int CFactory::GetBuildPiece()
@@ -116,26 +113,27 @@ void CFactory::Update()
 
 		if (solidObj == NULL || (dynamic_cast<const CUnit*>(solidObj) == this)) {
 			quedBuild = false;
-			CUnit* b = unitLoader.LoadUnit(nextBuild, buildPos + float3(0.01f, 0.01f, 0.01f), team,
+			CUnit* b = unitLoader->LoadUnit(nextBuild, buildPos + float3(0.01f, 0.01f, 0.01f), team,
 											true, buildFacing, this);
-			b->lineage = this->lineage;
 
 			if (!unitDef->canBeAssisted) {
 				b->soloBuilder = this;
-				b->AddDeathDependence(this);
+				b->AddDeathDependence(this, DEPENDENCE_BUILDER);
 			}
 
-			AddDeathDependence(b);
+			AddDeathDependence(b, DEPENDENCE_BUILD);
 			curBuild = b;
 
 			script->StartBuilding();
 
-			int soundIdx = unitDef->sounds.build.getRandomIdx();
+			#if (PLAY_SOUNDS == 1)
+			const int soundIdx = unitDef->sounds.build.getRandomIdx();
 			if (soundIdx >= 0) {
 				Channels::UnitReply.PlaySample(
 					unitDef->sounds.build.getID(soundIdx), pos,
 					unitDef->sounds.build.getVolume(0));
 			}
+			#endif
 		} else {
 			helper->BuggerOff(buildPos - float3(0.01f, 0, 0.02f), radius + 8, true, true, team, NULL);
 		}
@@ -150,25 +148,22 @@ void CFactory::Update()
 
 			// buildPiece is the rotating platform
 			const int buildPiece = GetBuildPiece();
+			const float3& buildPos = CalcBuildPos(buildPiece);
 			const CMatrix44f& mat = script->GetPieceMatrix(buildPiece);
 			const int h = GetHeadingFromVector(mat[2], mat[10]); //! x.z, z.z
 
 			// rotate unit nanoframe with platform
-			curBuild->heading = (h + GetHeadingFromFacing(buildFacing)) & 65535;
-
-			const float3 buildPos = CalcBuildPos(buildPiece);
+			curBuild->heading = (h + GetHeadingFromFacing(buildFacing)) & (SPRING_CIRCLE_DIVS - 1);
 			curBuild->pos = buildPos;
 
-			if (curBuild->floatOnWater) {
-				float waterline = ground->GetHeight(buildPos.x, buildPos.z) - curBuild->unitDef->waterline;
-				if (waterline > curBuild->pos.y)
-					curBuild->pos.y = waterline;
-			}
-			curBuild->midPos = curBuild->pos + (UpVector * curBuild->relMidPos.y);
+			if (curBuild->floatOnWater && (buildPos.y <= 0.0f))
+				curBuild->pos.y = -curBuild->unitDef->waterline;
+
+			curBuild->UpdateMidPos();
 
 			const CCommandQueue& queue = commandAI->commandQue;
 
-			if(!queue.empty() && (queue.front().id == CMD_WAIT)) {
+			if (!queue.empty() && (queue.front().GetID() == CMD_WAIT)) {
 				curBuild->AddBuildPower(0, this);
 			} else {
 				if (curBuild->AddBuildPower(buildSpeed, this)) {
@@ -259,11 +254,6 @@ void CFactory::DependentDied(CObject* o)
 	CUnit::DependentDied(o);
 }
 
-void CFactory::FinishedBuilding(void)
-{
-	CBuilding::FinishedBuilding();
-}
-
 
 
 void CFactory::SendToEmptySpot(CUnit* unit)
@@ -273,7 +263,7 @@ void CFactory::SendToEmptySpot(CUnit* unit)
 
 	for (int a = 0; a < 20; ++a) {
 		float3 testPos = pos + frontdir * r * cos(a * PI / 10) + rightdir * r * sin(a * PI / 10);
-		testPos.y = ground->GetHeight(testPos.x, testPos.z);
+		testPos.y = ground->GetHeightAboveWater(testPos.x, testPos.z);
 
 		if (qf->GetSolidsExact(testPos, unit->radius * 1.5f).empty()) {
 			foundPos = testPos;
@@ -281,9 +271,7 @@ void CFactory::SendToEmptySpot(CUnit* unit)
 		}
 	}
 
-	Command c;
-	c.id = CMD_MOVE;
-	c.options = 0;
+	Command c(CMD_MOVE);
 	c.params.push_back(foundPos.x);
 	c.params.push_back(foundPos.y);
 	c.params.push_back(foundPos.z);
@@ -299,8 +287,7 @@ void CFactory::AssignBuildeeOrders(CUnit* unit) {
 		return;
 	}
 
-	Command c;
-	c.id = CMD_MOVE;
+	Command c(CMD_MOVE);
 
 	if (!unit->unitDef->canfly) {
 
@@ -357,7 +344,7 @@ bool CFactory::ChangeTeam(int newTeam, ChangeType type)
 }
 
 
-void CFactory::CreateNanoParticle(void)
+void CFactory::CreateNanoParticle(bool highPriority)
 {
 	const int piece = script->QueryNanoPiece();
 
@@ -366,21 +353,12 @@ void CFactory::CreateNanoParticle(void)
 		return;
 #endif
 
-	if (ph->currentNanoParticles < ph->maxNanoParticles && unitDef->showNanoSpray) {
-		const float3 relWeaponFirePos = script->GetPiecePos(piece);
-		const float3 weaponPos = pos + (frontdir * relWeaponFirePos.z)
-			+ (updir    * relWeaponFirePos.y)
-			+ (rightdir * relWeaponFirePos.x);
-		float3 dif = (curBuild->midPos - weaponPos);
-		const float l = fastmath::apxsqrt2(dif.SqLength());
-		dif /= l;
-		dif += gu->usRandVector() * 0.15f;
-		float3 color = unitDef->nanoColor;
+	const float3 relWeaponFirePos = script->GetPiecePos(piece);
+	const float3 weaponPos = pos
+		+ (frontdir * relWeaponFirePos.z)
+		+ (updir    * relWeaponFirePos.y)
+		+ (rightdir * relWeaponFirePos.x);
 
-		if (globalRendering->teamNanospray) {
-			unsigned char* tcol = teamHandler->Team(team)->color;
-			color = float3(tcol[0] * (1.0f / 255.0f), tcol[1] * (1.0f / 255.0f), tcol[2] * (1.0f / 255.0f));
-		}
-		new CGfxProjectile(weaponPos, dif, (int) l, color);
-	}
+	// unsynced
+	ph->AddNanoParticle(weaponPos, curBuild->midPos, unitDef, team, highPriority);
 }
