@@ -4,9 +4,12 @@
 #define __FACTORY_H__
 
 #include "Building.h"
-#include "Sim/Units/UnitDef.h"
+#include "System/float3.h"
+
 #include <string>
 
+struct UnitDef;
+
 class CFactory : public CBuilding
 {
 public:
@@ -14,30 +17,32 @@ public:
 
 	CFactory();
 	virtual ~CFactory();
+
 	void PostLoad();
 	void StopBuild();
 	void StartBuild(const UnitDef* ud);
 	void Update();
 	void DependentDied(CObject* o);
-	void FinishedBuilding(void);
-	void CreateNanoParticle(void);
+	void CreateNanoParticle(bool highPriority = false);
 
-	float3 CalcBuildPos(int buildPiece=-1); // supply the build piece to speed up
+	/// supply the build piece to speed up
+	float3 CalcBuildPos(int buildPiece = -1);
 	int GetBuildPiece();
 
-	void UnitInit(const UnitDef* def, int team, const float3& position);
+	void PreInit(const UnitDef* def, int team, int facing, const float3& position, bool build);
 
 	float buildSpeed;
 
-	bool quedBuild;						//if we have a unit that we want to start to nanolath when script is ready
+	/// if we have a unit that we want to start to nanolath when script is ready
+	bool quedBuild;
 	const UnitDef* nextBuild;
 	std::string nextBuildName;
-	CUnit* curBuild;					//unit that we are nanolathing
+	CUnit* curBuild;            ///< unit that we are nanolathing
 	bool opening;
 
-	int lastBuild;						//last frame we wanted to build something
+	int lastBuild;              ///< last frame we wanted to build something
 
-	void SlowUpdate(void);
+	void SlowUpdate();
 	bool ChangeTeam(int newTeam, ChangeType type);
 
 private:
