@@ -7,15 +7,14 @@
 #include <vector>
 #include <map>
 
-#include "EventClient.h"
-#include "EventBatchHandler.h"
+#include "System/EventClient.h"
+#include "System/EventBatchHandler.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Features/Feature.h"
 #include "Sim/Projectiles/Projectile.h"
 
 class CWeapon;
 struct Command;
-class CLogSubsystem;
 
 class CEventHandler
 {
@@ -37,17 +36,22 @@ class CEventHandler
 		bool IsController(const std::string& ciName) const;
 
 	public:
-		// Synced events
-		void Load(CArchiveBase* archive);
+		/**
+		 * @name Synced_events
+		 * @{
+		 */
+		void Load(IArchive* archive);
 
 		void GamePreload();
 		void GameStart();
-		void GameOver();
+		void GameOver(const std::vector<unsigned char>& winningAllyTeams);
+		void GamePaused(int playerID, bool paused);
 		void GameFrame(int gameFrame);
 
 		void TeamDied(int teamID);
 		void TeamChanged(int teamID);
 		void PlayerChanged(int playerID);
+		void PlayerAdded(int playerID);
 		void PlayerRemoved(int playerID, int reason);
 
 		void UnitCreated(const CUnit* unit, const CUnit* builder);
@@ -63,9 +67,9 @@ class CEventHandler
 		void RenderUnitCloakChanged(const CUnit* unit, int cloaked);
 		void RenderUnitLOSChanged(const CUnit* unit, int allyTeam, int newStatus);
 
-		void DeleteSyncedUnits();
-		void UpdateDrawUnits();
 		void UpdateUnits();
+		void UpdateDrawUnits();
+		void DeleteSyncedUnits();
 
 		void UnitIdle(const CUnit* unit);
 		void UnitCommand(const CUnit* unit, const Command& command);
@@ -92,6 +96,9 @@ class CEventHandler
 		void UnitCloaked(const CUnit* unit);
 		void UnitDecloaked(const CUnit* unit);
 
+		void UnitUnitCollision(const CUnit* collider, const CUnit* collidee);
+		void UnitFeatureCollision(const CUnit* collider, const CFeature* collidee);
+		void UnitMoved(const CUnit* unit);
 		void UnitMoveFailed(const CUnit* unit);
 
 		void FeatureCreated(const CFeature* feature);
@@ -102,9 +109,9 @@ class CEventHandler
 		void RenderFeatureDestroyed(const CFeature* feature);
 		void RenderFeatureMoved(const CFeature* feature);
 
-		void DeleteSyncedFeatures();
-		void UpdateDrawFeatures();
 		void UpdateFeatures();
+		void UpdateDrawFeatures();
+		void DeleteSyncedFeatures();
 
 		void ProjectileCreated(const CProjectile* proj, int allyTeam);
 		void ProjectileDestroyed(const CProjectile* proj, int allyTeam);
@@ -115,21 +122,27 @@ class CEventHandler
 		void UnsyncedProjectileCreated(const CProjectile* proj);
 		void UnsyncedProjectileDestroyed(const CProjectile* proj);
 
-		void DeleteSyncedProjectiles();
-		void UpdateDrawProjectiles();
 		void UpdateProjectiles();
+		void UpdateDrawProjectiles();
+		void DeleteSyncedProjectiles();
 
 		void UpdateObjects();
+		void DeleteSyncedObjects();
 
-		bool Explosion(int weaponID, const float3& pos, const CUnit* owner);
+		bool Explosion(int weaponDefID, const float3& pos, const CUnit* owner);
 
 		void StockpileChanged(const CUnit* unit,
 		                      const CWeapon* weapon, int oldCount);
+		/// @}
 
 	public:
-		// Unsynced events
+		/**
+		 * @name Unsynced_events
+		 * @{
+		 */
 		void Save(zipFile archive);
 
+		void UnsyncedHeightMapUpdate(const SRectangle& rect);
 		void Update();
 
 		bool KeyPress(unsigned short key, bool isRepeat);
@@ -147,7 +160,9 @@ class CEventHandler
 
 		bool CommandNotify(const Command& cmd);
 
-		bool AddConsoleLine(const std::string& msg, const CLogSubsystem& zone);
+		bool AddConsoleLine(const std::string& msg, const std::string& section, int level);
+
+		void LastMessagePosition(const float3& pos);
 
 		bool GroupChanged(int groupID);
 
@@ -163,6 +178,8 @@ class CEventHandler
 		                const float3* pos1,
 		                const std::string* label);
 
+		void SunChanged(const float3& sunDir);
+
 		void ViewResize();
 
 		void DrawGenesis();
@@ -175,7 +192,13 @@ class CEventHandler
 		void DrawScreen();
 		void DrawInMiniMap();
 
+		/// @brief this UNSYNCED event is generated every gameProgressFrameInterval ( defined in gameserver.cpp ), skips network queuing and caching and it's useful
+		/// to calculate the current fast-forwarding % compared to the real game
+		void GameProgress(int gameFrame);
+		/// @}
+
 		// FIXME: void ShockFront(float power, const float3& pos, float areaOfEffect);
+		inline void LoadedModelRequested();
 
 	private:
 		typedef vector<CEventClient*> EventClientList;
@@ -226,10 +249,12 @@ class CEventHandler
 		EventClientList listGamePreload;
 		EventClientList listGameStart;
 		EventClientList listGameOver;
+		EventClientList listGamePaused;
 		EventClientList listGameFrame;
 		EventClientList listTeamDied;
 		EventClientList listTeamChanged;
 		EventClientList listPlayerChanged;
+		EventClientList listPlayerAdded;
 		EventClientList listPlayerRemoved;
 
 		EventClientList listUnitCreated;
@@ -267,6 +292,9 @@ class CEventHandler
 		EventClientList listRenderUnitCloakChanged;
 		EventClientList listRenderUnitLOSChanged;
 
+		EventClientList listUnitUnitCollision;
+		EventClientList listUnitFeatureCollision;
+		EventClientList listUnitMoved;
 		EventClientList listUnitMoveFailed;
 
 		EventClientList listFeatureCreated;
@@ -290,6 +318,7 @@ class CEventHandler
 		// unsynced
 		EventClientList listSave;
 
+		EventClientList listUnsyncedHeightMapUpdate;
 		EventClientList listUpdate;
 
 		EventClientList listKeyPress;
@@ -306,11 +335,14 @@ class CEventHandler
 		EventClientList listConfigCommand;
 		EventClientList listCommandNotify;
 		EventClientList listAddConsoleLine;
+		EventClientList listLastMessagePosition;
 		EventClientList listGroupChanged;
 		EventClientList listGameSetup;
 		EventClientList listWorldTooltip;
 		EventClientList listMapDrawCmd;
 
+		EventClientList listSunChanged;
+
 		EventClientList listViewResize;
 
 		EventClientList listDrawGenesis;
@@ -322,6 +354,8 @@ class CEventHandler
 		EventClientList listDrawScreenEffects;
 		EventClientList listDrawScreen;
 		EventClientList listDrawInMiniMap;
+
+		EventClientList listGameProgress;
 };
 
 
@@ -350,6 +384,7 @@ inline void CEventHandler::UnitCreated(const CUnit* unit, const CUnit* builder)
 }
 
 
+
 #define UNIT_CALLIN_NO_PARAM(name)                                 \
 	inline void CEventHandler:: name (const CUnit* unit)           \
 	{                                                              \
@@ -365,6 +400,7 @@ inline void CEventHandler::UnitCreated(const CUnit* unit, const CUnit* builder)
 
 UNIT_CALLIN_NO_PARAM(UnitFinished)
 UNIT_CALLIN_NO_PARAM(UnitIdle)
+UNIT_CALLIN_NO_PARAM(UnitMoved)
 UNIT_CALLIN_NO_PARAM(UnitMoveFailed)
 UNIT_CALLIN_NO_PARAM(UnitEnteredWater)
 UNIT_CALLIN_NO_PARAM(UnitEnteredAir)
@@ -372,6 +408,7 @@ UNIT_CALLIN_NO_PARAM(UnitLeftWater)
 UNIT_CALLIN_NO_PARAM(UnitLeftAir)
 
 
+
 #define UNIT_CALLIN_INT_PARAM(name)                                       \
 	inline void CEventHandler:: Unit ## name (const CUnit* unit, int p)   \
 	{                                                                     \
@@ -389,6 +426,7 @@ UNIT_CALLIN_INT_PARAM(Taken)
 UNIT_CALLIN_INT_PARAM(Given)
 
 
+
 #define UNIT_CALLIN_LOS_PARAM(name)                                        \
 	inline void CEventHandler:: Unit ## name (const CUnit* unit, int at)   \
 	{                                                                      \
@@ -408,6 +446,7 @@ UNIT_CALLIN_LOS_PARAM(LeftRadar)
 UNIT_CALLIN_LOS_PARAM(LeftLos)
 
 
+
 inline void CEventHandler::UnitFromFactory(const CUnit* unit,
                                                const CUnit* factory,
                                                bool userOrders)
@@ -492,7 +531,7 @@ inline void CEventHandler::UnitCloaked(const CUnit* unit)
 		CEventClient* ec = listUnitCloaked[i];
 		if (ec->CanReadAllyTeam(unitAllyTeam)) {
 			ec->UnitCloaked(unit);
-		} 
+		}
 	}
 }
 
@@ -506,15 +545,37 @@ inline void CEventHandler::UnitDecloaked(const CUnit* unit)
 		CEventClient* ec = listUnitDecloaked[i];
 		if (ec->CanReadAllyTeam(unitAllyTeam)) {
 			ec->UnitDecloaked(unit);
-		} 
+		}
 	}
 }
 
 
 
-inline void CEventHandler::UpdateUnits(void) { eventBatchHandler->UpdateUnits(); }
-inline void CEventHandler::UpdateDrawUnits() { eventBatchHandler->UpdateDrawUnits(); }
-inline void CEventHandler::DeleteSyncedUnits() { eventBatchHandler->DeleteSyncedUnits(); }
+inline void CEventHandler::UnitUnitCollision(const CUnit* collider, const CUnit* collidee)
+{
+	const int colliderAllyTeam = collider->allyteam;
+	const int clientCount = listUnitUnitCollision.size();
+
+	for (int i = 0; i < clientCount; i++) {
+		CEventClient* ec = listUnitUnitCollision[i];
+		if (ec->CanReadAllyTeam(colliderAllyTeam)) {
+			ec->UnitUnitCollision(collider, collidee);
+		}
+	}
+}
+
+inline void CEventHandler::UnitFeatureCollision(const CUnit* collider, const CFeature* collidee)
+{
+	const int colliderAllyTeam = collider->allyteam;
+	const int clientCount = listUnitFeatureCollision.size();
+
+	for (int i = 0; i < clientCount; i++) {
+		CEventClient* ec = listUnitFeatureCollision[i];
+		if (ec->CanReadAllyTeam(colliderAllyTeam)) {
+			ec->UnitFeatureCollision(collider, collidee);
+		}
+	}
+}
 
 
 
@@ -699,12 +760,6 @@ inline void CEventHandler::RenderFeatureMoved(const CFeature* feature)
 
 
 
-inline void CEventHandler::UpdateFeatures(void) { eventBatchHandler->UpdateFeatures(); }
-inline void CEventHandler::UpdateDrawFeatures() { eventBatchHandler->UpdateDrawFeatures(); }
-inline void CEventHandler::DeleteSyncedFeatures() { eventBatchHandler->DeleteSyncedFeatures(); }
-
-
-
 inline void CEventHandler::ProjectileCreated(const CProjectile* proj, int allyTeam)
 {
 	if (proj->synced) {
@@ -727,7 +782,11 @@ inline void CEventHandler::ProjectileCreated(const CProjectile* proj, int allyTe
 inline void CEventHandler::ProjectileDestroyed(const CProjectile* proj, int allyTeam)
 {
 	if (proj->synced) {
+#if DETACH_SYNCED
+		(eventBatchHandler->GetSyncedProjectileCreatedDestroyedBatch()).erase_delete(proj);
+#else
 		(eventBatchHandler->GetSyncedProjectileCreatedDestroyedBatch()).erase_remove_synced(proj);
+#endif
 	} else {
 		(eventBatchHandler->GetUnsyncedProjectileCreatedDestroyedBatch()).erase_delete(proj);
 	}
@@ -771,23 +830,26 @@ inline void CEventHandler::RenderProjectileDestroyed(const CProjectile* proj)
 }
 
 
-
-inline void CEventHandler::UpdateProjectiles() { eventBatchHandler->UpdateProjectiles(); }
-inline void CEventHandler::UpdateDrawProjectiles() { eventBatchHandler->UpdateDrawProjectiles(); }
-inline void CEventHandler::DeleteSyncedProjectiles() { eventBatchHandler->DeleteSyncedProjectiles(); }
-
-inline void CEventHandler::UpdateObjects() {
-	eventBatchHandler->UpdateObjects();
+inline void CEventHandler::UnsyncedHeightMapUpdate(const SRectangle& rect)
+{
+	const int count = listUnsyncedHeightMapUpdate.size();
+	for (int i = 0; i < count; i++) {
+		CEventClient* ec = listUnsyncedHeightMapUpdate[i];
+		ec->UnsyncedHeightMapUpdate(rect);
+	}
 }
 
-inline bool CEventHandler::Explosion(int weaponID, const float3& pos, const CUnit* owner)
+
+
+
+inline bool CEventHandler::Explosion(int weaponDefID, const float3& pos, const CUnit* owner)
 {
 	const int count = listExplosion.size();
 	bool noGfx = false;
 	for (int i = 0; i < count; i++) {
 		CEventClient* ec = listExplosion[i];
 		if (ec->GetFullRead()) {
-			noGfx = noGfx || ec->Explosion(weaponID, pos, owner);
+			noGfx = noGfx || ec->Explosion(weaponDefID, pos, owner);
 		}
 	}
 	return noGfx;
@@ -825,5 +887,9 @@ inline bool CEventHandler::DefaultCommand(const CUnit* unit,
 	return false;
 }
 
+inline void CEventHandler::LoadedModelRequested() {
+	eventBatchHandler->LoadedModelRequested();
+}
 
 #endif /* EVENT_HANDLER_H */
+
