@@ -1,14 +1,16 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef _UDPLISTENER 
-#define _UDPLISTENER
+#ifndef _UDP_LISTENER_H
+#define _UDP_LISTENER_H
 
 #include <boost/noncopyable.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/weak_ptr.hpp>
 #include <boost/asio/ip/udp.hpp>
 #include <list>
+#include <map>
 #include <queue>
+#include <string>
 
 namespace netcode
 {
@@ -17,8 +19,10 @@ typedef boost::shared_ptr<boost::asio::ip::udp::socket> SocketPtr;
 
 /**
  * @brief Class for handling Connections on an UDPSocket
- * Use this class if you want to use a UDPSocket to connect to more than one other clients.
- * You can Listen for new connections, initiate new ones and send/recieve data to/from them.
+ * Use this class if you want to use a UDPSocket to connect to more than
+ * one client.
+ * You can Listen for new connections, initiate new ones and send/recieve data
+ * to/from them.
  */
 class UDPListener : boost::noncopyable
 {
@@ -29,10 +33,10 @@ public:
 	 * @param  ip local IP to bind to, or "" for any
 	 */
 	UDPListener(int port, const std::string& ip = "");
-	
+
 	/**
-	@brief close the socket and DELETE all connections
-	*/
+	 * @brief close the socket and DELETE all connections
+	 */
 	~UDPListener() {}
 
 	/**
@@ -45,48 +49,55 @@ public:
 	 *         the default value "" results in the v6 any address "::",
 	 *         or the v4 equivalent "0.0.0.0", if v6 is no supported
 	 */
-	static bool TryBindSocket(int port, SocketPtr* socket, const std::string& ip = "");
-	
+	static bool TryBindSocket(int port, SocketPtr* socket,
+			const std::string& ip = "");
+
 	/**
-	@brief Run this from time to time
-	This does: recieve data from the socket and hand it to the associated UDPConnection, or open a new UDPConnection. It also Updates all of its connections
-	*/
+	 * @brief Run this from time to time
+	 * Recieve data from the socket and hand it to the associated UDPConnection,
+	 * or open a new UDPConnection. It also Updates all of its connections.
+	 */
 	void Update();
-	
+
 	/**
-	@brief Initiate a connection
-	Make a new connection to address:port. It will be pushed back in conn
-	*/
-	boost::shared_ptr<UDPConnection> SpawnConnection(const std::string& address, const unsigned port);
-	
+	 * @brief Initiate a connection
+	 * Make a new connection to ip:port. It will be pushed back in conn.
+	 */
+	boost::shared_ptr<UDPConnection> SpawnConnection(const std::string& ip,
+			const unsigned port);
+
 	/**
-	@brief Set if we are going to accept new connections or drop all data from unconnected addresses
-	*/
+	 * Set if we are going to accept new connections
+	 * or drop all data from unconnected addresses.
+	 */
 	bool Listen(const bool state);
 	bool Listen() const;
-	
+
 	bool HasIncomingConnections() const;
 	boost::weak_ptr<UDPConnection> PreviewConnection();
 	boost::shared_ptr<UDPConnection> AcceptConnection();
 	void RejectConnection();
-	
+
+	void UpdateConnections(); // Updates connections when the endpoint has been reconnected
+
 private:
 	/**
-	@brief Do we accept packets from unknown sources?
-	If true, we will create a new connection, if false, it get dropped
-	*/
+	 * @brief Do we accept packets from unknown sources?
+	 * If true, we will create a new connection, if false, they get dropped.
+	 */
 	bool acceptNewConnections;
-	
+
 	/// Our socket
 	/// typedef boost::shared_ptr<boost::asio::ip::udp::socket> SocketPtr;
 	SocketPtr mySocket;
 
 	/// all connections
-	std::list< boost::weak_ptr< UDPConnection> > conn;
-	
+	typedef std::map< boost::asio::ip::udp::endpoint, boost::weak_ptr<UDPConnection> > ConnMap;
+	ConnMap conn;
+
 	std::queue< boost::shared_ptr<UDPConnection> > waiting;
 };
 
 }
 
-#endif
+#endif // _UDP_LISTENER_H
