@@ -9,7 +9,12 @@
 #else
 	#include <pthread.h>
 #endif
+#include "System/Platform/Win/win32.h"
+#include <boost/cstdint.hpp>
 
+#ifdef __APPLE__
+#include <libkern/OSAtomic.h>
+#endif
 
 namespace Threading {
 #ifdef WIN32
@@ -53,6 +58,26 @@ namespace Threading {
 
 	void SetThreadError(const Error& err);
 	Error* GetThreadError();
+
+	class AtomicCounterInt64 {
+	public:
+		AtomicCounterInt64(boost::int64_t start) { num = start; }
+		boost::int64_t operator++() {
+#ifdef _MSC_VER
+			return InterlockedIncrement64(&num);
+#elif defined(__APPLE__)
+			return OSAtomicIncrement64(&num);
+#else // assuming GCC (__sync_fetch_and_add is a builtin)
+			return __sync_fetch_and_add(&num, boost::int64_t(1));
+#endif
+		}
+	private:
+#ifdef _MSC_VER
+		volatile __declspec(align(8)) boost::int64_t num;
+#else
+		volatile __attribute__ ((aligned (8))) boost::int64_t num;
+#endif
+	};
 };
 
 #endif // _THREADING_H_
