@@ -1,44 +1,33 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"
 #include "lib/gml/gml.h"
 #include <windows.h>
 #include <process.h>
 #include <imagehlp.h>
 #include <signal.h>
-#include <SDL.h> // for SDL_Quit
-#include <boost/thread/thread.hpp>
 
 #include "System/Platform/CrashHandler.h"
-
-#include "ConfigHandler.h"
-#include "LogOutput.h"
-#include "myTime.h"
-#include "NetProtocol.h"
+#include "System/Platform/errorhandler.h"
+#include "System/Log/ILog.h"
+#include "System/Log/LogSinkHandler.h"
+#include "System/LogOutput.h"
+#include "System/NetProtocol.h"
 #include "seh.h"
-#include "Util.h"
+#include "System/Util.h"
+#include "System/SafeCStrings.h"
 #include "Game/GameVersion.h"
 
+
 #define BUFFER_SIZE 2048
-
-#if defined(USE_GML) && GML_ENABLE_SIM
-extern volatile int gmlMultiThreadSim;
-#endif
-
-HANDLE simthread = INVALID_HANDLE_VALUE; // used in gmlsrv.h as well
-HANDLE drawthread = INVALID_HANDLE_VALUE;
-
+#define MAX_STACK_DEPTH 4096
 
 namespace CrashHandler {
 
-boost::thread* hangdetectorthread = NULL;
-volatile int keepRunning = 1;
-// watchdog timers
-volatile spring_time simwdt = spring_notime, drawwdt = spring_notime;
-spring_time hangTimeout = spring_msecs(0);
-volatile bool gameLoading = false;
+CRITICAL_SECTION stackLock;
+int stackLockInit() { InitializeCriticalSection(&stackLock); return 0; }
+int dummyStackLock = stackLockInit();
 
-void SigAbrtHandler(int signal)
+static void SigAbrtHandler(int signal)
 {
 	// cause an exception if on windows
 	// TODO FIXME do a proper stacktrace dump here
@@ -46,11 +35,12 @@ void SigAbrtHandler(int signal)
 }
 
 // Set this to the desired printf style output function.
-// Currently we write through the logOutput class to infolog.txt
-#define PRINT logOutput.Print
+// Currently we write through the ILog.h frontend to infolog.txt
+#define PRINT(fmt, ...) \
+		LOG_L(L_ERROR, fmt, ##__VA_ARGS__)
 
 /** Convert exception code to human readable string. */
-static const char *ExceptionName(DWORD exceptionCode)
+static const char* ExceptionName(DWORD exceptionCode)
 {
 	switch (exceptionCode) {
 		case EXCEPTION_ACCESS_VIOLATION:         return "Access violation";
@@ -79,16 +69,47 @@ static const char *ExceptionName(DWORD exceptionCode)
 	return "Unknown exception";
 }
 
-static void initImageHlpDll() {
 
+static bool InitImageHlpDll()
+{
 	char userSearchPath[8];
-	STRCPY(userSearchPath, ".");
+	STRCPY_T(userSearchPath, 8, ".");
 	// Initialize IMAGEHLP.DLL
-	SymInitialize(GetCurrentProcess(), userSearchPath, TRUE);
+	// Note: For some strange reason it doesn't work ~4 times after it was loaded&unloaded the first time.
+	int i = 0;
+	do {
+		if (SymInitialize(GetCurrentProcess(), userSearchPath, TRUE))
+			return true;
+		SymCleanup(GetCurrentProcess());
+		i++;
+	} while (i<20);
+	return false;
+}
+
+
+/** Callback for SymEnumerateModules */
+#if _MSC_VER >= 1500
+	static BOOL CALLBACK EnumModules(PCSTR moduleName, ULONG baseOfDll, PVOID userContext)
+	{
+		PRINT("0x%08lx\t%s", baseOfDll, moduleName);
+		return TRUE;
+	}
+#else // _MSC_VER >= 1500
+	static BOOL CALLBACK EnumModules(LPSTR moduleName, DWORD baseOfDll, PVOID userContext)
+	{
+		PRINT("0x%08lx\t%s", baseOfDll, moduleName);
+		return TRUE;
+	}
+#endif // _MSC_VER >= 1500
+
+static DWORD __stdcall AllocTest(void *param) {
+	GlobalFree(GlobalAlloc(GMEM_FIXED, 16384));
+	return 0;
 }
 
 /** Print out a stacktrace. */
-static void Stacktrace(LPEXCEPTION_POINTERS e, HANDLE hThread = INVALID_HANDLE_VALUE) {
+static void Stacktrace(const char *threadName, LPEXCEPTION_POINTERS e, HANDLE hThread = INVALID_HANDLE_VALUE)
+{
 	PIMAGEHLP_SYMBOL pSym;
 	STACKFRAME sf;
 	HANDLE process, thread;
@@ -97,43 +118,103 @@ static void Stacktrace(LPEXCEPTION_POINTERS e, HANDLE hThread = INVALID_HANDLE_V
 	int count = 0;
 	char modname[MAX_PATH];
 
-	pSym = (PIMAGEHLP_SYMBOL)GlobalAlloc(GMEM_FIXED, 16384);
+	process = GetCurrentProcess();
 
-	BOOL suspended = FALSE;
+	if(threadName)
+		PRINT("Stacktrace (%s):", threadName);
+	else
+		PRINT("Stacktrace:");
+
+	bool suspended = false;
 	CONTEXT c;
 	if (e) {
 		c = *e->ContextRecord;
 		thread = GetCurrentThread();
-	} else {
-		SuspendThread(hThread);
-		suspended = TRUE;
+	} else if (hThread != INVALID_HANDLE_VALUE) {
+		for (int allocIter = 0; ; ++allocIter) {
+			HANDLE allocThread = CreateThread(NULL, 0, &AllocTest, NULL, CREATE_SUSPENDED, NULL);
+			SuspendThread(hThread);
+			ResumeThread(allocThread);
+			if (WaitForSingleObject(allocThread, 10) == WAIT_OBJECT_0) {
+				CloseHandle(allocThread);
+				break;
+			}
+			ResumeThread(hThread);
+			if (WaitForSingleObject(allocThread, 10) != WAIT_OBJECT_0)
+				TerminateThread(allocThread, 0);
+			CloseHandle(allocThread);
+			if (allocIter < 10)
+				continue;
+			PRINT("Error: Stacktrace failed, allocator deadlock");
+			return;
+		}
+
+		suspended = true;
 		memset(&c, 0, sizeof(CONTEXT));
 		c.ContextFlags = CONTEXT_FULL;
-		// FIXME: This does not work if you want to dump the current thread's stack
+
 		if (!GetThreadContext(hThread, &c)) {
 			ResumeThread(hThread);
+			PRINT("Error: Stacktrace failed, failed to get context");
 			return;
 		}
 		thread = hThread;
 	}
+	else {
+#ifdef _M_IX86
+		ZeroMemory( &c, sizeof( CONTEXT ) );
+		c.ContextFlags = CONTEXT_CONTROL;
+#ifdef _MSC_VER
+		__asm
+		{
+			call func;
+			func: pop eax;
+			mov [c.Eip], eax;
+			mov [c.Ebp], ebp;
+			mov [c.Esp], esp;
+		}
+#else
+		DWORD eip, esp, ebp;
+		__asm__ __volatile__ ("call func; func: pop %%eax; mov %%eax, %0;" : "=m" (eip) : : "%eax" );
+		__asm__ __volatile__ ("mov %%ebp, %0;" : "=m" (ebp) : : );
+		__asm__ __volatile__ ("mov %%esp, %0;" : "=m" (esp) : : );
+		c.Eip=eip;
+		c.Ebp=ebp;
+		c.Esp=esp;
+#endif
+#else
+		RtlCaptureContext( &c );
+#endif
+		thread = GetCurrentThread();
+	}
 
+	DWORD MachineType = 0;
 	ZeroMemory(&sf, sizeof(sf));
+#ifdef _M_IX86
+	MachineType = IMAGE_FILE_MACHINE_I386;
 	sf.AddrPC.Offset = c.Eip;
 	sf.AddrStack.Offset = c.Esp;
 	sf.AddrFrame.Offset = c.Ebp;
+#elif _M_X64
+	MachineType = IMAGE_FILE_MACHINE_AMD64;
+	sf.AddrPC.Offset = c.Rip;
+	sf.AddrStack.Offset = c.Rsp;
+	sf.AddrFrame.Offset = c.Rsp;
+#else
+	#error "CrashHandler: Unsupported platform"
+#endif
 	sf.AddrPC.Mode = AddrModeFlat;
 	sf.AddrStack.Mode = AddrModeFlat;
 	sf.AddrFrame.Mode = AddrModeFlat;
 
-	process = GetCurrentProcess();
-
 	// use globalalloc to reduce risk for allocator related deadlock
+	pSym = (PIMAGEHLP_SYMBOL)GlobalAlloc(GMEM_FIXED, 16384);
 	char* printstrings = (char*)GlobalAlloc(GMEM_FIXED, 0);
 
 	bool containsOglDll = false;
 	while (true) {
 		more = StackWalk(
-			IMAGE_FILE_MACHINE_I386, // TODO: fix this for 64 bit windows?
+			MachineType,
 			process,
 			thread,
 			&sf,
@@ -143,7 +224,7 @@ static void Stacktrace(LPEXCEPTION_POINTERS e, HANDLE hThread = INVALID_HANDLE_V
 			SymGetModuleBase,
 			NULL
 		);
-		if (!more || sf.AddrFrame.Offset == 0) {
+		if (!more || sf.AddrFrame.Offset == 0 || count > MAX_STACK_DEPTH) {
 			break;
 		}
 
@@ -182,6 +263,8 @@ static void Stacktrace(LPEXCEPTION_POINTERS e, HANDLE hThread = INVALID_HANDLE_V
 		containsOglDll = containsOglDll || strstr(modname, "atiogl");
 		// OpenGL lib names (Nvidia): "nvoglnt.dll" "nvoglv32.dll" "nvoglv64.dll" (last one is a guess)
 		containsOglDll = containsOglDll || strstr(modname, "nvogl");
+		// OpenGL lib names (Intel): "ig4dev32.dll" "ig4dev64.dll" "ig4icd32.dll"
+		containsOglDll = containsOglDll || strstr(modname, "ig4");
 
 		++count;
 	}
@@ -202,64 +285,80 @@ static void Stacktrace(LPEXCEPTION_POINTERS e, HANDLE hThread = INVALID_HANDLE_V
 	}
 
 	GlobalFree(printstrings);
-
 	GlobalFree(pSym);
 }
 
-/** Callback for SymEnumerateModules */
-#if _MSC_VER >= 1500
-static BOOL CALLBACK EnumModules(PCSTR moduleName, ULONG baseOfDll, PVOID userContext)
-{
-	PRINT("0x%08lx\t%s", baseOfDll, moduleName);
-	return TRUE;
-}
-#else // _MSC_VER >= 1500
-static BOOL CALLBACK EnumModules(LPSTR moduleName, DWORD baseOfDll, PVOID userContext)
-{
-	PRINT("0x%08lx\t%s", baseOfDll, moduleName);
-	return TRUE;
-}
-#endif // _MSC_VER >= 1500
 
-/** Called by windows if an exception happens. */
-static LONG CALLBACK ExceptionHandler(LPEXCEPTION_POINTERS e)
+void Stacktrace(Threading::NativeThreadHandle thread, const std::string& threadName)
 {
-	// Prologue.
-	logOutput.SetSubscribersEnabled(false);
-	PRINT("Spring %s has crashed.", SpringVersion::GetFull().c_str());
-#ifdef USE_GML
-	PRINT("MT with %d threads.", gmlThreadCount);
-#endif
-	initImageHlpDll();
+	Stacktrace(threadName.c_str(), NULL, thread);
+}
 
-	// Record exception info.
-	PRINT("Exception: %s (0x%08lx)", ExceptionName(e->ExceptionRecord->ExceptionCode), e->ExceptionRecord->ExceptionCode);
-	PRINT("Exception Address: 0x%08lx", (unsigned long int) (PVOID) e->ExceptionRecord->ExceptionAddress);
+void PrepareStacktrace() {
+	EnterCriticalSection( &stackLock );
+
+	InitImageHlpDll();
 
 	// Record list of loaded DLLs.
 	PRINT("DLL information:");
 	SymEnumerateModules(GetCurrentProcess(), EnumModules, NULL);
+}
 
-	// Record stacktrace.
-	PRINT("Stacktrace:");
-	Stacktrace(e);
-
+void CleanupStacktrace() {
 	// Unintialize IMAGEHLP.DLL
 	SymCleanup(GetCurrentProcess());
 
-	// Cleanup.
-	SDL_Quit();
-	logOutput.End();  // Stop writing to log.
-	// FIXME: update closing of demo to new netcode
+	LeaveCriticalSection( &stackLock );
+
+}
+
+void OutputStacktrace() {
+	PRINT("Error handler invoked for Spring %s.", SpringVersion::GetFull().c_str());
+#ifdef USE_GML
+	PRINT("MT with %d threads.", gmlThreadCount);
+#endif
+
+	PrepareStacktrace();
+
+	PRINT("Stacktrace:");
+	Stacktrace(NULL,NULL);
+
+	CleanupStacktrace();
+}
+
+/** Called by windows if an exception happens. */
+LONG CALLBACK ExceptionHandler(LPEXCEPTION_POINTERS e)
+{
+	// Prologue.
+	logSinkHandler.SetSinking(false);
+	PRINT("Spring %s has crashed.", SpringVersion::GetFull().c_str());
+#ifdef USE_GML
+	PRINT("MT with %d threads.", gmlThreadCount);
+#endif
+
+	PrepareStacktrace();
+
+	const std::string error(ExceptionName(e->ExceptionRecord->ExceptionCode));
+
+	// Print exception info.
+	PRINT("Exception: %s (0x%08lx)", error.c_str(), e->ExceptionRecord->ExceptionCode);
+	PRINT("Exception Address: 0x%08lx", (unsigned long int) (PVOID) e->ExceptionRecord->ExceptionAddress);
+
+	// Print stacktrace.
+	Stacktrace(NULL, e);
+
+	CleanupStacktrace();
+
+	// Only the first crash is of any real interest
+	CrashHandler::Remove();
 
 	// Inform user.
-	char dir[MAX_PATH], msg[MAX_PATH+200];
-	GetCurrentDirectory(sizeof(dir) - 1, dir);
-	SNPRINTF(msg, sizeof(msg),
-		"Spring has crashed.\n\n"
-		"A stacktrace has been written to:\n"
-		"%s\\infolog.txt", dir);
-	MessageBox(NULL, msg, "Spring: Unhandled exception", 0);
+	std::ostringstream buf;
+	buf << "Spring has crashed:\n"
+		<< "  " << error << ".\n\n"
+		<< "A stacktrace has been written to:\n"
+		<< "  " << logOutput.GetFilePath();
+	ErrorMessageBox(buf.str(), "Spring: Unhandled exception", MBF_OK | MBF_CRASH); //calls exit()!
 
 	// this seems to silently close the application
 	return EXCEPTION_EXECUTE_HANDLER;
@@ -272,101 +371,6 @@ static LONG CALLBACK ExceptionHandler(LPEXCEPTION_POINTERS e)
 	//return EXCEPTION_CONTINUE_EXECUTION;
 }
 
-/** Print stack traces for relevant threads. */
-void HangHandler(bool simhang)
-{
-	PRINT("Hang detection triggered %sfor Spring %s.", simhang ? "(in sim thread) " : "", SpringVersion::GetFull().c_str());
-#ifdef USE_GML
-	PRINT("MT with %d threads.", gmlThreadCount);
-#endif
-
-	initImageHlpDll();
-
-	// Record list of loaded DLLs.
-	PRINT("DLL information:");
-	SymEnumerateModules(GetCurrentProcess(), EnumModules, NULL);
-
-
-	if (drawthread != INVALID_HANDLE_VALUE) {
-		// Record stacktrace.
-		PRINT("Stacktrace:");
-		Stacktrace(NULL, drawthread);
-	}
-
-#if defined(USE_GML) && GML_ENABLE_SIM
-	if (gmlMultiThreadSim && simthread != INVALID_HANDLE_VALUE) {
-		PRINT("Stacktrace (sim):");
-		Stacktrace(NULL, simthread);
-	}
-#endif
-
-	// Unintialize IMAGEHLP.DLL
-	SymCleanup(GetCurrentProcess());
-
-	logOutput.Flush();
-}
-
-void HangDetector() {
-	while (keepRunning) {
-		// increase multiplier during game load to prevent false positives e.g. during pathing
-		const int hangTimeMultiplier = CrashHandler::gameLoading ? 3 : 1;
-		const spring_time hangtimeout = spring_msecs(spring_tomsecs(hangTimeout) * hangTimeMultiplier);
-
-		spring_time curwdt = spring_gettime();
-#if defined(USE_GML) && GML_ENABLE_SIM
-		if (gmlMultiThreadSim) {
-			spring_time cursimwdt = simwdt;
-			if (spring_istime(cursimwdt) && curwdt > cursimwdt && (curwdt - cursimwdt) > hangtimeout) {
-				HangHandler(true);
-				simwdt = curwdt;
-			}
-		}
-#endif
-		spring_time curdrawwdt = drawwdt;
-		if (spring_istime(curdrawwdt) && curwdt > curdrawwdt && (curwdt - curdrawwdt) > hangtimeout) {
-			HangHandler(false);
-			drawwdt = curwdt;
-		}
-
-		spring_sleep(spring_secs(1));
-	}
-}
-
-
-void InstallHangHandler() {
-	DuplicateHandle(GetCurrentProcess(), GetCurrentThread(), GetCurrentProcess(),
-					&drawthread, 0, TRUE, DUPLICATE_SAME_ACCESS);
-	int hangTimeoutSecs = configHandler->Get("HangTimeout", 0);
-	CrashHandler::gameLoading = false;
-	// HangTimeout = -1 to force disable hang detection
-	if (hangTimeoutSecs >= 0) {
-		if (hangTimeoutSecs == 0)
-			hangTimeoutSecs = 10;
-		hangTimeout = spring_secs(hangTimeoutSecs);
-		hangdetectorthread = new boost::thread(&HangDetector);
-	}
-	InitializeSEH();
-}
-
-void ClearDrawWDT(bool disable) { drawwdt = disable ? spring_notime : spring_gettime(); }
-void ClearSimWDT(bool disable) { simwdt = disable ? spring_notime : spring_gettime(); }
-
-void GameLoading(bool loading) { CrashHandler::gameLoading = loading; }
-
-void UninstallHangHandler()
-{
-	if (hangdetectorthread) {
-		keepRunning = 0;
-		hangdetectorthread->join();
-		delete hangdetectorthread;
-	}
-	if (drawthread != INVALID_HANDLE_VALUE) {
-		CloseHandle(drawthread);
-	}
-	if (simthread != INVALID_HANDLE_VALUE) {
-		CloseHandle(simthread);
-	}
-}
 
 /** Install crash handler. */
 void Install()
@@ -375,6 +379,7 @@ void Install()
 	signal(SIGABRT, SigAbrtHandler);
 }
 
+
 /** Uninstall crash handler. */
 void Remove()
 {
