@@ -4,11 +4,12 @@
 
 #include <SDL.h>
 
-#include "FileSystem/FileHandler.h"
-#include "LogOutput.h"
+#include "System/FileSystem/FileHandler.h"
+#include "SoundLog.h"
 #include "ALShared.h"
 #include "VorbisShared.h"
 
+
 namespace VorbisCallbacks {
 	size_t VorbisStreamRead(void* ptr, size_t size, size_t nmemb, void* datasource)
 	{
@@ -68,9 +69,7 @@ COggStream::COggStream(ALuint _source)
 
 COggStream::~COggStream()
 {
-	if (!stopped) {
-		ReleaseBuffers();
-	}
+	Stop();
 }
 
 // open an Ogg stream from a given file and start playing it
@@ -82,7 +81,6 @@ void COggStream::Play(const std::string& path, float volume)
 	}
 
 	vorbisTags.clear();
-	int result = 0;
 
 	ov_callbacks vorbisCallbacks;
 		vorbisCallbacks.read_func  = VorbisCallbacks::VorbisStreamRead;
@@ -91,8 +89,10 @@ void COggStream::Play(const std::string& path, float volume)
 		vorbisCallbacks.tell_func  = VorbisCallbacks::VorbisStreamTell;
 
 	CFileHandler* buf = new CFileHandler(path);
-	if ((result = ov_open_callbacks(buf, &oggStream, NULL, 0, vorbisCallbacks)) < 0) {
-		logOutput.Print("Could not open Ogg stream (reason: %s).", ErrorString(result).c_str());
+	const int result = ov_open_callbacks(buf, &oggStream, NULL, 0, vorbisCallbacks);
+	if (result < 0) {
+		LOG_L(L_WARNING, "Could not open Ogg stream (reason: %s).",
+				ErrorString(result).c_str());
 		return;
 	}
 
@@ -117,7 +117,7 @@ void COggStream::Play(const std::string& path, float volume)
 		format = AL_FORMAT_STEREO16;
 	}
 
-	alGenBuffers(2, buffers); CheckError("COggStream");
+	alGenBuffers(2, buffers); CheckError("COggStream::Play");
 
 	if (!StartPlaying()) {
 		ReleaseBuffers();
@@ -126,7 +126,7 @@ void COggStream::Play(const std::string& path, float volume)
 		paused = false;
 	}
 
-	CheckError("COggStream");
+	CheckError("COggStream::Play");
 }
 
 float COggStream::GetPlayTime() const
@@ -139,31 +139,36 @@ float COggStream::GetTotalTime()
 	return ov_time_total(&oggStream, -1);
 }
 
-const COggStream::TagVector& COggStream::VorbisTags() const
-{
-	return vorbisTags;
-}
-
 bool COggStream::Valid() const
 {
 	return (vorbisInfo != 0);
 }
 
+bool COggStream::IsFinished()
+{
+	return !Valid() || (GetPlayTime() >= GetTotalTime());
+}
+
+const COggStream::TagVector& COggStream::VorbisTags() const
+{
+	return vorbisTags;
+}
+
 // display Ogg info and comments
 void COggStream::DisplayInfo()
 {
-	logOutput.Print("version:           %d", vorbisInfo->version);
-	logOutput.Print("channels:          %d", vorbisInfo->channels);
-	logOutput.Print("time (sec):        %lf", ov_time_total(&oggStream,-1));
-	logOutput.Print("rate (Hz):         %ld", vorbisInfo->rate);
-	logOutput.Print("bitrate (upper):   %ld", vorbisInfo->bitrate_upper);
-	logOutput.Print("bitrate (nominal): %ld", vorbisInfo->bitrate_nominal);
-	logOutput.Print("bitrate (lower):   %ld", vorbisInfo->bitrate_lower);
-	logOutput.Print("bitrate (window):  %ld", vorbisInfo->bitrate_window);
-	logOutput.Print("vendor:            %s", vendor.c_str());
+	LOG("version:           %d", vorbisInfo->version);
+	LOG("channels:          %d", vorbisInfo->channels);
+	LOG("time (sec):        %lf", ov_time_total(&oggStream,-1));
+	LOG("rate (Hz):         %ld", vorbisInfo->rate);
+	LOG("bitrate (upper):   %ld", vorbisInfo->bitrate_upper);
+	LOG("bitrate (nominal): %ld", vorbisInfo->bitrate_nominal);
+	LOG("bitrate (lower):   %ld", vorbisInfo->bitrate_lower);
+	LOG("bitrate (window):  %ld", vorbisInfo->bitrate_window);
+	LOG("vendor:            %s", vendor.c_str());
 
 	for (TagVector::const_iterator it = vorbisTags.begin(); it != vorbisTags.end(); ++it) {
-		logOutput.Print("%s", it->c_str());
+		LOG("%s", it->c_str());
 	}
 }
 
@@ -177,7 +182,7 @@ void COggStream::ReleaseBuffers()
 	EmptyBuffers();
 
 	alDeleteBuffers(2, buffers);
-	CheckError("COggStream");
+	CheckError("COggStream::ReleaseBuffers");
 
 	ov_clear(&oggStream);
 }
@@ -193,8 +198,8 @@ bool COggStream::StartPlaying()
 	if (!DecodeStream(buffers[0])) { return false; }
 	if (!DecodeStream(buffers[1])) { return false; }
 
-	alSourceQueueBuffers(source, 2, buffers); CheckError("COggStream");
-	alSourcePlay(source); CheckError("COggStream");
+	alSourceQueueBuffers(source, 2, buffers); CheckError("COggStream::StartPlaying");
+	alSourcePlay(source); CheckError("COggStream::StartPlaying");
 
 	return true;
 }
@@ -243,12 +248,12 @@ bool COggStream::UpdateBuffers()
 
 	while (buffersProcessed-- > 0) {
 		ALuint buffer;
-		alSourceUnqueueBuffers(source, 1, &buffer); CheckError("COggStream");
+		alSourceUnqueueBuffers(source, 1, &buffer); CheckError("COggStream::UpdateBuffers");
 
 		// false if we've reached end of stream
 		active = DecodeStream(buffer);
 		if (active)
-			alSourceQueueBuffers(source, 1, &buffer); CheckError("COggStream");
+			alSourceQueueBuffers(source, 1, &buffer); CheckError("COggStream::UpdateBuffers");
 	}
 
 	return active;
@@ -302,7 +307,8 @@ bool COggStream::DecodeStream(ALuint buffer)
 			size += result;
 		} else {
 			if (result < 0) {
-				logOutput.Print("Error reading Ogg stream (%s)", ErrorString(result).c_str());
+				LOG_L(L_WARNING, "Error reading Ogg stream (%s)",
+						ErrorString(result).c_str());
 			} else {
 				break;
 			}
@@ -314,7 +320,7 @@ bool COggStream::DecodeStream(ALuint buffer)
 	}
 
 	alBufferData(buffer, format, pcm, size, vorbisInfo->rate);
-	CheckError("COggStream");
+	CheckError("COggStream::DecodeStream");
 
 	return true;
 }
@@ -324,10 +330,10 @@ bool COggStream::DecodeStream(ALuint buffer)
 void COggStream::EmptyBuffers()
 {
 	int queuedBuffers = 0;
-	alGetSourcei(source, AL_BUFFERS_QUEUED, &queuedBuffers); CheckError("COggStream");
+	alGetSourcei(source, AL_BUFFERS_QUEUED, &queuedBuffers); CheckError("COggStream::EmptyBuffers");
 
 	while (queuedBuffers-- > 0) {
 		ALuint buffer;
-		alSourceUnqueueBuffers(source, 1, &buffer); CheckError("COggStream");
+		alSourceUnqueueBuffers(source, 1, &buffer); CheckError("COggStream::EmptyBuffers");
 	}
 }
