@@ -5,12 +5,10 @@
 
 #include <cassert>
 
-#include "BranchPrediction.h"
+#include "System/BranchPrediction.h"
 #include "lib/streflop/streflop_cond.h"
-#include "creg/creg_cond.h"
-#include "ExternalAI/Interface/SAIFloat3.h"
-#include "FastMath.h"
-
+#include "System/creg/creg_cond.h"
+#include "System/FastMath.h"
 
 
 /**
@@ -24,9 +22,9 @@ class float3
 {
 public:
 	CR_DECLARE_STRUCT(float3);
-/*	inline void* operator new(size_t size){return mempool.Alloc(size);};
-	inline void* operator new(size_t n, void *p){return p;}; //cp visual
-	inline void operator delete(void* p,size_t size){mempool.Free(p,size);};
+/*	void* operator new(size_t size) { return mempool.Alloc(size); }
+	void* operator new(size_t n, void* p) { return p; } // cp visual
+	void operator delete(void* p, size_t size) { mempool.Free(p, size); }
 */
 
 #ifdef _MSC_VER
@@ -43,7 +41,7 @@ public:
 	 *
 	 * With no parameters, x/y/z are just initialized to 0.
 	 */
-	inline float3() : x(0), y(0), z(0) {};
+	float3() : x(0.0f), y(0.0f), z(0.0f) {}
 
 	/**
 	 * @brief Constructor
@@ -53,38 +51,16 @@ public:
 	 *
 	 * With parameters, initializes x/y/z to the given floats.
 	 */
-	inline float3(const float x,const float y,const float z) : x(x),y(y),z(z) {};
+	float3(const float x,const float y,const float z)
+			: x(x), y(y), z(z) {}
 
 	/**
-	 * @brief Constructor
-	 * @param sAIFloat3 SAIFLoat3 sAIFLoat3
+	 * @brief float[3] Constructor
+	 * @param f float[3] to assign
 	 *
-	 * With parameters, initializes x/y/z to the given SAIFLoat3.
+	 * With parameters, initializes x/y/z to the given float[3].
 	 */
-	inline float3(const SAIFloat3& sAIFloat3) : x(sAIFloat3.x), y(sAIFloat3.y), z(sAIFloat3.z) {};
-
-	/**
-	 * @brief operator =
-	 * @param sAIFloat3 to copy the values from
-	 * @return const float3
-	 *
-	 * Simple default assignment operator
-	 */
-	inline float3& operator= (const SAIFloat3& sAIFloat3) {
-
-		x = sAIFloat3.x;
-		y = sAIFloat3.y;
-		z = sAIFloat3.z;
-
-		return *this;
-	}
-
-	/**
-	 * @brief Destructor
-	 *
-	 * Does nothing
-	 */
-	inline ~float3() {}
+	float3(const float f[3]) : x(f[0]), y(f[1]), z(f[2]) {}
 
 	/**
 	 * @brief operator =
@@ -92,7 +68,8 @@ public:
 	 *
 	 * Sets the float3 to the given float[3].
 	 */
-	inline float3& operator= (const float f[3]) {
+	float3& operator= (const float f[3]) {
+
 		x = f[0];
 		y = f[1];
 		z = f[2];
@@ -100,6 +77,19 @@ public:
 		return *this;
 	}
 
+	/**
+	 * @brief Copy x, y, z into float[3]
+	 * @param f float[3] to copy values into
+	 *
+	 * Sets the float[3] to this float3.
+	 */
+	void copyInto(float f[3]) const {
+
+		f[0] = x;
+		f[1] = y;
+		f[2] = z;
+	}
+
 
 	/**
 	 * @brief operator +
@@ -110,7 +100,7 @@ public:
 	 * calculate the sum of the positions in
 	 * space (adds the x/y/z components individually)
 	 */
-	inline float3 operator+ (const float3 &f) const {
+	float3 operator+ (const float3& f) const {
 		return float3(x+f.x, y+f.y, z+f.z);
 	}
 
@@ -122,7 +112,7 @@ public:
 	 * When adding just a float, the point is
 	 * increased in all directions by that float.
 	 */
-	inline float3 operator+ (const float f) const {
+	float3 operator+ (const float f) const {
 		return float3(x+f, y+f, z+f);
 	}
 
@@ -133,7 +123,7 @@ public:
 	 * Just like adding a float3, but updates this
 	 * float with the new sum.
 	 */
-	inline void operator+= (const float3 &f) {
+	void operator+= (const float3& f) {
 
 		x += f.x;
 		y += f.y;
@@ -148,7 +138,7 @@ public:
 	 * Decreases the float3 by another float3,
 	 * subtracting each x/y/z component individually.
 	 */
-	inline float3 operator- (const float3 &f) const {
+	float3 operator- (const float3& f) const {
 		return float3(x-f.x, y-f.y, z-f.z);
 	}
 
@@ -159,7 +149,7 @@ public:
 	 * When negating the float3, inverts all three
 	 * x/y/z components.
 	 */
-	inline float3 operator- () const {
+	float3 operator- () const {
 		return float3(-x, -y, -z);
 	}
 
@@ -171,7 +161,7 @@ public:
 	 * When subtracting a single fixed float,
 	 * decreases all three x/y/z components by that amount.
 	 */
-	inline float3 operator- (const float f) const {
+	float3 operator- (const float f) const {
 		return float3(x-f, y-f, z-f);
 	}
 
@@ -182,7 +172,7 @@ public:
 	 * Same as subtracting a float3, but stores
 	 * the new float3 inside this one.
 	 */
-	inline void operator-= (const float3 &f) {
+	void operator-= (const float3& f) {
 
 		x -= f.x;
 		y -= f.y;
@@ -197,7 +187,7 @@ public:
 	 * When multiplying by another float3,
 	 * multiplies each x/y/z component individually.
 	 */
-	inline float3 operator* (const float3 &f) const {
+	float3 operator* (const float3& f) const {
 		return float3(x*f.x, y*f.y, z*f.z);
 	}
 
@@ -209,7 +199,7 @@ public:
 	 * When multiplying by a single float, multiplies
 	 * each x/y/z component by that float.
 	 */
-	inline float3 operator* (const float f) const {
+	float3 operator* (const float f) const {
 		return float3(x*f, y*f, z*f);
 	}
 
@@ -220,7 +210,7 @@ public:
 	 * Same as multiplying a float3, but stores
 	 * the new float3 inside this one.
 	 */
-	inline void operator*= (const float3 &f) {
+	void operator*= (const float3& f) {
 		x *= f.x;
 		y *= f.y;
 		z *= f.z;
@@ -233,7 +223,7 @@ public:
 	 * Same as multiplying a float, but stores
 	 * the new float3 inside this one.
 	 */
-	inline void operator*= (const float f) {
+	void operator*= (const float f) {
 		x *= f;
 		y *= f;
 		z *= f;
@@ -247,7 +237,7 @@ public:
 	 * When dividing by a float3, divides
 	 * each x/y/z component individually.
 	 */
-	inline float3 operator/ (const float3 &f) const {
+	float3 operator/ (const float3& f) const {
 		return float3(x/f.x, y/f.y, z/f.z);
 	}
 
@@ -259,10 +249,10 @@ public:
 	 * When dividing by a single float, divides
 	 * each x/y/z component by that float.
 	 */
-	inline float3 operator/ (const float f) const {
+	float3 operator/ (const float f) const {
 
-		const float inv = (float) 1.f / f;
-		return float3(x*inv, y*inv, z*inv);
+		const float inv = (float) 1.0f / f;
+		return *this * inv;
 	}
 
 	/**
@@ -272,7 +262,7 @@ public:
 	 * Same as dividing by a float3, but stores
 	 * the new values inside this float3.
 	 */
-	inline void operator/= (const float3 &f) {
+	void operator/= (const float3& f) {
 
 		x /= f.x;
 		y /= f.y;
@@ -286,12 +276,10 @@ public:
 	 * Same as dividing by a single float, but stores
 	 * the new values inside this float3.
 	 */
-	inline void operator/= (const float f) {
+	void operator/= (const float f) {
 
 		const float inv = (float) 1.f / f;
-		x *= inv;
-		y *= inv;
-		z *= inv;
+		*this *= inv;
 	}
 
 	/**
@@ -302,10 +290,10 @@ public:
 	 * Tests if this float3 is equal to another, by
 	 * checking each x/y/z component individually.
 	 */
-	inline bool operator== (const float3 &f) const {
-		return math::fabs(x-f.x) <= math::fabs(CMP_EPS*x)
-			&& math::fabs(y-f.y) <= math::fabs(CMP_EPS*y)
-			&& math::fabs(z-f.z) <= math::fabs(CMP_EPS*z);
+	bool operator== (const float3& f) const {
+		return math::fabs(x - f.x) <= math::fabs(CMP_EPS * x)
+			&& math::fabs(y - f.y) <= math::fabs(CMP_EPS * y)
+			&& math::fabs(z - f.z) <= math::fabs(CMP_EPS * z);
 	}
 
 	/**
@@ -316,7 +304,7 @@ public:
 	 * Tests if this float3 is not equal to another, by
 	 * checking each x/y/z component individually.
 	 */
-	inline bool operator!= (const float3 &f) const {
+	bool operator!= (const float3& f) const {
 		return !(*this == f);
 	}
 
@@ -328,7 +316,7 @@ public:
 	 * Array access for x/y/z components
 	 * (index 0 is x, index 1 is y, index 2 is z)
 	 */
-	inline float& operator[] (const int t) {
+	float& operator[] (const int t) {
 		return (&x)[t];
 	}
 
@@ -340,7 +328,7 @@ public:
 	 * Same as plain [] operator but used in
 	 * a const context
 	 */
-	inline const float& operator[] (const int t) const {
+	const float& operator[] (const int t) const {
 		return (&x)[t];
 	}
 
@@ -353,8 +341,8 @@ public:
 	 * another float3 (sums the products of each
 	 * x/y/z component).
 	 */
-	inline float dot (const float3 &f) const {
-		return x*f.x + y*f.y + z*f.z;
+	float dot (const float3& f) const {
+		return (x * f.x) + (y * f.y) + (z * f.z);
 	}
 
 	/**
@@ -363,12 +351,14 @@ public:
 	 * @return cross product of two float3s
 	 *
 	 * Calculates the cross product of this and
-	 * another float3 (y1*z2-z1*y2,z1*x2-x1*z2,x1*y2-y1*x2)
+	 * another float3:
+	 * (y1*z2 - z1*y2, z1*x2 - x1*z2, x1*y2 - y1*x2)
 	 */
-	inline float3 cross(const float3 &f) const {
-		return float3(	y*f.z - z*f.y,
-						z*f.x - x*f.z,
-						x*f.y - y*f.x  );
+	float3 cross(const float3& f) const {
+		return float3(
+				(y * f.z) - (z * f.y),
+				(z * f.x) - (x * f.z),
+				(x * f.y) - (y * f.x));
 	}
 
 	/**
@@ -380,7 +370,8 @@ public:
 	 * and another float3 (sums the differences in each
 	 * x/y/z component, square root for pythagorean theorem)
 	 */
-	inline float distance(const float3 &f) const {
+	float distance(const float3& f) const {
+
 		const float dx = x - f.x;
 		const float dy = y - f.y;
 		const float dz = z - f.z;
@@ -398,7 +389,8 @@ public:
 	 * differences in the x and z components, square
 	 * root for pythagorean theorem
 	 */
-	inline float distance2D(const float3 &f) const {
+	float distance2D(const float3& f) const {
+
 		const float dx = x - f.x;
 		const float dz = z - f.z;
 		return (float) math::sqrt(dx*dx + dz*dz);
@@ -412,7 +404,7 @@ public:
 	 * (squares and sums each x/y/z component,
 	 * square root for pythagorean theorem)
 	 */
-	inline float Length() const {
+	float Length() const {
 		//assert(x!=0.f || y!=0.f || z!=0.f);
 		return (float) math::sqrt(SqLength());
 	}
@@ -425,7 +417,7 @@ public:
 	 * (squares and sums only the x and z components,
 	 * square root for pythagorean theorem)
 	 */
-	inline float Length2D() const {
+	float Length2D() const {
 		//assert(x!=0.f || y!=0.f || z!=0.f);
 		return (float) math::sqrt(SqLength2D());
 	}
@@ -437,7 +429,7 @@ public:
 	 * Normalizes the vector by dividing each
 	 * x/y/z component by the vector's length.
 	 */
-	inline float3& Normalize() {
+	float3& Normalize() {
 #if defined(__SUPPORT_SNAN__) && !defined(USE_GML)
 		assert(SqLength() > NORMALIZE_EPS);
 		return UnsafeNormalize();
@@ -453,7 +445,7 @@ public:
 	 * Normalizes the vector by dividing each
 	 * x/y/z component by the vector's length.
 	 */
-	inline float3& UnsafeNormalize() {
+	float3& UnsafeNormalize() {
 		*this *= math::isqrt(SqLength());
 		return *this;
 	}
@@ -466,10 +458,13 @@ public:
 	 * Normalizes the vector by dividing each
 	 * x/y/z component by the vector's length.
 	 */
-	inline float3& SafeNormalize() {
+	float3& SafeNormalize() {
+
 		const float sql = SqLength();
-		if (likely(sql > NORMALIZE_EPS))
+		if (likely(sql > NORMALIZE_EPS)) {
 			*this *= math::isqrt(sql);
+		}
+
 		return *this;
 	}
 
@@ -481,7 +476,7 @@ public:
 	 * Normalizes the vector by dividing each x/y/z component by
 	 * the vector's approx. length.
 	 */
-	inline float3& ANormalize() {
+	float3& ANormalize() {
 #if defined(__SUPPORT_SNAN__) && !defined(USE_GML)
 		assert(SqLength() > NORMALIZE_EPS);
 		return UnsafeANormalize();
@@ -492,29 +487,34 @@ public:
 
 
 	/**
-	 * @brief normalizes the vector approximately without checking for ZeroVector
+	 * @brief normalizes the vector approximately without checking
+	 *        for ZeroVector
 	 * @return pointer to self
 	 *
 	 * Normalizes the vector by dividing each x/y/z component by
 	 * the vector's approx. length.
 	 */
-	inline float3& UnsafeANormalize() {
+	float3& UnsafeANormalize() {
 		*this *= fastmath::isqrt(SqLength());
 		return *this;
 	}
 
 
 	/**
-	 * @brief normalizes the vector approximately and safely (check for *this == ZeroVector)
+	 * @brief normalizes the vector approximately and safely
 	 * @return pointer to self
 	 *
 	 * Normalizes the vector by dividing each x/y/z component by
-	 * the vector's approx. length.
+	 * the vector's approximate length, if (this != ZeroVector),
+	 * else do nothing.
 	 */
-	inline float3& SafeANormalize() {
+	float3& SafeANormalize() {
+
 		const float sql = SqLength();
-		if (likely(sql > NORMALIZE_EPS))
+		if (likely(sql > NORMALIZE_EPS)) {
 			*this *= fastmath::isqrt(sql);
+		}
+
 		return *this;
 	}
 
@@ -525,7 +525,7 @@ public:
 	 *
 	 * Returns the length of this vector squared.
 	 */
-	inline float SqLength() const {
+	float SqLength() const {
 		return x*x + y*y + z*z;
 	}
 
@@ -536,7 +536,7 @@ public:
 	 * Returns the 2-dimensional length of this
 	 * vector squared.
 	 */
-	inline float SqLength2D() const {
+	float SqLength2D() const {
 		return x*x + z*z;
 	}
 
@@ -548,11 +548,12 @@ public:
 	 *
 	 * Returns the squared distance of 2 float3s
 	 */
-	inline float SqDistance(const float3 &f) const {
+	float SqDistance(const float3& f) const {
+
 		const float dx = x - f.x;
 		const float dy = y - f.y;
 		const float dz = z - f.z;
-		return (float) (dx*dx + dy*dy + dz*dz);
+		return (float)(dx*dx + dy*dy + dz*dz);
 	}
 
 
@@ -563,7 +564,8 @@ public:
 	 *
 	 * Returns the squared 2d-distance of 2 float3s
 	 */
-	inline float SqDistance2D(const float3 &f) const {
+	float SqDistance2D(const float3& f) const {
+
 		const float dx = x - f.x;
 		const float dz = z - f.z;
 		return (float)(dx*dx + dz*dz);
@@ -588,10 +590,10 @@ public:
 	 */
 	static float maxzpos;
 
-	bool IsInBounds() const; //!< Check if this vector is in bounds without clamping x and z
-	bool CheckInBounds(); //!< Check if this vector is in bounds and clamp x and z if not
-
-	SAIFloat3 toSAIFloat3() const;
+	/// Check if this vector is in bounds without clamping x and z
+	bool IsInBounds() const;
+	/// Check if this vector is in bounds and clamp x and z if not
+	bool CheckInBounds();
 
 	float x; ///< x component
 	float y; ///< y component
